

// NOTES IMPORTANTS:
//  - Abans de comprovar cap lectura de corrent: s'ha d'activar el on_logica (5v)
//    i esperar a que alimenti tot el circuit de la placa, amb un delay()

// Resultats proves de pins 2015-02-03
//   Outputs leds: OK
//   Analog Input 5v: OK
//   Analog Input 3.5v: OK
//   Digital input 24v: OK
//   Output on_logica (5v): OK
//   Output on_potencia (24v): OK
//   Temperatura: OK
//   EEPROM: OK

#include <Wire.h>
#include <EEPROM.h>
#include "define.h"
#include "var.h"

void setup(){
  inicialitzarPins();
  inicialitzarVariables();
  delay(500);
  proves();
  while(1){}
  // De moment ho apliquem sempre. Quan definim que es comprova per cada estat switch, ho modifiquarem
  digitalWrite(on_logica, HIGH); // TEMPORAL ELIMINAR
  digitalWrite(on_potencia, LOW); // TEMPORAL ELIMINAR
  //aplicarTotesComprovacions(); // TEMPORAL DESCOMENTAR
  
  /* 
  // Temporal
  pinMode(1, OUTPUT);
  digitalWrite(1, HIGH);
  sistemaCorrecte = false;
  // Fi temporal
  */
}

void loop(){
  if(sistemaCorrecte){
    // El sistema nomes es considera correcte si totes les comprovacions de la FA previes han resultat en exit, i la HMI ha respost inicialment
    if(millisActual < millis()){
      // Han passat 5 minuts. Llegim temperatura
      if(countTempExcessiva < 2){
        if(!temperaturaCorrecte()){
          if(countTempExcessiva+1 == 2){
            setTemperaturaCorrecte(0);
            countTempExcessiva = 2;  
          }
          else{
            countTempExcessiva++;
          }   
        }
      }
      millisActual = millis() + 600000UL;
    }
    if(digitalRead(p_emergencia) == LOW){
      digitalWrite(on_potencia, LOW);
      digitalWrite(on_logica, LOW);
      sistemaCorrecte = false;  
    }
  }
  delay(20); 
}

void inicialitzarPins(){
  pinMode(dip0, INPUT);
  pinMode(dip1, INPUT);
  pinMode(dip2, INPUT);
  pinMode(dip3, INPUT);
  pinMode(dip4, INPUT);
  pinMode(dip5, INPUT);
  pinMode(p_emergencia, INPUT);
  pinMode(temperatura, INPUT);
  pinMode(lector24v, INPUT);
  pinMode(lector5v, INPUT);
  pinMode(lector3_5v, INPUT);
  
  pinMode(led0, OUTPUT);
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);
  pinMode(led3, OUTPUT);
  pinMode(on_logica, OUTPUT);
  pinMode(on_potencia, OUTPUT);
  
  digitalWrite(p_emergencia, HIGH);
}

void inicialitzarVariables(){
  Wire.begin(2);                // join i2c bus with address #2
  Wire.onReceive(receiveEvent); // register event
  Wire.onRequest(requestEvent); // request event
  sistemaCorrecte = false;
  valorTemperatura = 0;
  countTempExcessiva = 0;
  respostaHMI[0] = 233;
}

void activarLeds(byte l0, byte l1, byte l2, byte l3){
  digitalWrite(led0, l0);
  digitalWrite(led1, l1);
  digitalWrite(led2, l2);
  digitalWrite(led3, l3);
}

void aplicarTotesComprovacions(){
  if(digitalRead(p_emergencia) == LOW){
    // Pin d'emergencia activat (logica inversa). No continuem
    activarLeds(1,0,0,0);
  }
  else{
    if(!temperaturaCorrecte()){
      // Temperatura fora de limit. No continuem
      activarLeds(0,1,0,0);  
    }
    else{
      // Pin d'emergencia no activat i temperatura correcte -> Activem logica (corrent a les plaques)
      digitalWrite(on_logica, HIGH);
      millisActual = millis() + 2000UL;
      while(millisActual > millis()){
        // Esperem 4 segons a que la HMI pregunti a la FA
      }
      // Se suposa que la HMI, en cas d'estar viva, ja ha pogut preguntar a la FA
      if(respostaHMI[0] == 233){
        // Si no hem rebut resposta de la HMI (valor 233), no continuem
        activarLeds(0,0,1,0);  
      }
      else if(respostaHMI[0] == 3){
        activarLeds(1,1,1,1);
        delay(1000);
        activarLeds(0,0,0,0);
        // En aqui es quan se suposa que el master (HMI) demana una resposta a l'esclau. Gestionat a RequestEvent
        // Esperem uns segons abans de restaurar de nou la temperatura
        delay(2000);
        if(byteToSend == 0){
          // Indicada la temperatura excesiva en l'execucio anterior, la restaurem com a bona per defecte
          // Si en l'execucio actual (dins el loop) torna a ser excessiva, ja la valorarem com a dolenta altre cop
          setTemperaturaCorrecte(1);
        }
        // Activem potencia (corrent a la resta de la maquina, mes enlla de les plaques)
        digitalWrite(on_potencia, HIGH);
        sistemaCorrecte = true;
        millisActual = millis() + 600000UL;
      }
    }
  }  
}

boolean temperaturaCorrecte(){
  for(byte i=0; i<5; i++){
    valorTemperatura += analogRead(temperatura);
  }
  valorTemperatura /= 5;
  myBytes[1] = valorTemperatura & 0xff;		
  myBytes[0] = (valorTemperatura >> 8) & 0xff;
  // return myBytes[0] + myBytes[1]*256;
  if(valorTemperatura/5 < TEMP_MAX){
    myBytes[2] = 1;
    return true;
  }
  else{
    myBytes[2] = 0;
    return false;
  }
}

byte getTemperaturaCorrecte(){
  return EEPROM.read(0);  
}

void setTemperaturaCorrecte(byte x){
  if(EEPROM.read(0) != x){
    EEPROM.write(0, x);
  }  
}

void receiveEvent(int howMany){
  respostaHMI[0] = Wire.read();  
}

void requestEvent(){
  // Un cop que el master (HMI) ha escrit a l'esclau (FA) que esta viu, li demanara una resposta
  // Es en aquest moment quan aquesta funcio s'activa i nosaltres li responem amb l'estat de la temperatura en l'ultim exercici de la maquina
  Wire.write(myBytes, 3);
  //Wire.write(myBytes[0]);
  //Wire.write(myBytes[1]);
  //Wire.write(myBytes[2]);
}

void proves(){
  /*
  // PROVAR LEDS
  digitalWrite(led0, HIGH);
  delay(1000);
  digitalWrite(led0, LOW);
  digitalWrite(led1, HIGH);
  delay(1000);
  digitalWrite(led1, LOW);
  digitalWrite(led2, HIGH);
  delay(1000);
  digitalWrite(led2, LOW);
  digitalWrite(led3, HIGH);
  delay(1000);
  digitalWrite(led3, LOW);
  */
  
  // PROVAR DIGITAL OUTPUTS: on_logica (5v) i on_potencia (24v)
  digitalWrite(on_logica, HIGH);
  if(digitalRead(on_logica) == HIGH){
    digitalWrite(led2, HIGH);  
  }
  delay(100);
  digitalWrite(on_potencia, HIGH);
  if(digitalRead(on_potencia) == HIGH){
    digitalWrite(led3, HIGH);  
  }
  /*
  // PROVAR DIGITAL INPUTS: parada emergencia i lector 24 volts
  if(digitalRead(p_emergencia) == HIGH){
    digitalWrite(led0, HIGH);  
  }
  
  if(digitalRead(lector24v) == HIGH){ // 
    digitalWrite(led1, HIGH);
 
  */
  /*
  // PROVA ANALOG INPUTS: lector 5v i 3.5v
  // Valors analogs actius: haurien d'oscilar els 500
  digitalWrite(on_logica, HIGH);
  delay(100);
  if(analogRead(lector5v) > 300 && analogRead(lector5v) < 700){
    digitalWrite(led0, HIGH);
  }
  if(analogRead(lector3_5v) > 300 && analogRead(lector3_5v) < 700){
    digitalWrite(led1, HIGH);
  }
  */
  /*
  // PROVA ANALOG TEMPERATURA
  delay(20);
  for(byte i=0; i<10; i++){
    valorTemperatura += analogRead(temperatura);
    delay(50);
  }
  valorTemperatura /= 10;
  // VALORS TEMPERATURA poc acurats
  if(valorTemperatura > 0 && valorTemperatura < 250){
    digitalWrite(led0, HIGH);
  }
  else if(valorTemperatura >= 250 && valorTemperatura < 500){
    digitalWrite(led1, HIGH);
  }
  else if(valorTemperatura >= 500 && valorTemperatura < 750){
    digitalWrite(led2, HIGH);
  }
  else if(valorTemperatura >= 750){
    digitalWrite(led3, HIGH);
  }
  // VALORS TEMPERATURA acurats
  if(valorTemperatura >= 350 && valorTemperatura < 365){
    digitalWrite(led0, HIGH);
  }
  else if(valorTemperatura >= 365 && valorTemperatura < 380){
    digitalWrite(led1, HIGH);
  }
  else if(valorTemperatura >= 380 && valorTemperatura < 395){
    digitalWrite(led2, HIGH);
  }
  else if(valorTemperatura >= 395){
    digitalWrite(led3, HIGH);
  }
  */
  /*
  // VALORS DIP-SWITCH
  digitalWrite(on_logica, HIGH);
  delay(100);
  if(digitalRead(dip5) == HIGH){
    digitalWrite(led0, HIGH);  
  }
  */
  /*
  // VALORS EEPROM
  EEPROM.write(500, 132);
  delay(100);
  if(EEPROM.read(500) == 132){
    digitalWrite(led0, HIGH);  
  }
  */
  // VALOR INPUT EMERGENCIA
  /*
  digitalWrite(on_logica, HIGH);
  delay(100);
  boolean valor = true;
  while(1){
    digitalWrite(led0, digitalRead(p_emergencia));
    delay(500);
  }
  */
}
