#include <EEPROM.h>

#include "define.h"
#include "var.h"


void setup(){
  // CHANGE: s'activa dos cops cada pas per zero. Interrupcio pin canvi de valor.
  // RISING: s'activa un cop cada pas per zero. Interrupcio pin de LOW a HIGH.
  // Per detectar mitjos cicles: millor RISING o FALLING que change
  attachInterrupt(0, canvi, RISING);
  attachInterrupt(1, envolp, CHANGE);
  
  pinMode(xarxa_pas_zero, INPUT);
  pinMode(boto, INPUT);
  digitalWrite(boto, HIGH);
  pinMode(lectorCorrent, INPUT);
  pinMode(tristor, OUTPUT);
  pinMode(rele, OUTPUT);
  pinMode(18, OUTPUT);
  
  inicialitzarVariablesEEPROM();
  // Fins que no detectem un primer RISING, no iterem loop
  pas_ant = pas;
  //digitalWrite(rele, 0);
  //delay(2000);
}

void loop(){
  // Ens interessa executar el programa en sincronitzacio amb l'estat de xarxa.
  // Concretament, volem executar cada mig cicle
  // Per mirar mig cicles, podem mirar el pas de LOW a HIGH, o de HIGH a LOW
  // CHANGE (detectar cada canvi de valor) s'activa 2 cops cada pas per zero, NO DETECTA MITJOS CICLES
  
  if(pas_ant != pas){
    switch(magDesmagEnCurs){
      case(0): // Cap proces de magnititzar o desmagnetitzer en curs. Per tant te sentit llegir botons
        // El pin boto de la placa del magnetitzador, quan s'adapta a una maquina, te un valor que ve donat pel extendedOutput.write(platMagnetic, HIGH)
        // En altres paraules, ja no ve donat per un boto instalat expressament per a la placa magnetitzadora, tal i com en l'exemple original
        if(digitalRead(boto) == HIGH/* && !botoPitjat*/){
          delay(500);
          byte valor = digitalRead(boto);
          if(valor == LOW && !magnetitzat){
            // Inici magnetitzacio. Guardem en quin dels dos nivells de xarxa (inidicat per pas) estem magnetitzant
            // Necessari per a quan toqui desmagnetitzar, ja que consistira en aplicar magnetisme a l'altre nivell de xarxa
            estatMagnetisme = 1; // 1 = Inici proces de magnetitzacio / desmagnetitzacio
            // Activem Relee. Deixem temps per a que hi hagi corrent a tot arreu abans d'activar el transistor que crea magnetisme
            digitalWrite(rele, 1);
            delay(500);
            /*
            if(nivellMagnetitzat == 2){
              // Encara no esta definit. Marquem ara quin valor de pas sera el que magnetitzara.
              // Ho apliquem UNICAMENT un cop, FIXAT PER SEMPRE. Sempre es magnetitzara en aquest canto de la xarxa
              delay(5000);
              nivellMagnetitzat = pas;
              EEPROM.write(1, pas);
            }
            */
            magDesmagEnCurs = 1; // 1 = magnetitzacio en curs
            estatMagnetisme = 0;
            iteracioDesmag = 0; // 0 = cap iteracio perque no estem desmagnetitzant
            indexDelay = 0;
            indexLimit = 0;
            limitMag = valorsLimitMag[indexLimit];  
          }     
          else if(valor == HIGH && magnetitzat){
            // Inici desmagnetitzacio. Desmagnetitzar = parelles de conjunts de magnetitzacions i desmagnetitzacions
            // Primer començem magnetitzant novament (1)
            // Aixi sabem segur que començem la desmagnetitzacio sempre del mateix punt: magnetitzat a tope
            estatMagnetisme = 1; // 1 = Inici proces de magnetitzacio / desmagnetitzacio
            // Activem Relee. Deixem temps per a que hi hagi corrent a tot arreu abans d'activar el transistor que crea magnetisme
            digitalWrite(rele, 1);
            delay(500);
            magDesmagEnCurs = 1;
            estatMagnetisme = 0;
            iteracioDesmag = 1; // primera iteracio del proces de desmagnatitzacio (iteracio = parella <mag,desmag>)
            indexDelay = 1;
            indexLimit = 1;
            limitMag = valorsLimitMag[indexLimit];
          }
        }
        /*
        else if(digitalRead(boto) == HIGH && botoPitjat){
          botoPitjat = false;
        }
        */
      break;
      case 1: // Magnetitzacio en curs (pot ser pel proces de magnetitzar en si, o com a part del proces de desmagnetitzacio)
        if(estatMagnetisme == limitMag){
          // Hem finalitzat una subiteracio de magnetitzacio (ex. limitMag = 80, magnetitzem 80 cops en la iteracio actual)
          if(iteracioDesmag == 0){
            // Estem magnetitzant com a proces en si (iteracioDesmag = 0), no com a part del proces de desmagnetitzacio
            // Hem acabat de magnetitzar. Finalitzem proces a l'espera de nova ordre d'usuari via boto
            magnetitzat = true;
            EEPROM.write(0,1);
            magDesmagEnCurs = 0;
            digitalWrite(rele, 0); // desactivem relee
            delay(500);
          }
          else{
            // Estem magnetitzant com a subproces de la desmagnetitzacio (iteracioDesmag > 0).
            // Hem acabat de magnetitzar. Passem a desmagnetitzar en la mateixa iteracio
            magDesmagEnCurs = 2;
            estatMagnetisme = 0;
            indexLimit++;
            limitMag = valorsLimitMag[indexLimit];
            indexDelay++;
            //delay(3000);  // Pausa per visualitzar valors amb el mesurador de magnetisme
          }
        }
        else{
          // Encara estem dins la subrutina de magnetitzar varies vegades en la iteracio actual
          //if(pas == nivellMagnetitzat){ // podem magnetitzar
          if(iniciar == 0){ // podem magnetitzar
            //delay(5000);
            switchIndex();    
            trenPulsosTristor();
            estatMagnetisme++; // Aplicarem l'activacio de tristor al seguen mig cicle de magnetitzar
          } 
        }
      break;
      case 2: // Desmagnetitzacio en curs
        if(estatMagnetisme == limitMag){
          // Hem finalitzat una subiteracio de magnetitzacio (ex. limitMag = 80, magnetitzem 80 cops en la iteracio actual)
          iteracioDesmag++;
          if(( ITERACIONS_DESMAG > 1 && iteracioDesmag == ITERACIONS_DESMAG ) || ( ITERACIONS_DESMAG == 1 && iteracioDesmag == ITERACIONS_DESMAG+1 )){
            // Hem finalitzat tot el proces de desmagnetitzacio. Esperem nova ordre d'usuari via boto
            magnetitzat = false;
            EEPROM.write(0,0);
            magDesmagEnCurs = 0;
            digitalWrite(rele, 0); // desactivem relee
            delay(500);
          }
          else{
            // Passem a la seguent iteracio de iteracioDesmag (parella <mag/desmag>) comensant per a magnetitzar
            magDesmagEnCurs = 1;
            estatMagnetisme = 0;
            indexLimit++;
            limitMag = valorsLimitMag[indexLimit];
            indexDelay++;
            //delay(3000); // Pausa per visualitzar valors amb el mesurador de magnetisme
          }
        }
        else{
          // Encara estem dins la subrutina de magnetitzar varies vegades en la iteracio actual
          //if(pas != nivellMagnetitzat){ // podem magnetitzar
          if(iniciar == 1){ // podem magnetitzar
            switchIndex();
            trenPulsosTristor();
            estatMagnetisme++; // Aplicarem l'activacio de tristor al seguen mig cicle de magnetitzar
          }
        }
      break;
    }
    pas_ant = pas;
  }
}

void trenPulsosTristor(){
  // Apliquem nova magnetitzacio al mig cicle de xarxa actual
  // Delay previ al activar Tristor (que magnetitza)
  switchIndex();
  // Activem tristor i li apliquem tren de pulsos per a que s'acabi d'activar del tot
  // Si no ho fem: perill de que no s'activi i per tant no magnetitzi
  for(byte i=0; i < 5; i++){
    HIGH_D9 //digitalWrite(tristor, HIGH);            
    delayMicroseconds(5);
    LOW_D9 // digitalWrite(tristor, LOW);           
    delayMicroseconds(5);
  }
}

void switchIndex(){
  switch(indexDelay){
    case 0:
      delayMicroseconds(500);
    break;
    case 1:
      delayMicroseconds(1000);
    break;
    case 2:
      delayMicroseconds(1000);
    break;
    case 3:
      delayMicroseconds(1600);
    break;
    case 4:
      delayMicroseconds(1600);
    break;       
    case 5:
      delayMicroseconds(2000);
    break;              
    case 6:
      delayMicroseconds(2000);
    break;
    case 7:
      delayMicroseconds(2400);
    break;
    case 8:
      delayMicroseconds(2400);
    break;
    case 9:
      delayMicroseconds(2600);
    break;
    case 10:
      delayMicroseconds(2600);
    break;
    case 11:
      delayMicroseconds(3000); // 3000
    break;
    case 12:
      delayMicroseconds(3000); // 3000
    break;
    case 13:
      delayMicroseconds(3100); // 3100
    break;
    case 14:
      delayMicroseconds(3100); // 3100
    break;
    case 15:
      delayMicroseconds(3200); // 3200
    break;
    case 16:
      delayMicroseconds(3200); // 3200
    break;
    case 17:
      delayMicroseconds(3300); // era 3300
    break;
    case 18:
      delayMicroseconds(3300); // era 3300
    break;
    // FINS AQUI AMB ITERACIONS_DESMAG 10 
    /*
    case 19:
      delayMicroseconds(3200);
    break;
    case 20:
      delayMicroseconds(3200);
    break;
    case 21:
      delayMicroseconds(3250);
    break;
    case 22:
      delayMicroseconds(3250);
    break;
    case 23:
      delayMicroseconds(3400);
    break;
    case 24:
      delayMicroseconds(3400);
    break;
    */
    // FINS AQUI AMB ITERACIONS_DESMAG 13
    /*
    case 25:
      delayMicroseconds(3500);
    break;
    case 26:
      delayMicroseconds(3500);
    break;
    case 27:
      delayMicroseconds(3600);
    break;
    case 28:
      delayMicroseconds(3600);
    break;
    */
  }
}

void inicialitzarVariablesEEPROM(){
  // Inicialitzem variable que guarda si esta magnetitzat o no
  byte x = EEPROM.read(0);
  if(x < 0 || x > 1){
    EEPROM.write(0,0);
    //EEPROM.write(1,2); // Si no esta magnetitzat, el nivell de magnetitzat tambe 0 (no nivell)
  }
  magnetitzat = 0;/*EEPROM.read(0);*/
  /*
  // Inicialitzem variable que guarda en quin nivell de xarxa esta magnetitzat o s'estaba magnetitzant
  x = EEPROM.read(1);
  if(x < 0 || x > 2){
    EEPROM.write(1,2);
  }
  */
  //EEPROM.write(1,0);
  nivellMagnetitzat = 0;/*EEPROM.read(1);*/
  
}

void canvi(){ 
  pas = !pas;
  if(count == count_ant){
    iniciar = 1; 
  }
  else{
    iniciar = 0;
  }
  count_ant = count;
  
  /*
  if(pas)
  {
    HIGH_D18
  }
  else
  {
    LOW_D18
  }
  */
}

void envolp(){
  count++;  
}
