
#include <LiquidCrystal.h>
#include <AccelStepper.h>
#include <Rtc_Pcf8563.h>
#include <EEPROM.h>
#include <Wire.h>
#include <MUX16In.h>
#include <OUT16.h>
#include <math.h>

#include "define.h"
#include "var.h"

AccelStepper eixX(1, eixXstep, eixXdirection);
AccelStepper eixZ(1, eixZstep, eixZdirection);
AccelStepper eixA(1, eixAstep, eixAdirection);
AccelStepper eixY(1, eixYstep, eixYdirection);
LiquidCrystal lcd(RS, RW, E1, E2, db3, db4, db6, db10); //LiquidCrystal lcd(25, 24, 26, 22, 6, 7, 8, 9);
Rtc_Pcf8563 rtc;
MUX16In extendedInput(control0, control1, control2, control3, digitalData, analogData);
OUT16 extendedOutput(latchPin, clockPin, dataPin);

void setup(){
  inicialitzarPins();
  inicialitzarInterrupcions();
  inicialitzarVariables();
  inicialitzarVariablesEEPROM();
  inicialitzarMotors();
  printBenvinguda();
}

void loop(){
  if(millis() > 1000UL && millis() < MAX_MILLIS){
    switch(configNivell){
      case 1:                                       // CONFIGNIVELL 1: Arrencada del sistema
        switch(fase){
          case 0:
            sistemaArrencada();
          break;
          case 1:
            menuSecretOmenuPrincipal();
          break;
        }
      break;
      case 2:                                       // CONFIGNIVELL 2: Menu secret
        switch(pantalla){
          case 1:                                   // Pantalla 1 menu secret
            navegacioMenuSecretPantalla1();
          break;
          case 2:                                   // Pantalla 2 menu secret
            navegacioMenuSecretPantalla2();
          break;
          case 3:                                   // Pantalla 3 menu secret
            navegacioMenuSecretPantalla3();
          break;
        }
      break;
      
      case 3:                                       // CONFIGNIVELL 3: Menu inicial
        switch(pantalla){
          case N_PANTALLES_MENUINICIAL:             // Pantalla Configuracions
            navegacioMenuInicialConfiguracions();
          break;
          default:                                  // Pantalles de programes
            navegacioMenuInicialProgrames();
          break;
        }
      break;

      case 4:                                       // CONFIGNIVELL 4: Valors programa d'esmolat
        accionsMagnetisme();
        switch(pantalla){                
          case 1:                                   // Pantalla 1 valors programa d'esmolat
            navegacioProgramaPantalla1();
          break;
          case 2:                                   // Pantalla 2 valors programa d'esmolat
            if(!programaQuartEix()){
              // Programes de sempre
              navegacioProgramaPantalla2();  
            }
            else{
              navegacioProgramaPantalla2b();  
            }
          break;
          case 3:                                   // Pantalla 3 valors programa d'esmolat
            navegacioProgramaPantalla3();
          break;
        }
      break;
      
      case 5:                                       // CONFIGNIVELL 5: Valors de proces
        switch(pantalla){
          case 1:                                   // Pantalla 1 valors de proces
            navegacioValorsProcesPantalla1();
          break;
          case 2:                                   // Pantalla 2 valors de proces
            navegacioValorsProcesPantalla2();
          break;
          case 3:                                   // Panatalla 3 valors de proces
            navegacioValorsProcesPantalla3();
          break;   
        }
      break;
      
      case 6:                                       // CONFIGNIVELL 6: Eliminat de programes
        navegacioEliminatProgrames();
      break;

      case 7:                                       // CONFIGNIVELL 7: Esborrar un programa
        navegacioEliminarUnPrograma();
      break;
      
      case 8:                                       // CONFIGNIVELL 8: Esborrar tots els programes
        navegacioEliminarTotsProgrames();
      break;
      
      case 9:                                       // CONFIGNIVELL 9: Moviments manuals
        aplicarStart(45);
        comprovacionsStepsEixosZYA();
        accionsRotatoriModeManual();
        accionsBotonsModeManual();
      break;
      
      case 10:                                      // CONFIGNIVELL 10: Previ automatic, posicionament per a l'esmolat
        aplicarStart(50);
        comprovacionsStepsEixosZYA();
        if(!programaQuartEix()){
          accionsRotatoriPreviesEsmolatNormal();
        }
        else{
          accionsRotatoriPreviesEsmolatQuartEix();
        }
        checkConfirmarIprintarSalir();
        accionsBotonsPreviesEsmolat();
      break;
      
      case 11:                                      // CONFIGNIVELL 11: Esmolat automatic
        aplicarStart(55);
        navegacioAutomatics();
        switch(movimentsAutomatics){
          case 0:                                   // Esmolat automatic aturat
            accionsEsmolatAturat();
          break;
          case 1:                                   // Esmolat automatic en accio
            accionsEsmolatEnProces();
          break;
        }   
      break;
      
      case 12:                                      // CONFIGNIVELL 12: Confirmar pas a un proxim nivell
        navegacioConfirmarPas();
      break;
      
      case 13:                                      // CONFIGNIVELL 13: Confirmar pas recanvi peces, post Arrancada i pre Menu Inicial
        navegacioAvisRecanvis();
      break;
      
      case 14:                                      // CONFIGNIVELL 14: Mode de proves, des del menu secret
        accionsModeProves();
      break;
      
      case 15:                                      // CONFIGNIVELL 15: Medicio automatica
        accionsIcomprovacionsMedicioAutomatica();
      break;
    }
    if(edit){
      edit = false;
    }
  }
  else if(millis() < 2000 && !edit){
    // reinici de totes les variables associades al millis
    edit = true;
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * SETUP FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */


void inicialitzarPins(){
  // Per la placa vermella
  for(byte t=0;t<16;t++)
  {
    escriureOut16(t, LOW);
  }
  
  pinMode(pitjar, INPUT);
  pinMode(girA, INPUT); // lligat a la Interrupcio 1
  pinMode(girB, INPUT); // lligat a la Interrupcio 2

  pinMode(eixZstep, OUTPUT);
  pinMode(eixZdirection, OUTPUT);
  pinMode(eixXstep, OUTPUT);
  pinMode(eixXdirection, OUTPUT);
  pinMode(eixAstep, OUTPUT);
  pinMode(eixAdirection, OUTPUT);
  pinMode(eixYstep, OUTPUT);
  pinMode(eixYdirection, OUTPUT);
  pinMode(activarRotatori2, OUTPUT);

  digitalWrite(girA, HIGH);
  digitalWrite(girB, HIGH);
  digitalWrite(pitjar, HIGH);
  digitalWrite(activarRotatori2, LOW);
   
  escriureOut16(onGeneral, HIGH); // on general que activem al principi sempre
}

void inicialitzarInterrupcions(){
  attachInterrupt(1, actualitzarEncoderEsquerra, FALLING);
  attachInterrupt(2, actualitzarEncoderDreta, FALLING);
}

void inicialitzarVariables(){
  lcd.begin(40,4);
  valorEncoderNew = 0;
  valorEncoderOld = valorEncoderNew;
  rtc.initClock2(); // El initClock2() es igual, pero no reinicia cap valor
                    //day, weekday, month, century, year
                    //rtc.setDate(15, 0, 9, 0, 14);
                    //hr, min, sec
                    //rtc.setTime(17, 40, 0);
  Wire.begin();     // Si cridem el RTC ja no fem begin(), perque el constructor del rtc JA EL CRIDA
  configNivell = 1;
  firstIn = true;
  fase = 0;
  gestioTemps2 = millis() + 800UL;
  pos = INIT_POS_VARSGENERALS_EEPROM;
  idxVar = idxActivarQuartEix;
  if(getVariable() == 0){
    nProgs = N_PROGS1;
  }
  else{
    nProgs = N_PROGS2;
  }
}

void inicialitzarVariablesEEPROM(){
  // VARIABLES DE PROGRAMA
  for(byte i = 0; i < nProgs; i++){
    pos = i * N_VARS_PROGRAMA;
    for(byte k = 0; k < N_VARS_PROGRAMA; k++){
      idxVar = k*PROPS_VAR;
      /*
      if(i==0){
        esborrarPantalla();
        lcd.print(vars[idxVar]);
      }
      */
      if(esIntDePrograma()){
        // Valors int (2 bytes) de variables Programa
        consistenciaValorInt();
        k++;
      }
      else{
        // Valors d'1 byte
        consistenciaValorByte();
      }
      /*
      if(i==0){
        posCursor(0,1);
        lcd.print(vars[idxVar]);
      }
      delay(5000);
      */
    } 
  }
  // VARIABLES COMUNES
  pos = INIT_POS_VARSGENERALS_EEPROM;
  for(byte j = N_VARS_PROGRAMA; j <= N_INDEX_VARS; j++){
    idxVar = j*PROPS_VAR;
    if(idxVar == idxRecorregutZ_low || idxVar == idxRecorregutY_low){
       consistenciaValorLong();
       j+=2;
    }  
    else if(idxVar == idxIncrementVel_low || idxVar == idxVelocitatZ_low || idxVar == idxAcc_low || idxVar == idxDacc_low || idxVar == idxSensibilitat_low || idxVar == idxCompensacio_low || idxVar == idxHoresTotals_low || idxVar == idxHoresCicle_low || idxVar == idxHoresRecanvis_low){
      // Valors int (2 bytes) de variables menu secret
      consistenciaValorInt();
      j++;
    }
    else if(idxVar == idxOffsetA){
      // Valors amb signe
      consistenciaValorUint8();
    }
    else{
      // Valors d'1 byte
      consistenciaValorByte();
    }
  }
  stepsEixXmanual = 40 + (vars[idxIncrementVel_low]+vars[idxIncrementVel_high]*256) * (vars[idxVelManualX]-1);
  if(vars[idxCodiFinalIntroduit] == 0){
    if(vars[idxHoresCicle_low] + vars[idxHoresCicle_high]*256 >= vars[idxHoresCicle_high+1]+vars[idxHoresCicle_high+2]*256){
      bloqueigMaquina = true; // Per defecte previament esta a false sempre
    }
  }
  velMola = vars[idxVelocitatZ_low] + vars[idxVelocitatZ_high]*256;
  if(vars[idxActivarMedicio] == 0){
    reiniciarValorsMedicio();   
    pos = INIT_POS_VARSGENERALS_EEPROM;
  }
}

void inicialitzarMotors(){
  eixX.setMaxSpeed(5000);
  eixX.setSpeed(1000);
  eixX.setAcceleration(6000);
  eixX.setCurrentPosition(0);
  eixX.setMinPulseWidth(5);
  eixZ.setMaxSpeed(5000);
  eixZ.setAcceleration(2000);
  eixZ.setCurrentPosition(0);
  eixZ.setMinPulseWidth(5);
  eixA.setMaxSpeed(4000);
  eixA.setAcceleration(2000);
  eixA.setCurrentPosition(0);
  eixA.setMinPulseWidth(5);
  eixY.setMaxSpeed(4000);
  eixY.setAcceleration(2000);
  eixY.setCurrentPosition(0);
  eixY.setMinPulseWidth(5);
}


/*
 ************************************************************************************
 ************************************************************************************
 * LOOP FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 
// CONFIGNIVELL 1: Arrencada del sistema

void sistemaArrencada(){
  aplicarStart(0);
  gestioPunts(); // Permet anar a la seguent fase
  checkComFA();
  finalitzarCom();  
}

void menuSecretOmenuPrincipal(){
  encoderPitjar(0);
  rightButton(0);
  leftButton(4);
  menuSecret();
}

void navegacioMenuSecretPantalla1(){
  aplicarStart(1);
  switch(filera){
    case 0:
      encoderPitjar(34);         
      encoderDreta(66);             
      encoderEsquerra(255);              // Sense efectes. Nomes actualitzem valor encoder
    break;
    case 1:
      encoderPitjar(20);            
      encoderDreta(107);            
      encoderEsquerra(3);        
    break;
    case 2:
      switch(mode){
        case 0:
          encoderPitjar(49);       
          encoderDreta(140);        
          encoderEsquerra(66);    
        break;
        case 1:
          encoderPitjar(74);       
          encoderDreta(164);          
          encoderEsquerra(164);    
        break;
      }
    break;
    case 3:
      switch(columna){
        case 20:
          switch(mode){
            case 0:
              encoderPitjar(22);      
              encoderDreta(156);     
              encoderEsquerra(107);
            break;
            case 1:
              encoderPitjar(74);     
              encoderDreta(164);     
              encoderEsquerra(164);  
            break;  
          }
        break;
        case 36:
          switch(mode){
            case 0:
              encoderPitjar(23);      
              encoderDreta(169);       
              encoderEsquerra(140);   
            break;
            case 1:
              encoderPitjar(74);     
              encoderDreta(164);       
              encoderEsquerra(164);   
            break;
          }
        break;  
      }
    break;
  }  
}

void navegacioMenuSecretPantalla2(){
  aplicarStart(2);
  switch(filera){
    case 0:
      encoderPitjar(35);           
      encoderDreta(66);      
      encoderEsquerra(169);          
    break;
    case 1:
      encoderPitjar(255);    
      encoderDreta(106);      
      encoderEsquerra(25);            
    break;
    case 2:
      switch(mode){
        case 0:
          encoderPitjar(44);
          encoderDreta(145);
          encoderEsquerra(66);
        break;
        case 1:
          encoderPitjar(84);
          encoderDreta(171);
          encoderEsquerra(172);
        break;
      }
    break;
    case 3:
      encoderPitjar(46);
      encoderDreta(169);
      encoderEsquerra(106);
    break;
  }
}

void navegacioMenuSecretPantalla3(){
  aplicarStart(3);
  switch(filera){
    case 0:
      switch(mode){
        case 0:
          encoderPitjar(47);
          encoderDreta(66);
          encoderEsquerra(170);
        break;
        case 1:
          encoderPitjar(84);
          encoderDreta(171);
          encoderEsquerra(172);
        break;
      }
    break;
    case 1:
      switch(mode){
        case 0:
          encoderPitjar(85);
          encoderDreta(84);
          encoderEsquerra(26);
        break;
        case 1:
          encoderPitjar(83);
          encoderDreta(164);
          encoderEsquerra(164);
        break;  
      }
    break;
    case 2:
      encoderPitjar(1);
      encoderDreta(255);
      encoderEsquerra(66);
    break; 
  }
}

void navegacioMenuInicialProgrames(){
  aplicarStart(10);
  encoderPitjar(2);
  switch(filera){
    case 0:                            
      encoderDreta(160);
      if(pantalla == 1){
        encoderEsquerra(255);
      }
      else{
        encoderEsquerra(161);
      }           
    break;    
    case 1:
      encoderDreta(160);          
      encoderEsquerra(160);      
    break;
    case 2:
      encoderDreta(160);         
      encoderEsquerra(160);        
    break;
    case 3:
      if(pantalla != 2){
        encoderDreta(161);        
      }
      else{
        encoderDreta(170);         
      }              
      encoderEsquerra(160);         
    break;
  }
}

void navegacioMenuInicialConfiguracions(){
  aplicarStart(10);
  switch(filera){
    case 0:
      encoderPitjar(3);           
      encoderDreta(160);           
      encoderEsquerra(171);       
    break;
    case 1:
      encoderPitjar(4);     
      encoderDreta(160);             
      encoderEsquerra(160);         
    break;
    case 2:
      encoderPitjar(5);           
      encoderDreta(255);              
      encoderEsquerra(160);    
    break;
  }  
}

void accionsMagnetisme(){
  if(vars[idxActivarMagnetisme] == 1){
    // Si la maquina incorpora barra magnetica, llegim el pulsador esquerre
    // El pulsador esquerre magnetitza / desmagnetitza la barra
    // desactivarPlatMagnetic() s'activa si el pulsador esquerre desmagnetitza, i s'ha pitjat durant 2 segons
    leftButton(3);
    desactivarPlatMagnetic();
  }
}

void navegacioProgramaPantalla1(){
  aplicarStart(20);
  switch(filera){
    case 0:                           
      switch(columna){
        case 14:                  
          switch(mode){
            case 0:
              encoderPitjar(51);
              encoderDreta(18);  
              encoderEsquerra(255);
            break;
            case 1:
              encoderPitjar(24);     
              encoderDreta(163);        
              encoderEsquerra(163);    
            break;
          }
        break;
        case 18:                     
          switch(mode){
            case 0:
              encoderPitjar(52);            
              encoderDreta(26);     
              encoderEsquerra(14);  
            break;
            case 1:
              encoderPitjar(25);    
              encoderDreta(164);      
              encoderEsquerra(164);
            break;    
          }
        break;
        case 26:                  
          switch(mode){
            case 0:
              encoderPitjar(53);
              encoderDreta(38);    
              encoderEsquerra(18);  
            break;
            case 1:
              encoderPitjar(54); 
              encoderDreta(164);       
              encoderEsquerra(164);     
            break;
          }
        break;
        case 38:             
          switch(mode){
            case 0:
              encoderPitjar(55);
              encoderDreta(58);   
              encoderEsquerra(26);   
            break;
            case 1:
              encoderPitjar(10);   
              encoderDreta(163);      
              encoderEsquerra(163);  
            break;
          }
        break; 
      } 
    break;
    case 1:
      switch(columna){
        case 18:               
          switch(mode){
            case 0:
              encoderPitjar(56);
              encoderDreta(66);     
              encoderEsquerra(38);   
            break;
            case 1:
              encoderPitjar(27);   
              encoderDreta(164);     
              encoderEsquerra(164);  
            break;
          }
        break;
        case 26:                    
          switch(mode){
            case 0:
              encoderPitjar(57);
              encoderDreta(78);     
              encoderEsquerra(58); 
            break;
            case 1:
              encoderPitjar(58);   
              encoderDreta(163);    
              encoderEsquerra(163);  
            break;
          }
        break;
        case 38:               
          switch(mode){
            case 0:
              encoderPitjar(59);            
              encoderDreta(94);  
              encoderEsquerra(66); 
            break;
            case 1:                     
              encoderPitjar(10);
              encoderDreta(163);   
              encoderEsquerra(163); 
            break;
          }
        break; 
      }
    break;
    case 2:                              
      switch(columna){
        case 14:                          
          switch(mode){
            case 0:
              encoderPitjar(60);            
              encoderDreta(118);  
              encoderEsquerra(78); 
            break;
            case 1:                     
              encoderPitjar(10);  
              encoderDreta(163);    
              encoderEsquerra(163);
            break;
          }
        break;
      case 38:                  
        switch(mode){
          case 0:
            encoderPitjar(61);          
            encoderDreta(159);   
            encoderEsquerra(94); 
          break;
          case 1:              
            encoderPitjar(10);  
            encoderDreta(163);  
            encoderEsquerra(163); 
          break;
        }
        break;
      }
    break;
    case 3:                             
      switch(columna){
        case 39:                     
          encoderPitjar(15);      
          encoderDreta(255);         
          encoderEsquerra(118);   
        break;    
      }
    break;
  }  
}

void navegacioProgramaPantalla2(){
  aplicarStart(21);
  switch(filera){
    case 0:                               
      switch(columna){
        case 14:                       
          switch(mode){
            case 0:                     
              encoderPitjar(62);  
              encoderDreta(16);  
              encoderEsquerra(120); 
            break;
            case 1:                   
              encoderPitjar(37);
              encoderDreta(163); 
              encoderEsquerra(163);
            break;
          }
        break;
        case 16:
          switch(mode){
            case 0:                
              encoderPitjar(63); 
              encoderDreta(22);   
              encoderEsquerra(14); 
            break;
            case 1:                   
              encoderPitjar(45);  
              encoderDreta(163);  
              encoderEsquerra(163); 
            break;
          }
        break;
        case 22:
          switch(mode){
            case 0:                    
              encoderPitjar(64);  
              encoderDreta(24);    
              encoderEsquerra(16); 
            break;
            case 1:                   
              encoderPitjar(37);   
              encoderDreta(163);   
              encoderEsquerra(163); 
            break;
          }
        break;
        case 24:
          switch(mode){
            case 0:                    
              encoderPitjar(65);   
              encoderDreta(54);  
              encoderEsquerra(22); 
            break;
            case 1:                    
              encoderPitjar(45);   
              encoderDreta(163);  
              encoderEsquerra(163); 
            break;
          }
        break;                 
      }
    break;
    case 1:                          
      switch(columna){
        case 14:                   
          encoderPitjar(28);  
          encoderDreta(94);  
          encoderEsquerra(24);
        break;
      }
    break;
    case 2:                             
      switch(columna){
        case 14:                       
          switch(mode){
            case 0:                 
              encoderPitjar(66);  
              encoderDreta(159);  
              encoderEsquerra(54); 
            break;
            case 1:                
              encoderPitjar(10);  
              encoderDreta(166);   
              encoderEsquerra(166); 
            break;
          }
        break;  
      }
    break;
    case 3:                             
      navegacioFilera3Pantalla2Nivell4();
    break;
  }  
}

void navegacioProgramaPantalla2b(){
  aplicarStart(22);
  switch(filera){
    case 0:
      switch(columna){
        case 14:
          switch(mode){
            case 0:
              encoderPitjar(67);
              encoderDreta(34);
              encoderEsquerra(120);
            break;
            case 1:
              encoderPitjar(21);
              encoderDreta(164);
              encoderEsquerra(164);
            break;  
          }
        break;
        case 34:
          switch(mode){
            case 0:
              encoderPitjar(68);
              encoderDreta(36);
              encoderEsquerra(14);
            break;
            case 1:
              encoderPitjar(48);
              encoderDreta(163);
              encoderEsquerra(163);
            break;  
          }
        break;
        case 36:
          switch(mode){
            case 0:
              encoderPitjar(69);
              encoderDreta(54);
              encoderEsquerra(34);
            break;
            case 1:
              encoderPitjar(80);
              encoderDreta(163);
              encoderEsquerra(163);
            break;  
          }
        break; 
      }
    break;
    case 1:
      switch(columna){
        case 14:
          switch(mode){
            case 0:
              encoderPitjar(70);
              encoderDreta(56);
              encoderEsquerra(36);
            break;
            case 1:
              encoderPitjar(37);
              encoderDreta(163);
              encoderEsquerra(163);
            break;  
          }
        break;
        case 16:
          switch(mode){
            case 0:
              encoderPitjar(71);
              encoderDreta(101);
              encoderEsquerra(54);
            break;
            case 1:
              encoderPitjar(45);
              encoderDreta(163);
              encoderEsquerra(163);
            break;  
          }
        break;
        /*
        case 35:
          switch(mode){
            case 0:
              encoderPitjar(72);
              encoderDreta(101);
              encoderEsquerra(56);
            break;
            case 1:
              encoderPitjar(81);
              encoderDreta(164);
              encoderEsquerra(164);
            break;  
          }
        break; 
        */ 
      }
    break;
    case 2:
      switch(columna){
        case 21:
          switch(mode){
            case 0:
              encoderPitjar(73);
              encoderDreta(159);
              encoderEsquerra(56);
            break;
            case 1:
              encoderPitjar(10);
              encoderDreta(163);
              encoderEsquerra(163);
            break;  
          }
        break; 
      }
    break;
    case 3:
      navegacioFilera3Pantalla2Nivell4();
    break;
  }  
}

void navegacioProgramaPantalla3(){
  aplicarStart(23);        
  if(!bloqueigBotons){
    switch(filera){
      case 0:                             
        switch(columna){
          case 15:                      
            encoderPitjar(29);
            if(vars[idxActivarMedicio] == 1 && !programaQuartEix()){
              encoderDreta(55);
            }
            else{
              encoderDreta(39);
            }
            encoderEsquerra(120);
          break;
          case 39:                
            encoderPitjar(13);
            encoderDreta(79);
            if(vars[idxActivarMedicio] == 1 && !programaQuartEix()){
              encoderEsquerra(55);       
            }
            else{
              encoderEsquerra(15); 
            }
          break;
        }
      break;
      case 1:
        switch(columna){
          case 15:
            encoderPitjar(30);
            encoderDreta(39);  
            encoderEsquerra(15);
          break;
          case 39:
            encoderPitjar(14);
            encoderDreta(159);
            encoderEsquerra(39);
          break;
        }
      break;
      case 3:
        switch(columna){
          case 0:  
            encoderPitjar(16);
            encoderDreta(15); 
            encoderEsquerra(255);
          break;
          case 39:                  
            encoderPitjar(8);   
            encoderDreta(255);
            encoderEsquerra(79);
          break;
        }
      break;
    }
  }  
}

void navegacioValorsProcesPantalla1(){
  aplicarStart(30);
  switch(filera){
    case 0:                    
      switch(mode){
        case 0:
          encoderPitjar(34);  
          encoderDreta(76);     
          encoderEsquerra(255); 
        break;
        case 1:    
          encoderPitjar(32); 
          encoderDreta(163);      
          encoderEsquerra(163);   
        break;
      }
    break;
    case 1:                
      switch(mode){
        case 0:              
          encoderPitjar(75);   
          if(vars[idxActivarQuartEix] == 1){
            encoderDreta(116);     
          }
          else{
            encoderDreta(159);  
          }
          encoderEsquerra(36);    
        break;
        case 1:                      
          encoderPitjar(32);     
          encoderDreta(163);       
          encoderEsquerra(163);   
        break;
      }
    break;
    case 2:                         
      switch(mode){
        case 0:
          encoderPitjar(76);        
          encoderDreta(159);          
          encoderEsquerra(76);     
        break;
        case 1:
          encoderPitjar(32);
          encoderDreta(163);
          encoderEsquerra(163);
        break;  
      }
    break;
    case 3:
      case 39:
        encoderPitjar(15);      
        encoderDreta(255);
        if(vars[idxActivarQuartEix] == 1){                    
          encoderEsquerra(116);
        }
        else{
          encoderEsquerra(76);
        }
      break;
      /*
      switch(columna){
        case 0:
          encoderPitjar(8);  
          encoderDreta(39);
          encoderEsquerra(255);
        break;
        case 39:
          encoderPitjar(15);      
          encoderDreta(255);
          if(vars[idxActivarQuartEix] == 1){                    
            encoderEsquerra(116);
          }
          else{
            encoderEsquerra(76);
          }
        break;
      }
      */
    break;
  }  
}

void navegacioValorsProcesPantalla2(){
  aplicarStart(31);
  switch(filera){
    case 0:
      switch(columna){
        case 34:
          switch(mode){
            case 0:
              encoderPitjar(77);
              encoderDreta(36);
              encoderEsquerra(120);
            break;
            case 1:
              encoderPitjar(43);
              encoderDreta(167);
              encoderEsquerra(167);
            break;
          }
        break;
        case 36:
          switch(mode){
            case 0:
              encoderPitjar(78);
              encoderDreta(76);
              encoderEsquerra(34);
            break;
            case 1:
              encoderPitjar(79);
              encoderDreta(163);
              encoderEsquerra(163);
            break;
          }
        break;
      }
    break;
    case 1:
      switch(mode){
        case 0:
          encoderPitjar(39);
          encoderDreta(116);
          encoderEsquerra(36);
        break;
        case 1:
          encoderPitjar(74);
          encoderDreta(164);
          encoderEsquerra(164);
        break;
      }
    break;
    case 2:
      switch(mode){
        case 0:
          encoderPitjar(40);
          encoderDreta(159);
          encoderEsquerra(76);
        break;
        case 1:
          encoderPitjar(74);
          encoderDreta(164);
          encoderEsquerra(164);
        break;
      }
    break;
    case 3:
      switch(columna){
        case 0:
          encoderPitjar(19);
          encoderDreta(34);
          encoderEsquerra(255);
        break;
        case 39:
          encoderPitjar(17);
          encoderDreta(255);
          encoderEsquerra(116);
        break;
      }
    break;  
  }
}

void navegacioValorsProcesPantalla3(){
  aplicarStart(32);
  switch(filera){
    case 0:
      encoderPitjar(33);
      encoderDreta(68);
      encoderEsquerra(120);
    break;
    case 1:
      switch(columna){
        case 28:
          switch(mode){
            case 0:
              encoderPitjar(82);
              encoderDreta(162);
              encoderEsquerra(28);
            break;
            case 1:
              navegacioEditarDigitBloqueig();
            break;
          }
        break;
        case 36:
          switch(mode){
            case 0:
              encoderPitjar(9);
              encoderDreta(78);
              encoderEsquerra(162);
            break;
            case 1:
              navegacioEditarDigitBloqueig();
            break;
          }
        break;
        case 38:
          encoderPitjar(11);
          encoderDreta(159);
          encoderEsquerra(76);
        break;
        default:
          switch(mode){
            case 0:
              encoderPitjar(82);
              encoderDreta(162);
              encoderEsquerra(162);
            break;
            case 1:
              navegacioEditarDigitBloqueig();
            break;
          }
        break;
      }
    break;
    case 3:
      switch(columna){
        case 0:
          encoderPitjar(16);
          encoderDreta(27);
          encoderEsquerra(255);
        break;
        case 39:
          encoderPitjar(8);
          encoderDreta(255);
          encoderEsquerra(78);
        break;
      }
    break;
  }
}

void navegacioEliminatProgrames(){
  aplicarStart(40);
  switch(filera){
    case 0:
      encoderPitjar(6);
      encoderDreta(40);              
      encoderEsquerra(255);         
    break;
    case 1:                  
      encoderPitjar(7);   
      encoderDreta(120);     
      encoderEsquerra(0);
    break;
    case 3:              
      encoderPitjar(8);    
      encoderDreta(255);
      encoderEsquerra(40);
    break;  
  }  
}

void navegacioEliminarUnPrograma(){
  aplicarStart(41);
  switch(filera){
    case 2:                           
      switch(columna){
        case 19:
          switch(mode){
            case 0:       
              encoderPitjar(41); 
              encoderDreta(129);
              encoderEsquerra(255); 
            break;
            case 1:       
              encoderPitjar(10);  
              encoderDreta(163);     
              encoderEsquerra(163);  
            break;  
          }
        break;
      }
    break;    
    case 3:    
      switch(columna){
        case 9:        
          encoderPitjar(8); 
          encoderDreta(141);
          encoderEsquerra(99);
        break;
        case 21:
          encoderPitjar(8); 
          encoderDreta(255);    
          encoderEsquerra(129); 
        break;
      }            
    break;  
  }  
}

void navegacioEliminarTotsProgrames(){
  aplicarStart(42);
  switch(filera){
    case 2:                              
      switch(columna){
        case 10:    
          encoderPitjar(8);
          encoderDreta(101); 
          encoderEsquerra(255);
        break;
        case 21:                  
          encoderPitjar(8); 
          encoderDreta(255); 
          encoderEsquerra(90);
        break;  
      }
    break;
  }  
}

void comprovacionsStepsEixosZYA(){
  switch(funcioRotatoriManual){
    case 0:
      checkStepsConstantsMolaManual();
    break;
    case 1:
      checkStepsConstantsDivisorManual();
    break;
    case 2:
      checkStepsConstantsEixYmanual();
    break;
  }
}

void accionsRotatoriModeManual(){
  checkTempsRotatoriPitjat();
  encoderPitjar(26);
  encoderDreta(168);
  encoderEsquerra(168); 
}

void accionsBotonsModeManual(){
  accionsMoureEixX();
}

void accionsRotatoriPreviesEsmolatNormal(){
  checkStepsConstantsMolaManual();
  if(!pitjat && !leftPitjat && !rightPitjat){
    encoderDreta(168);                           // moure mola amunt
    encoderEsquerra(168);                        // moure mola avall
  }
  accioPitjarPrevisAutomatic();
}

void accionsRotatoriPreviesEsmolatQuartEix(){
  switch(columna){
    case 16:
      switch(mode){
        case 0:
          encoderPitjar(38);
          encoderDreta(138);
          encoderEsquerra(255);
        break;
        case 1:
          if(!leftPitjat && !rightPitjat){
            encoderPitjar(42);
            if(!pitjat){
              encoderDreta(168);
              encoderEsquerra(168);
            }
          }
        break;  
      }
    break;
    case 18:
      switch(mode){
        case 0:
          encoderPitjar(18);
          encoderDreta(142);
          encoderEsquerra(136);
        break;
        case 1:
          if(!leftPitjat && !rightPitjat){
            encoderPitjar(42);
            if(!pitjat){
              encoderDreta(168);
              encoderEsquerra(168);
            }
          }
        break;  
      }
    break;
    case 22:
      accioPitjarPrevisAutomatic();
      if(!confirmarAutoOsalir){
        encoderDreta(255);
        encoderEsquerra(138);
      }
    break;  
  }
}

void checkConfirmarIprintarSalir(){
  if(confirmarAutoOsalir){
    if(tempsConfirmarAutoOsalir < millis()){
      if(printarSalir){
        printSalir();
        printarSalir = false;
      }
    }
  }
}

void accionsBotonsPreviesEsmolat(){
  if(programaQuartEix()){
    accionsMoureEixXambAccel();
  }
  else{
    accionsMoureEixX();  
  }
}

void navegacioAutomatics(){
  if(!programaQuartEix()){
    if(mode == 1){
      if(!(filera == 2 && (columna == 34 || columna == 35))){
        encoderPitjar(10);
      }
      else{
        encoderPitjar(87);
      }
    }
    switch(filera){
      case 0:                                 
        switch(mode){
          case 0:    
            encoderPitjar(55);        
            encoderDreta(72);          
            encoderEsquerra(255);       
          break;
          case 1:                         
            encoderDreta(165);      
            encoderEsquerra(165);
          break;
        }
      break;
      case 1:                           
        switch(mode){
          case 0:
            encoderPitjar(59);
            encoderDreta(112);
            encoderEsquerra(32);
          break;
          case 1:
            encoderDreta(165);
            encoderEsquerra(165);
          break;
        }
      break;
      case 2:
      switch(columna){
        case 32:
          switch(mode){
            case 0:
              encoderPitjar(61);
              if(movimentsAutomatics == 0){
                encoderDreta(115);
              }
              else{
                encoderDreta(255);
              }
              encoderEsquerra(72);
            break;
            case 1:
              encoderDreta(165);
              encoderEsquerra(165);
            break;
          }
        break;
        case 35:
          switch(mode){
            case 0:
              encoderPitjar(86);
              encoderDreta(255);
              encoderEsquerra(112);
            break;
            case 1:
              // Sense encoderDreta ni esquerra. El pitjar, a l'inici del metode
            break;
          }
        break;  
      }
      break;
    }
  }
}

void accionsEsmolatAturat(){
  parpadeigBotoDret();              // llum boto dret intermitent
  if(pausat){
    rightButton(3);                 // rependre moviments automatics
  }
  else{
    rightButton(2);                 // iniciar moviments automatics
  }
  leftButton(1);
  if(vars[idxActivarMagnetisme] == 1){
    desactivarPlatMagnetic();
  }
}

void accionsEsmolatEnProces(){
  if(!programaQuartEix()){
    comprovacionsPosEixXauto();
    if(tempsCarroDeFCx1AFCx2 > 0){
      // Fase d'inici d'angle2, havent ja calculat el temps que tarda en desplaçarse d'un extrem a l'altre en X
      checkMeitatDeCami();
    }
  }
  else{
    comprovacionsPosEixYauto();
  }
  if(tempsCarroDeFCx1AFCx2 == 0){ // Només es > 0 en la part previa a l'angle2 que recorre la meitat del trajecte. I és just on no volem els botons
    rightButton(4);                          // pausar cicle
    leftButton(2);                           // aturar cicle
  }
  sumarSegonsCicle();
}

void navegacioConfirmarPas(){
  aplicarStart(60);
  switch(filera){
    case 3:                                   // FILERA 3 (configNivell 13)
      switch(columna){
        case 16:                              // COLUMNA 16 (filera 3, configNivell 13)
          encoderPitjar(50);              // a configNivell 12
          encoderDreta(143);               // a columna 23
          encoderEsquerra(255);            // sense efectes. Nomes actualitzem el valorEncoder
        break;
        case 23:          // COLUMNA 23 (filera 3, configNivell 13)
          encoderPitjar(2);              // a configNivell 4, pantalla 3, filera 0, columna 13
          encoderDreta(255);               // sense efectes. Nomes actualitzem el valorEncoder
          encoderEsquerra(136);            // a columna 16
        break;
      }
    break;
  }
}

void navegacioAvisRecanvis(){
  aplicarStart(61);
  encoderPitjar(8);
  switch(columna){
    case 0:
      encoderDreta(143);
      encoderEsquerra(255);
    break;
    case 23:
      encoderDreta(255);
      encoderEsquerra(120);
    break;  
  }
}

void accionsModeProves(){
  aplicarStart(62);
  provesMUX16In();
  provesEncoder();
}

void accionsIcomprovacionsMedicioAutomatica(){
  aplicarStart(65);
  if(iniciantEsmolatAngle2){
    sumarSegonsCicle();
    navegacioAutomatics();
  }
  if(mesurarSoroll){
    sorollAssolit(); 
    leftButton(5); 
  }
  else if(desplasantCarroPrevi){
    comprovacionsPosEixXauto();
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * LOOP SUBFUNCIONS COMUNES
 ************************************************************************************
 ************************************************************************************
 */


void navegacioFilera3Pantalla2Nivell4(){
  switch(columna){
    case 0:                        
      encoderPitjar(19);        
      encoderDreta(14);            
      encoderEsquerra(255);     
    break;
    case 39:                         
      encoderPitjar(17);             
      encoderDreta(255); 
      if(!programaQuartEix()){      
        encoderEsquerra(94);    
      }
      else{
        encoderEsquerra(101);
      }
    break;
  }
}

void navegacioEditarDigitBloqueig(){
  encoderPitjar(12);
  encoderDreta(163);
  encoderEsquerra(163);
}

void accionsMoureEixX(){
  if(!bloqueigBotons){
    rightButton(1);
    leftButton(0);
    comprovacionsPosEixXmanual();
  }  
}

void accionsMoureEixXambAccel(){
  if(!bloqueigBotons){
    rightButton(5);
    leftButton(6);
  }  
}

void accioPitjarPrevisAutomatic(){
  if(!bloqueigBotons && !leftPitjat && !rightPitjat){
    // S'ha de posar despres de les lectures de gir dreta o esquerra.
    // Cas contrari, si es compleix el cas que despitjem el rotatori i sortim a pantalla transmetre
    // (on bloqueigBotons = false, pitjat ja es true), just despres s'entra a comprovar els girs, els quals
    // ja pertanyen al rotatori senzill (s'ha aplicat ja desactivarRotatoriMola).
    // En tal cas, si al despitjar el boto sense voler el fem girar una mica, com que ja es el responsable dels girs,
    // s'aplicaria un gir, activant en ultima instancia el bloqueigBotons = ture, fent que en la pantalla de transmetre
    // no es contempli la navegacio (que necessita bloqueigBotons = false)
    encoderPitjar(31);       
  }
}

void comprovacionsPosEixXmanual(){
  comprovacionsMovCarroDretManual(); 
  comprovacionsMovCarroEsquerreManual();
}

void comprovacionsPosEixXauto(){
  comprovacionsMovCarroDretAuto(); 
  comprovacionsMovCarroEsquerreAuto();  
}


/*
 ************************************************************************************
 ************************************************************************************
 * START
 ************************************************************************************
 ************************************************************************************
 */ 


void aplicarStart(byte option){
  /*
    0-9:     Reservats a menu secret
    10-19:   Reservats a menu inicial
    20-29:   Reservats a configuracio de programa
    30-39:   Reservats a valors de proces
    40-44:   Reservats a eliminat de programes
    45-49:   Reservats a Moviments manuals
    50-54:   Reservats a Previs automatic
    55-59:   Reservats a Esmolat automatic
    60:      Reservat  a Confirmar pas
    61:      Reservat  a Confirmar pas recanvi peces
    62-64:   Reservat  a Mode proves
    65+:     Altres
  */
  if(firstIn){
    if(!blinkOff){
      noBlincar();  
    }
    switch(option){
      case 0:
        startArrencadaSistema();
      break;
      case 1:
        startMenuSecretPantalla1();                   
      break;
      case 2:
        startMenuSecretPantalla2();
      break;
      case 3:
        startMenuSecretPantalla3();
      break;
      case 10:
        startMenuInicial();
      break;
      case 20:
        startProgramaPantalla1();
      break;
      case 21:
        startProgramaPantalla2a();
      break;
      case 22:
        startProgramaPantalla2b();
      break;
      case 23:
        startProgramaPantalla3();
      break;
      case 30:
        startValorsProcesPantalla1();
      break;
      case 31:
        startValorsProcesPantalla2();
      break;
      case 32:
        startValorsProcesPantalla3();
      break;
      case 40:
        startEliminatProgrames();
      break;
      case 41:
        startEliminarUnPrograma();
      break;
      case 42:
        startEliminarTotsProgrames();
      break;
      case 45:
        startMovimentsManuals();
      break;
      case 50:
        startPrevisAutomatic();
      break;
      case 55:
        startEsmolatAutomatic();
      break;
      case 60:
        startConfirmarPasNivell();
      break;
      case 61:
        startConfirmarPasAltres(); 
      break;
      case 62:
        startModeProves();
      break;
      case 65:
        startMedicioAutomatica();
      break;
    }
    firstIn = false;
  }  
}


/*
 ************************************************************************************
 ************************************************************************************
 * START SUBFUNCIONS
 ************************************************************************************
 ************************************************************************************
 */


void startArrencadaSistema(){
  comRTC();
  actualitzarRellotge();
  initComFA();
}
 
void startMenuSecretPantalla1(){
  printMenuSecret();
  printMenuSecretValors();
  if(cursorAlInici){
    setBlink(3,0);
  }
  else{
    setBlink(36,3);
    cursorAlInici = true;
  }
}

void startMenuSecretPantalla2(){
  printMenuSecret2();
  printMenuSecret2Valors();
  if(cursorAlInici){
    setBlink(25,0);
  }
  else{
    setBlink(25,3);
    cursorAlInici = true;
  }
}

void startMenuSecretPantalla3(){
  printMenuSecret3();
  printMenuSecret3Valors();
  setBlink(26,0);
}

void startMenuInicial(){
  printMenuInicial();
  blinkCoordenades();
}

void startProgramaPantalla1(){
  llumPerMagnetisme();
  printProgramaPantalla1();
  printProgramaPantalla1Valors();
  if(!cursorAlInici){
    setBlink(38,2);
    cursorAlInici = true;
  }
  else{
    setBlink(14,0);
  }
}

void startProgramaPantalla2a(){
  printProgramaPantalla2();
  printProgramaPantalla2Valors();
  if(!cursorAlInici){
    setBlink(14,2);
    cursorAlInici = true;
  }
  else{
    setBlink(14,0);
  }
}

void startProgramaPantalla2b(){
  printProgramaPantalla2b();
  printProgramaPantalla2bValors();
  if(!cursorAlInici){
    setBlink(21,2);
    cursorAlInici = true;
  }
  else{
    setBlink(14,0);
  }  
}

void startProgramaPantalla3(){
  printProgramaPantalla3();
  printProgramaPantalla3Valors();
  setBlink(15,0);
}

void startValorsProcesPantalla1(){
  printValorsProces();
  printValorsProcesValors();
  if(!cursorAlInici){
    if(vars[idxActivarQuartEix] == 1){
      setBlink(36,2);
    }
    else{
      setBlink(36,1);
    }
    cursorAlInici = true;
  }
  else{
    setBlink(36,0);
  }
}

void startValorsProcesPantalla2(){
  printValorsProces2();
  printValorsProces2Valors();
  if(!cursorAlInici){
    setBlink(36,2);
    cursorAlInici = true;  
  }
  else{
    setBlink(34,0);
  }
}

void startValorsProcesPantalla3(){
  printValorsProces3();
  printValorsProces3Valors();
  setBlink(28,0);
}

void startEliminatProgrames(){
  printEliminatProgrames();
  setBlink(0,0);
}

void startEliminarUnPrograma(){
  vars[idxPrograma] = 0;
  printEliminarUnPrograma();
  printEliminarUnProgramaValors();
  setBlink(19,2);
}

void startEliminarTotsProgrames(){
  printEliminarTotsProgrames();
  setBlink(10,2);
}

void startMovimentsManuals(){
  manualsIprevisConfigInicialComuna();
  printSalir2();
  printMuela();
  printPosMolaEixYStatic();
  printEixZcurrentPosition();
  if(vars[idxRefrigeracio] == 0){
    escriureOut16(aiguaMotor, HIGH);
  }
  canviarRotatori();
}

void startPrevisAutomatic(){
  manualsIprevisConfigInicialComuna();
  if(!programaQuartEix()){
    printPosMolaEixYStatic();
    printEixZcurrentPosition();
    activarGirMola();
    canviarRotatori();
  }
  else{
    activarGirMola2();
    setBlink(16,3); 
  }
}
      
void startEsmolatAutomatic(){
  if(!iniciantEsmolatAngle2){
    initAutomatic();
  }
  else{
    initAutomatic2();
  }
  if(!programaQuartEix()){
    inicialitzarDireccioCarro();
    moureCarroAutomatic();
  }
  else{
    // Eix X anira tota l'estona per el mateix sentit. Steps cridats via HIGH_D14 i LOW_D14
    digitalWrite(eixXdirection, direccioCarroAuto);
    countEtapaIniciat = true;
    dirEixYauto *= -1;
    direccioCarroAuto = 1; // Fins que no es pugui escollir a quina direccio esmolar, sempre cap a l'esquerra
    initPosYauto = eixYcurrentPosition;
    moureEixYautomatic();
  }
}

void startConfirmarPasNivell(){
  printConfirmarUnPas();
  setBlink(23,3);
}

void startConfirmarPasAltres(){
  printPantallaAvisRecanvis();
  setBlink(23,3);
}

void startModeProves(){
  printProves();
  provesOUT16();
}

void startMedicioAutomatica(){
  ajustarMolaAutomatic();
}


/*
 ************************************************************************************
 ************************************************************************************
 * ENCODER PITJAR
 ************************************************************************************
 ************************************************************************************
 */


void encoderPitjar(byte option){
  if((PINB & (1<<PORTB0)) == LOW && !pitjat){ // digitalRead(pitjar)
    pitjat = true;
    delay(10);
    switch(option){
      case 0:
        if(vars[idxHoresRecanvis_low]+vars[idxHoresRecanvis_high]*256 >= vars[idxHoresRecanvis_high+1]+vars[idxHoresRecanvis_high+2]*256){
          encoderPitjarAConfigNivell(13);
        }
        else{
          previReferenciar = true;
          encoderPitjarAConfigNivell(12);
        }
      break;
      case 1:
        encoderPitjarAConfigNivell(14);
      break;
      case 2:
        if(!previReferenciar){
          encoderPitjarAConfigNivell(4);
        }
      break;
      case 3:
        encoderPitjarAConfigNivell(9);
      break;
      case 4:
        encoderPitjarAConfigNivell(5);
      break;
      case 5:
        encoderPitjarAConfigNivell(6);
      break;
      case 6:
        encoderPitjarAConfigNivell(7);
      break;
      case 7:
        encoderPitjarAConfigNivell(8);
      break;
      case 8:
        encoderPitjarAConfigNivell(3);
      break;
      case 9:
        aModeEdicio();
      break;
      case 10:
        encoderPitjarAmodeMoures(255);
      break;
      case 11:
        comprovarCodiEntrat();
      break;
      case 12:
        encoderPitjarAmodeMoures(0);
      break;
      case 13:
        encoderPitjarGuardarPrograma();
      break;
      case 14:
        guardarPrograma();              // guardem al transmetre, per si de cas
        if(!bloqueigMaquina){
          if(vars[idxActivarMagnetisme] == 1 && !platMagneticOn){
            noBlincar();
            printErrorPlat(1);
            delay(3000);
            firstIn = true;
          }
          else{
            establirIntsAutomatics();
            establirMovimentsAutomatics();
            /*
            esborrarPantalla();
            for(int i=0; i<5; i++){
                lcd.print(arrayEsmolat[i]);
            }
            delay(3000);
            */
            if(hiHaEtapes){
              if(!programaQuartEix()){
                nextEtapaAutomatic();
              }
              else{
                nextEtapaAutomatic2();  
              } 
              posicionarDivisor();
              encoderPitjarAConfigNivell(12);
            }
          }
        }
      break;
      case 15:
        encoderPitjarAPantalla(2);
      break;
      case 16:
        cursorAlInici = false;
        encoderPitjarAPantalla(2);
      break;
      case 17:
        encoderPitjarAPantalla(3);
      break;
      case 18:
        aModeEdicio();
        printPosMolaEixYStatic();
        printEixStaticPrevis("Y:");
        printEixYcurrentPosition();
        funcioRotatoriManual = 2;
        canviarRotatori();
      break;
      case 19:
        cursorAlInici = false;
        encoderPitjarAPantalla(1);
      break;
      case 20:
        encoderPitjarVariableBinaria(idxActivarMedicio, idxActivarMedicioPrint, true);
        if(vars[idxActivarMedicio] == 0){
          reiniciarValorsMedicio();   
          pos = INIT_POS_VARSGENERALS_EEPROM;
        }
      break;
      case 21:
        encoderPitjarAmodeMoures(1);
      break;
      case 22:
        encoderPitjarAmodeEdicio(idxAcc_low);
        establirValorIlimitsInt();
      break;
      case 23:
        encoderPitjarAmodeEdicio(idxDacc_low);
        establirValorIlimitsInt();
      break;
      case 24:
        encoderPitjarAmodeMoures(4);
      break;
      case 25:
        encoderPitjarAmodeMoures(5);
      break;
      case 26:
        if(!bloqueigBotons && !leftPitjat && !rightPitjat){
          gestioTemps = millis() + 1500UL;
        }
      break;
      case 27:
        encoderPitjarAmodeMoures(6);
      break;
      case 28:
        encoderPitjarVariableBinaria(idxGirMola, idxGirMolaPrint, false);
      break;
      case 29:
        encoderPitjarVariableBinaria(idxProduccio, idxProduccioPrint, false);
      break;
      case 30:
        encoderPitjarVariableBinaria(idxMedicio, idxMedicioPrint, false);
      break;
      case 31:
        confirmarAutoOsalir = true;
        printarSalir = true;
        tempsConfirmarAutoOsalir = millis() + 2000UL;
        if(programaQuartEix()){
          noBlincar();
        }
      break;
      case 32:
        encoderPitjarAmodeMoures(14);
      break;
      case 33:
        encoderPitjarVariableBinaria(idxRefrigeracio, idxRefrigeracioPrint, true);
      break;
      case 34:
        encoderPitjarAmodeEdicio(idxVelManualX);
      break;
      case 35:
        encoderPitjarVariableBinaria(idxActivarMagnetisme, idxActivarMagnetismePrint, true);
      break;
      case 36:
        encoderPitjarMagnetisme();
      break;
      case 37:
        encoderPitjarAmodeMoures(12);
      break;
      case 38:
        aModeEdicio();
        printPosMolaEixYStatic();
        printEixStaticPrevis("Z:");
        printEixZcurrentPosition();
        funcioRotatoriManual = 0;
        canviarRotatori();
      break;
      case 39:
        encoderPitjarAmodeEdicio(idxSensibilitat_low);
        establirValorIlimitsInt();
      break;
      case 40:
        encoderPitjarAmodeEdicio(idxCompensacio_low);
        establirValorIlimitsInt();
      break;
      case 41:
        vars[idxPrograma] = 0;
        encoderPitjarAmodeEdicio(idxPrograma);
      break;
      case 42:
        encoderPitjarAmodeMoures(8);
      break;
      case 43:
        encoderPitjarAmodeMoures(3);
      break;
      case 44:
        increment = 100;
        encoderPitjarAmodeEdicio(idxRecorregutZ_low);
        establirValorIlimitsLong();
      break;
      case 45:
        encoderPitjarAmodeMoures(13);
      break;
      case 46:
        encoderPitjarVariableBinaria(idxActivarQuartEix, idxActivarQuartEixPrint, true);
      break;
      case 47:
        increment = 20;
        encoderPitjarAmodeEdicio(idxRecorregutY_low);
        establirValorIlimitsLong();
      break;
      case 48:
        encoderPitjarAmodeMoures(15);
      break;
      case 49:
        encoderPitjarAmodeEdicio(idxIncrementVel_low);
        establirValorIlimitsInt();       
      break;
      case 50:
        if(previReferenciar){
          previReferenciar = false;
          referenciar();
          encoderPitjarAConfigNivell(3);
        }
        else{
          encoderPitjarAConfigNivell(10);
        }
      break;
      case 51:
        encoderPitjarAmodeEdicio(idxDesbast);
      break;
      case 52:
        encoderPitjarAmodeEdicio(idxDesbastDec_low);
        establirValorIlimitsInt();
      break;
      case 53:
        encoderPitjarAmodeEdicio(idxDesbastDivDec_low);
        establirValorIlimitsInt();
      break;
      case 54:
        encoderPitjarAmodeMoures(9);
      break;
      case 55:
        encoderPitjarAmodeEdicio(idxDesbastVel);
      break;
      case 56:
        encoderPitjarAmodeEdicio(idxAcabatDec_low);
        establirValorIlimitsInt();
      break;
      case 57:
        encoderPitjarAmodeEdicio(idxAcabatDivDec);
      break;
      case 58:
        encoderPitjarAmodeMoures(10);
      break;
      case 59:
        encoderPitjarAmodeEdicio(idxAcabatVel);
      break;
      case 60:
        encoderPitjarAmodeEdicio(idxLapejat);
      break;
      case 61:
        encoderPitjarAmodeEdicio(idxLapejatVel);
      break;
      case 62:
        encoderPitjarAmodeEdicio(idxAngle1);
      break;
      case 63:
        encoderPitjarAmodeEdicio(idxAngle1Dec);
      break;
      case 64:
        encoderPitjarAmodeEdicio(idxAngle2);
      break;
      case 65:
        encoderPitjarAmodeEdicio(idxAngle2Dec);
      break;
      case 66:
        encoderPitjarAmodeEdicio(idxBaixada);
      break;
      case 67:
        encoderPitjarAmodeEdicio(idxNdents_low);
        establirValorIlimitsInt();
      break;
      case 68:
        encoderPitjarAmodeEdicio(idxPasDent);
      break;
      case 69:
        encoderPitjarAmodeEdicio(idxPasDentDec);
      break;
      case 70:
        encoderPitjarAmodeEdicio(idxAngle3);
      break;
      case 71:
        encoderPitjarAmodeEdicio(idxAngle3Dec);
      break;
      case 72:
        increment = 2000;
        encoderPitjarAmodeEdicio(idxMolaRpm_low);
        establirValorIlimitsInt();
      break;
      case 73:
        encoderPitjarAmodeEdicio(idxRecAfilat);
      break;
      case 74:
        encoderPitjarAmodeMoures(2);
      break;
      case 75:
        encoderPitjarAmodeEdicio(idxStepsManualZ);
      break;
      case 76:
        encoderPitjarAmodeEdicio(idxStepsManualY);
      break;
      case 77:
        encoderPitjarAmodeEdicio(idxOffsetA);
      break;
      case 78:
        encoderPitjarAmodeEdicio(idxOffsetAdec);
      break;
      case 79:
        encoderPitjarAmodeMoures(11);
      break;
      case 80:
        encoderPitjarAmodeMoures(7);
      break;
      /*
      case 81:
        increment = increment - (increment-1); // Posar increment a 1 altre cop
        encoderPitjarAmodeMoures(1);
      break;
      */
      case 82:
        encoderPitjarAmodeEdicio(idxDigitCodiActual);
      break;
      case 83:
        increment = increment - (increment-1); // Posar increment a 1 altre cop
        encoderPitjarAmodeMoures(2);
      break;
      case 84:
        increment = increment - (increment-1); // Posar increment a 1 altre cop
        encoderPitjarAmodeMoures(16);
      break;
      case 85:
        increment = 100;
        encoderPitjarAmodeEdicio(idxVelocitatZ_low);
        establirValorIlimitsInt();
      break;
      case 86:
        aturarParpadeigBotoDret();
        bloqueigBotons = true;
        if(platAzero){
          noBlincar();
          printPlatEnEsmolatApos();
          setBlink(34,2);
          funcioRotatoriManual = 2;
        }
        else{
          noBlincar();
          printPlatEnEsmolatA0();
          setBlink(35,2);
          funcioRotatoriManual = 1;
          //posAux = pos;
          //pos = vars[idxPrograma] * N_VARS_PROGRAMA;
          if(idxArrayEsmolat < 3){
            angleAux = vars[idxAngle1];
            angleDecAux = vars[idxAngle1Dec];
            vars[idxAngle1] = 0;
            vars[idxAngle1Dec] = 0;
          }
          else{
            angleAux = vars[idxAngle2];
            angleDecAux = vars[idxAngle2Dec];
            vars[idxAngle2] = 0;
            vars[idxAngle2Dec] = 0;
          }
          //pos = posAux;
          grauDivisorAux = grauDivisor;
          grauDivisorDecimalAux = grauDivisorDecimal;
        }       
        mode = 1;
      break;
      case 87:
        switch(funcioRotatoriManual){
          case 1:
            noBlincar();
            iniciantEsmolatAngle2 = true;
            posicionarDivisor();
            iniciantEsmolatAngle2 = false;
            initParpadeigBotoDret();
            printPlatEnEsmolatPlat();
            blinkCoordenades();
            platAzero = true;
            funcioRotatoriManual = 0;
            if(idxArrayEsmolat < 3){
              vars[idxAngle1] = angleAux;
              vars[idxAngle1Dec] = angleDecAux;
            }
            else{
              vars[idxAngle2] = angleAux;
              vars[idxAngle2Dec] = angleDecAux;
            }
            grauDivisor = grauDivisorAux;
            grauDivisorDecimal = grauDivisorDecimalAux;           
          break;
          case 2:
            noBlincar();
            iniciantEsmolatAngle2 = true;
            posicionarDivisor();
            iniciantEsmolatAngle2 = false;
            initParpadeigBotoDret();
            printPlatEnEsmolatPlat();
            setBlink(35,2);
            platAzero = false;
            funcioRotatoriManual = 0;
          break;
        }
        bloqueigBotons = false;
        mode = 0;
      break;
      
      case 255:
        // Casos sense efectes
      break;
    }
  }
  if((PINB & (1<<PORTB0)) == HIGH && pitjat){ // digitalRead(pitjar)
    pitjat = false;
    delay(10);
    switch(option){
      case 26:
        if(millis() <= gestioTemps){
          // Encoder pitjat durant menys d'un segon, i estem dins de moviments manuals.
          // Per tant, en comptes de sortir dels manuals, cambiem la funcio del rotatori plat/mola
          printSalir2();
          if(funcioRotatoriManual == 0){
            printPlato();
            printEixAcurrentPosition();
            funcioRotatoriManual = 1;
          }
          else if(funcioRotatoriManual == 1){
            if(vars[idxActivarQuartEix] == 0){
              printMuela();
              printPosMolaEixYStatic();
              printEixZcurrentPosition();
              funcioRotatoriManual = 0;
            }
            else{
              printEixY();
              printPosMolaEixYStatic();
              printEixYcurrentPosition();
              funcioRotatoriManual = 2;
            }
          }
          else{
            printMuela();
            printPosMolaEixYStatic();
            printEixZcurrentPosition();
            funcioRotatoriManual = 0;
          }
          gestioTemps = 0;
        }
      break;
      case 31:
        if(confirmarAutoOsalir){
          if(!programaQuartEix()){
            canviarRotatori();  
          }
          bloqueigBotons = false;
          confirmarAutoOsalir = false;
          tempsConfirmarAutoOsalir = 0UL;
          if(printarSalir){ // No ha donat temps a que l'accio es converteixi en salir
            if(vars[idxMedicio] == 0){
              encoderPitjarAConfigNivell(11);
            }
            else{
              encoderPitjarAConfigNivell(15); 
            }
          }
          else{ // Apretat prou temps com per a que l'accio sigui salir
            // Com que en aqu´ ja s'han inicialitzat certes variables del proces automatic (a mes de les llums), cal desfer-ho tot
            encoderPitjarAConfigNivell(4);
          }
          printarSalir = false;
        }
      break;
    }
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * ENCODER PITJAR SUBFUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 

void encoderPitjarAConfigNivell(byte conf){
  switch(conf){
    case 2:
      pantalla = 1;
      pos = INIT_POS_VARSGENERALS_EEPROM;
    break;
    case 3:
      if(configNivell == 1 || configNivell == 12 || configNivell == 13){
        if(configNivell == 13){
          if(columna == 0){
            guardarVariable(idxHoresRecanvis_low, 0, true, 0, false, 0);
          }
        }
        vars[idxPrograma] = 0;
        pantalla = 1;
        setCoordenades(0,0);
        llumPerMagnetisme();
      }
      else if(configNivell == 4){
        vars[idxPrograma] = 0;
        pantalla = 1;
        setCoordenades(0,0); 
      }
      else if(configNivell == 5){
        pantalla = N_PANTALLES_MENUINICIAL;
        setCoordenades(0,1);
      }
      else if(configNivell == 6){
        setCoordenades(0,2);
      }
      else if(configNivell == 7){
        if(columna == 9){
          pos = vars[idxPrograma] * N_VARS_PROGRAMA;
          eliminarPrograma();
          noBlincar();
          printProgramaEliminat();
          delay(1500); 
        }
        setCoordenades(0,2);
      }
      else if(configNivell == 8){
        if(columna == 10){
          eliminarTotsProgrames();
          noBlincar();
          printProgramesEliminats();
          delay(1500);
        }
        setCoordenades(0,2);  
      }
      else if(configNivell == 9){
        canviarRotatori();
        if(vars[idxRefrigeracio] == 0){
          escriureOut16(aiguaMotor, LOW);
        }
        apagarLlumBotons();
        llumPerMagnetisme();
      }
    break;
    case 4:
      if(configNivell == 3){
        vars[idxPrograma] = (pantalla-1)*4 + filera;
        pantalla = 1;
        recuperarPrograma();
      }
      else if(configNivell == 10 || configNivell == 15){
        if(!programaQuartEix()){
          aturarGirMola();
        }
        else{
          aturarGirMola2();
        }
        apagarLlumBotons();
        desferVariablesInicialsAutomatic();
      }
      else if(configNivell == 12){
        desferVariablesInicialsAutomatic();
      }
      else if(configNivell == 11){
        desferVariablesInicialsAutomatic();
        bloqueigBotons = false;
        nCuchilla = 1;
        molaPosicioAngle1 = 0L;
        molaPosicioAngle2 = 0L;
      }
    break;
    case 5:
      pantalla = 1;
      pos = INIT_POS_VARSGENERALS_EEPROM;
    break;
    case 9:
      gestioTemps = 0UL;
    break;
    case 10:
      if(programaQuartEix()){
        gestioTemps = millis() + 10000UL;  
      }
      else{
        gestioTemps = 0UL;  
      }
    break;
    case 11:
      pos = INIT_POS_VARSGENERALS_EEPROM;
      if(configNivell == 15){
        // Venim de medicio automatica
        // Despres de medicio, la primera pasada la farem lenta. Nomes la primera. Sempre que la actual ja no sigui mes lenta
        if(velAux > 30){
          vars[idxCurrentVel] = 30;
          actualitzarVelocitat();
          aplicarMenysVelPrimeraPasada = true;
        }       
        if(existeixAngle2 && vars[idxProduccio] == 1){
          // guardem posicio de la mola en ange1 o angle2 
          if(idxArrayEsmolat < 3){
            // Guardem posicio mola angle 1, per al finalitzar l'esmolat recuperar-ne la posició
            molaPosicioAngle1 = eixZposActual;
          }
          else{
            molaPosicioAngle2 = eixZposActual;
          }
        }
        if(mode == 0){
          noBlincar();
        }
        printLapeadoCompleto(); // Printa 0 ja que prèviament count s'ha posat a 0 en el initAcabadoVariables();
        blinkSegonsMode();
      }
    break;
  }
  configNivell = conf;
  firstIn = true;
}

void encoderPitjarAmodeEdicio(int op){
  idxVar = op;
  aModeEdicio();
}

void encoderPitjarAmodeMoures(byte op){
  switch(op){
    case 0:
      digitsCodiActual[columna-28] = vars[idxDigitCodiActual];
    break;
    case 1:
      guardarIntEnEdicioAarray(); // Variables de programa int. Guardat a l'array vars al sortir de mode edicio
    break;
    case 2:
      guardarIntEnEdicioAarray(); // Variables generals int. Guardat a vars i a EEPROM
      guardarVariable(idxVar, vars[idxVar], true, vars[idxVar+PROPS_VAR], false, 0);
    break;
    case 3:
      // Check consistencia variables generals signed. Despres guardem el valor a EEPROM
      checkConsistenciaDecimal(2, 0, 2);
      setVariable();
    break;
    case 4:
      checkConsistenciaDecimal(4, 1, 1);
      consistenciaDivisorTotal(0, true, true); // pos, valorDecEs16b, divDecEs16b
    break;
    case 5:
      // Guardem primer per a que les comprovacions prenguin els valors actualitzats de vars[]
      guardarIntEnEdicioAarray();
      checkConsistenciaDecimal(0, 1, 1);
      consistenciaDivisorTotal(1, true, true);
      //guardarIntEnEdicioAarray();
    break;
    case 6:
      consistenciaDivisorTotal(1, true, false);
      guardarIntEnEdicioAarray();
    break;
    case 7:
      if(vars[idxVar-PROPS_VAR] == vars[idxVar-PROPS_VAR+1] && vars[idxVar] == 0){
        vars[idxVar] = 1;
        printVariableByte(idxVar);
      }
      else if(vars[idxVar-PROPS_VAR] == vars[idxVar-PROPS_VAR+2] && vars[idxVar] > 0){
        vars[idxVar] = 0;
        printVariableByte(idxVar);
      }
    break;
    case 8:
      canviarRotatori();
      funcioRotatoriManual = 0;
      printBorrarMolaEixY();
    break;
    case 9:
      consistenciaDivisorTotal(2, true, true);
      guardarIntEnEdicioAarray();
    break;
    case 10:
      consistenciaDivisorTotal(2, true, false);
    break;
    case 11:
      checkConsistenciaDecimal(0, 2, 2);
      setVariable();
    break;
    case 12:
      checkConsistenciaDecimal(2, false, 1);
      editarSegonAngle();
    break;
    case 13:
      checkConsistenciaDecimal(0, false, 1);
      editarSegonAngle();
    break;
    case 14:
      setVariable();
      stepsEixXmanual = 40 + (vars[idxIncrementVel_low]+vars[idxIncrementVel_high]*256) * (vars[idxVar]-1);
    break;
    case 15:
      if(vars[idxVar] == vars[idxVar+1] && vars[idxVar+PROPS_VAR] == 0){
        vars[idxVar+PROPS_VAR] = 1;
        columna += 2;
        printVariableByte(idxVar+PROPS_VAR);
        columna -= 2;
      }
      else if(vars[idxVar] == vars[idxVar+2] && vars[idxVar+PROPS_VAR] > 0){
        vars[idxVar+PROPS_VAR] = 0;
        columna += 2;
        printVariableByte(idxVar+PROPS_VAR);
        columna -= 2;
      }
    break;
    case 16:
      guardarLongEnEdicioAarray(); // Variables generals int. Guardat a vars i a EEPROM
      guardarVariable(idxVar, vars[idxVar], true, vars[idxVar+PROPS_VAR], true, vars[idxVar+PROPS_VAR*2]);
    break;
    case 255:
      // Nothing to do
    break;
  }
  aModeMoures();
}

void encoderPitjarVariableBinaria(int idx1, int idx2, boolean calGuardar){
  idxVar = idx1;
  if(vars[idxVar] == 0){
    vars[idxVar] = 1;  
  }
  else{
    vars[idxVar] = 0;
  }
  noBlincar();
  printVariableBinaria(idx2+vars[idxVar]);
  blinkCoordenades();
  if(calGuardar){
    setVariable();  
  }
}

void encoderPitjarAPantalla(byte x){
  pantalla = x;
  firstIn = true;
}

void encoderPitjarMagnetisme(){
  if(!platMagneticOn){
    aturarParpadeigBotoDret();
    magnetitzar();
    printMagDesmag(1);
    lcd.print("AAA");
    noBlincar();
    delay(2000);
    initParpadeigBotoDret();
  }
  else{
    comprovarDesactivarPlat = true;
    gestioTemps = millis() + 1500UL;
  }
}

void establirValorIlimitsInt(){
  intEnEdicio = vars[idxVar] + vars[idxVar+PROPS_VAR]*256;
  intEnEdicio_limitLow = vars[idxVar+1] + vars[idxVar+2]*256;
  intEnEdicio_limitHigh = vars[idxVar+PROPS_VAR+1] + vars[idxVar+PROPS_VAR+2]*256;  
}

void establirValorIlimitsLong(){
  longEnEdicio = vars[idxVar] + vars[idxVar+PROPS_VAR]*256L + vars[idxVar+PROPS_VAR*2]*256L*256L;
  intEnEdicio_limitLow = vars[idxVar+1] + vars[idxVar+2]*256L;
  // Aixo fa el 983040 + 16896 + 100 = 1.000.036
  longEnEdicio_limitHigh = intEnEdicio_limitLow + (vars[idxVar+PROPS_VAR+1] + vars[idxVar+PROPS_VAR+2]*256L)*256L + (vars[idxVar+PROPS_VAR*2+1] + vars[idxVar+PROPS_VAR*2+2]*256L)*256L*256L;
}

void guardarIntEnEdicioAarray(){
  intAbytes(intEnEdicio);
  vars[idxVar] = myBytes[0];
  vars[idxVar+PROPS_VAR] = myBytes[1]; 
}

void guardarLongEnEdicioAarray(){
  longAbytes(longEnEdicio);
  vars[idxVar] = myBytes[0];
  vars[idxVar+PROPS_VAR] = myBytes[1];
  vars[idxVar+PROPS_VAR*2] = myBytes[2];
}

void encoderPitjarGuardarPrograma(){
  guardarPrograma();
  noBlincar();
  printGuardarPrograma();
  delay(2000);
  printProgramaPantalla3();
  printProgramaPantalla3Valors();
  blinkCoordenades();
}


/*
 ************************************************************************************
 ************************************************************************************
 * ENCODER DRETA
 ************************************************************************************
 ************************************************************************************
 */

 
void encoderDreta(byte option){
  if(valorEncoderNew < valorEncoderOld && !pitjat){
    switch(option){
      case 160:
        encoderDretaSumaFilera();
      break;
      case 161:
        encoderDretaAPantalla(1,0,0);
      break;
      case 162:
        encoderDretaSumaColumna();
        vars[idxDigitCodiActual] = digitsCodiActual[columna-28];
      break;
      case 163:
        encoderEditVariableByte(1);
      break;
      case 164:
        encoderEditVariableInt(1);
      break;
      case 165:
        encoderEditVariableByte(1);
        actualitzarVelocitatAutomatic();
      break;
      case 166:
        encoderEditVariableBinaria(1);
      break;
      case 167:
        encoderEditVariableSigned(1);
      break;
      case 168:
        if(!leftPitjat && !rightPitjat){
          if(funcioRotatoriManual == 0){
            molaHdivisorAHmanual(&eixZposActual, vars[idxStepsManualZ]);
          }
          else if(funcioRotatoriManual == 1){
            molaAHdivisorHmanual(&eixAposActual, STEPS_EIXA, RECORREGUT_A);
          }
          else{
            molaAHdivisorHmanual(&eixYposActual, vars[idxStepsManualY], vars[idxRecorregutY_low] + vars[idxRecorregutY_high]*256L + vars[idxRecorregutY_high2]*256L*256L);
          }
        }
      break;
      case 169:
        encoderDretaAPantalla(1,25,0);
      break;
      case 170:
        if(vars[idxActivarQuartEix] == 0){
          // Pantalla de configuracions. Valor 4 (2+2)
          encoderDretaAPantalla(2,0,0);
        }
        else{
          // Tercera pantalla de programes (9-12). Valor 3 (2+1)
          encoderDretaAPantalla(1,0,0);
        }
      break;
      case 171:
        encoderEditVariableLong(1);
      break;
      case 255:
        // Casos sense efectes
      break;
      
      default:
        // TOTS ELS CASOS DE NAVEGACIO. OPTION = fileraColumna. 0-39, 100-139, 200-239, 300-339
        convertirCaseAposCursor(option);
      break; 
    }
    valorEncoderOld = valorEncoderNew;
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * ENCODER DRETA SUBFUNCIONS
 ************************************************************************************
 ************************************************************************************
 */


void encoderDretaSumaFilera(){
  filera++;
  blinkCoordenades();
}

void encoderDretaAPantalla(byte sumant, byte c, byte f){
  pantalla += sumant;
  setCoordenades(c,f);
  firstIn = true;            
}

void encoderDretaSumaColumna(){
  columna++;
  blinkCoordenades();  
}


/*
 ************************************************************************************
 ************************************************************************************
 * ENCODER ESQUERRA
 ************************************************************************************
 ************************************************************************************
 */


void encoderEsquerra(byte option){
  if(valorEncoderNew > valorEncoderOld && !pitjat){
    switch(option){
      case 160:
        encoderEsquerraRestaFilera();
      break;
      case 161:
        encoderEsquerraAPantalla(1,0,3);
      break;  
      case 162:
        encoderEsquerraRestaColumna();
        vars[idxDigitCodiActual] = digitsCodiActual[columna-28];
      break;
      case 163:
        encoderEditVariableByte(-1);
      break;
      case 164:
        encoderEditVariableInt(-1);
      break;
      case 165:
        encoderEditVariableByte(-1);
        actualitzarVelocitatAutomatic();
      break;
      case 166:
        encoderEditVariableBinaria(-1);
      break;
      case 167:
        encoderEditVariableSigned(-1);
      break;
      case 168:
        if(!leftPitjat && !rightPitjat){
          if(funcioRotatoriManual == 0){
            molaAHdivisorHmanual(&eixZposActual, vars[idxStepsManualZ], vars[idxRecorregutZ_low] + vars[idxRecorregutZ_high]*256L + vars[idxRecorregutZ_high2]*256L*256L);       
          }
          else if(funcioRotatoriManual == 1){
            molaHdivisorAHmanual(&eixAposActual, STEPS_EIXA);
          }
          else{
            molaHdivisorAHmanual(&eixYposActual, vars[idxStepsManualY]);
          }
        }
      break;
      case 169:
        cursorAlInici = false;
        encoderEsquerraAPantalla(1,36,3);
      break;
      case 170:
        cursorAlInici = false;
        encoderEsquerraAPantalla(1,16,3);
      break;
      case 171:
        if(vars[idxActivarQuartEix] == 0){
          // Pantalla 2 de programes (5-8). Valor 2 (4-2)
          encoderEsquerraAPantalla(2,0,3);
        }
        else{
          // Pantalla 3 de programes (9-12). Valor 3 (4-1)
          encoderEsquerraAPantalla(1,0,3);
        }
      break;
      case 172:
        encoderEditVariableLong(-1);
      break;
      case 255:
        // Casos sense efectes
      break;
      
      default:
        // TOTS ELS CASOS DE NAVEGACIO. OPTION = fileraColumna. 0-39, 40-79, 80-119, 120-159
        convertirCaseAposCursor(option);
      break;
    }
    valorEncoderOld = valorEncoderNew;
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * ENCODER ESQUERRA SUBFUNCIONS
 ************************************************************************************
 ************************************************************************************
 */


void encoderEsquerraRestaFilera(){
  filera--;
  blinkCoordenades();
}

void encoderEsquerraAPantalla(byte restant, byte c, byte f){
  pantalla -= restant;
  firstIn = true;
  setCoordenades(c,f);
}

void encoderEsquerraRestaColumna(){
  columna--;
  blinkCoordenades(); 
}


/*
 ************************************************************************************
 ************************************************************************************
 * ENCODER DRETA I ESQUERRA SUBFUNCIONS COMUNES
 ************************************************************************************
 ************************************************************************************
 */


void convertirCaseAposCursor(int option){
  if(option < 40){
    // Opcions de navegacio Filera 0 (0-39)
    filera = 0;
    columna = option;  
  }
  else if(option < 80){
    filera = 1;
    columna = option - 40;
  }
  else if(option < 120){
    filera = 2;
    columna = option - 80; 
  }
  else if(option < 160){
    filera = 3;
    columna = option - 120;
  }
  if(configNivell == 11){
    cAux = columna;
    fAux = filera;  
  }
  blinkCoordenades();
}

void encoderEditVariableByte(int8_t signe){
  if(incrementDinsElsLimits(increment*signe)){
    vars[idxVar] += increment*signe;
    if(idxVar == idxStepsManualY){
      vars[idxVar] *= 5;  
    }
    printVariableByte(idxVar);
    if(idxVar == idxStepsManualY){
      vars[idxVar] /= 5;  
    }
  }
}

void encoderEditVariableSigned(int8_t signe){
  int8_t valor = vars[idxVar];
  int8_t minim = vars[idxVar+1];
  int8_t maxim = vars[idxVar+2];
  if(valor+increment*signe >= minim && valor+increment*signe <= maxim){
    valor += increment*signe;
    vars[idxVar] = valor;
    printVariableSigned(idxVar);
  }
}

void encoderEditVariableInt(int8_t signe){
  if(intEnEdicio+increment*signe >= intEnEdicio_limitLow && intEnEdicio+increment*signe <= intEnEdicio_limitHigh){
    intEnEdicio += increment*signe;
    printVariableInt(idxVar);
  } 
}

void encoderEditVariableLong(int8_t signe){
  if(longEnEdicio+increment*signe >= intEnEdicio_limitLow && longEnEdicio+increment*signe <= longEnEdicio_limitHigh){
    longEnEdicio += increment*signe;
    if(idxVar == idxRecorregutY_low){
      longEnEdicio *= 5;  
    }
    printVariableLong(idxVar);
    if(idxVar == idxRecorregutY_low){
      longEnEdicio /= 5;  
    }
  }
}

void encoderEditVariableBinaria(int8_t signe){
  if(incrementDinsElsLimits(increment*signe)){
    vars[idxVar] += increment*signe;
    printVariableBinaria(vars[idxVar+4]+vars[idxVar]);
    posCursor(columna, filera);
  }
}

boolean incrementDinsElsLimits(int8_t increment){
  return vars[idxVar]+increment >= vars[idxVar+1] && vars[idxVar]+increment <= vars[idxVar+2]; 
}


/*
 ************************************************************************************
 ************************************************************************************
 * RIGHT BUTTON
 ************************************************************************************
 ************************************************************************************
 */


void rightButton(byte option){
  if(llegirPinMux16(botoRight) == LOW && !rightPitjat){
    switch(option){
      case 0:
        firstRight = true;
      break;
      case 1:
        if(llegirPinMux16(botoLeft) == HIGH){
          if(llegirPinMux16(FCx2) == HIGH && !finalDretDetectat){
            if(!carroMoventse){
              comprovarMovimentCarroDretManual = true;
              direccioCarroManual = 0;
              moureCarroManual();
            }
          }
        }
      break;
      case 2:
        if(vars[idxActivarMagnetisme] == 1 && !platMagneticOn){
          noBlincar();
          printErrorPlat(1);
          delay(3000);
          printAutomatic();
          printAutomaticValors();
          printOpcionsBotonsAutomatic(0);
          printMagDesmag(0);
          printDesUltimCompleto();
          printAcaUltimCompleto();
          printLapeadoCompleto();
        }
        else{
          posicionarDivisor();
          platAzero = false;
          iniciPerBotoIniciar = true;
          if(!programaQuartEix()){
            nextEtapaAutomatic();
          }
          else{
            pasDentAcumulat = 0L;
            nextEtapaAutomatic2(); 
          }
          firstIn = true;
        }
      break;
      case 3:
        rependreAutomatic();
      break;
      case 4:
        if((carroMoventse || comprovarMovEixY) && !aturaEnEspera){ // Si esta en fase de baixar mola no volem pausar, ni si ja hem demanat aturar
          pausaEnEspera = true;
        }
      break;
      case 5:
        // Moure eix X amb accelStepper
        if(llegirPinMux16(botoLeft) == HIGH){
          aplicarMoveRunXmanual(-50000, botoRight, FCx2);
        }
      break;
    }
    rightPitjat = true;
  }
  else if(llegirPinMux16(botoRight) == HIGH && rightPitjat){
    switch(option){
      case 0:
        firstRight = false;
        comprovarMenuSecret = false;
        gestioTemps = 0UL;
      break;
      case 1:
        if(carroMoventse && direccioCarroManual == 0){
          aturarCarro(stepsEixXmanual);
          comprovarMovimentCarroDretManual = false;
        }
      break;
    }
    rightPitjat = false;
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * LEFT BUTTON
 ************************************************************************************
 ************************************************************************************
 */


void leftButton(byte option){
  if(llegirPinMux16(botoLeft) == LOW && !leftPitjat){
    switch(option){
      case 0:
        if(llegirPinMux16(botoRight) == HIGH){
          if(llegirPinMux16(FCx1) == HIGH && !finalEsquerreDetectat){
            if(!carroMoventse){
              comprovarMovimentCarroEsquerreManual = true;
              direccioCarroManual = 1;
              moureCarroManual();
            }
          }
        }
      break;
      case 1:
        pausat = false;
        /*
        if(!programaQuartEix()){
          aturarGirMola();
        }
        else{
          aturarGirMola2();
        }
        */
        if(programaQuartEix()){
          // Utilitzat per comptar el temps que porta 
          tempsConfirmarAutoOsalir = 0UL;
        }
        escriureOut16(aiguaMotor, LOW);
        aturarParpadeigBotoDret();
        llumPerMagnetisme();
        dirEixYauto = 1;
        encoderPitjarAConfigNivell(4);
      break;
      case 2:
        if((carroMoventse || comprovarMovEixY) && !pausaEnEspera){ // Si esta en fase de baixar mola no volem aturar, ni si ja hem demanat que pausi
          aturaEnEspera = true;
        }
      break;
      case 3:
        if(!platMagneticOn){
          magnetitzar();
          escriureOut16(llumBotoLeft, HIGH);
        } 
        else{
          comprovarDesactivarPlat = true;
          gestioTemps = millis() + 1500UL;
        }
      break;
      case 4:
        firstLeft = true;
      break;
      case 5:
        aturarMedicio();
        if(iniciantEsmolatAngle2){
          iniciantEsmolatAngle2 = false;
          tempsCarroDeFCx1AFCx2 = 0UL;
          tempsConfirmarAutoOsalir = 0UL;
        }
        gestioTemps = 0UL;
        encoderPitjarAConfigNivell(4);
        llumPerMagnetisme();
      break;
      case 6:
        // Moure eix X amb accelStepper
        if(llegirPinMux16(botoRight) == HIGH){
          aplicarMoveRunXmanual(50000, botoLeft, FCx1);
        }
      break;
    }
    leftPitjat = true;
  }
  else if(llegirPinMux16(botoLeft) == HIGH && leftPitjat){
    switch(option){
      case 0:
        if(carroMoventse && direccioCarroManual == 1){
          aturarCarro(stepsEixXmanual);
          comprovarMovimentCarroEsquerreManual = false;
        }
      break;
      case 4:
        firstLeft = false;
        comprovarMenuSecret = false;
        gestioTemps = 0UL;
      break;
    }
    leftPitjat = false;
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * SISTEMA ARRENCADA FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */


void comRTC(){
  // Funcions previes necesaries per a obtenir valors de data i hora correctes amb els getters
  rtc.formatDate();
  rtc.formatTime();
  vars[idxAny] = rtc.getYear();
  vars[idxMes] = rtc.getMonth();
  vars[idxDia] = rtc.getDay();
  vars[idxHora] = rtc.getHour();
  vars[idxMinut] = rtc.getMinute();
  vars[idxSegon] = rtc.getSecond();
  
  idxVar = idxAny;
  compararDatesRecursivament();
  if(respostaRTC == 0){
    //avisos = true;
  }
}

void actualitzarRellotge(){
  for(idxVar == idxAny; idxVar <= idxSegon; idxVar += PROPS_VAR){
    setVariable();  
  }
}

void initComFA(){
  Wire.beginTransmission(2); // transmit to device #2
  Wire.write(3);             // sends one byte  
  Wire.endTransmission();    // stop transmitting
  delay(50);
  indexRespostaFA = 0;
  gestioTemps = millis() + 2000UL;
  Wire.requestFrom(2, 3);    // request 1 byte from slave device #2
}

void gestioPunts(){
  if(gestioTemps2 < millis()){
    if(nPunts < 3){
      printPunts();
      nPunts++;
      gestioTemps2 = millis() + 800UL;
    }
    else{
      printResultatArrencada();
      if(!errors){
        fase = 1;
      }      
    }
  }  
}

void checkComFA(){  
  if(gestioTemps > millis()){
    if(Wire.available()){
      respostaFA[indexRespostaFA] = Wire.read();
      indexRespostaFA++;
    }  
  }  
}

void finalitzarCom(){
  if(gestioTemps < millis()){
    if(respostaFA[2] == 233){
      // errors = true;
    }
    else if(respostaFA[2] == 0){
      // avisos = true;
    }
  }
}

void menuSecret(){
  if(firstRight && firstLeft){
    gestioTemps = millis() + 5000UL;
    comprovarMenuSecret = true;
    firstRight = false;
    firstLeft = false;
  }
  if(comprovarMenuSecret){
    if(gestioTemps < millis()){
      encoderPitjarAConfigNivell(2);
    }
  }
}

void compararDatesRecursivament(){
  if(vars[idxVar] < getVariable()){
    respostaRTC = 0;  // Menor hora que en l'ultim apagat de maquina. Error de rellotge
  }
  else if(vars[idxVar] == getVariable()){
    if(idxVar == idxSegon){
      respostaRTC = 0; // Mateixa hora que en l'ultim apagat de maquina. Error de rellotge
    }
    else{
      idxVar += PROPS_VAR;
      compararDatesRecursivament(); // Encara no sabem si l'hora rtc actual es menor, igual o major
    }
  }
  else{
    // Major hora que en l'ultim apagat de maquina. Rellotge correcte
    respostaRTC = 1;
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * MENU INICIAL FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 

void recuperarPrograma(){
  pos = vars[idxPrograma] * N_VARS_PROGRAMA;
  for(byte k = 0; k < N_VARS_PROGRAMA; k++){
    idxVar = k*PROPS_VAR;
    vars[idxVar] = getVariable();
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * CONFIGURACIO DE PROGRAMES FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */


void guardarPrograma(){
  //pos = programa * N_VARS_PROGRAMA;
  for(byte k = 0; k < N_VARS_PROGRAMA; k++){
    idxVar = k*PROPS_VAR;
    setVariable();
  } 
}

void editarSegonAngle(){
  if(idxVar >= idxAngle1 && idxVar <= idxAngle2Dec){
    if(!angle2MajorAngle1()){
      if(vars[idxAngle1]*10 + vars[idxAngle1Dec] > 0){
        vars[idxAngle2] = 0;
        vars[idxAngle2Dec] = 0;
        colAux = columna;
        columna = 22;
        printVariableByte(idxAngle2);
        columna = 24;
        printVariableByte(idxAngle2Dec);
        columna = colAux;
      }
    } 
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * VALORS DE PROCES FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 

void comprovarCodiEntrat(){
  if(vars[idxCodiFinalIntroduit] == 0){
    byte resposta = codiSeguretatCorrecte();
    if(resposta == 2){
      guardarVariable(idxCodiFinalIntroduit, 1, false, 0, false, 0);
    }
    else if(resposta == 1){
      if(vars[idxCodiProv1Introduit] == 0){
        guardarVariable(idxCodiProv1Introduit, 1, false, 0, false, 0);
      }
      else if(vars[idxCodiProv2Introduit] == 0){
        guardarVariable(idxCodiProv2Introduit, 1, false, 0, false, 0);
      }
      guardarVariable(idxHoresCicle_low, 0, true, 0, false, 0);
      guardarVariable(idxMinutsCicleRestants, 0, false, 0, false, 0);
    }
  }
}

byte codiSeguretatCorrecte(){
  unsigned long codiActual = 0;
  unsigned long unitat = 1;
  unsigned long provisionalActual;
  byte resposta = 0;
  for(int8_t i=8; i>=INICI_DIGIT_CODIFINAL; i--){   // Comprovant codi final, de DIGITS_CODI_FINAL digits
    codiActual += digitsCodiActual[i] * unitat;
    unitat *= 10; 
  }
  if(codiActual == CODI_FINAL){
    resposta = 2;
  }
  else{
    codiActual = 0;
    unitat = 1;
    if(vars[idxCodiProv1Introduit] == 0){
      provisionalActual = CODI_PROVISIONAL;
    }
    else if(vars[idxCodiProv2Introduit] == 0){
      provisionalActual = CODI_PROVISIONAL2;  
    }
    for(int8_t i=8; i >= 0; i--){ // Comprovant codi providional, de 9 digits
      codiActual += digitsCodiActual[i] * unitat;
      unitat *= 10; 
    }
    if(codiActual == provisionalActual){
      resposta = 1;
    }
  }
  return resposta;
}


/*
 ************************************************************************************
 ************************************************************************************
 * ELIMINACIO DE PROGRAMES FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 

void eliminarPrograma(){
  for(byte k = 0; k < N_VARS_PROGRAMA; k++){
    idxVar = k*PROPS_VAR;
    vars[idxVar] = vars[idxVar+1] + vars[idxVar+2]*256;
    setVariable();
    if(esIntDePrograma()){
      idxVar += PROPS_VAR;
      vars[idxVar] = 0;
      setVariable();
      k++;
    }
  } 
}

void eliminarTotsProgrames(){
  for(byte i = 0; i < nProgs; i++){
    pos = i * N_VARS_PROGRAMA;
    eliminarPrograma();
  }  
}


/*
 ************************************************************************************
 ************************************************************************************
 * MOVIMENTS MANUALS FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 

void checkTempsRotatoriPitjat(){
  if(gestioTemps > 0){
    if(millis() > gestioTemps){
      encoderPitjarAConfigNivell(3);
      gestioTemps = 0UL;
      funcioRotatoriManual = 0;
    }
  }
}

void comprovacionsMovCarroDretManual(){
  if(comprovarMovimentCarroDretManual){
    /* CAS DELICAT 1: estem al final de l'esquerra i ordenem mourens a la dreta
     finalEsquerreDetectat estara activat, i es quan ens movem a la dreta quan cal gestionarlo
     Si esta actiu, a la que detectem que estem al limit esquerra (FCx1 on), el desactivem    
     */
    if(finalEsquerreDetectat){
      if(llegirPinMux16(FCx1) == HIGH){
        finalEsquerreDetectat = false;
      }
    }
    else{
      if(llegirPinMux16(FCx1) == LOW){
        finalEsquerreDetectat = true;  
      }
    }
    /* CAS DELICAT 2: ens estem movent cap a la dreta i arribem al final de la dreta
     activem el finalDretDetectat i aturem el motor amb tone
     */
     if(llegirPinMux16(FCx2) == LOW){
      finalDretDetectat = true;
      fcDretAssolit = 1;
      aturarCarro(stepsEixXmanual);
      comprovarMovimentCarroDretManual = false;
    }
  }
}

void comprovacionsMovCarroEsquerreManual(){
  if(comprovarMovimentCarroEsquerreManual){
    //eixX.run();
    /* CAS DELICAT 1: estem al final dret i ordenem mourens a l'esquerra
     finalDretDetectat estara activat, i es quan ens movem a l'esquerra quan cal gestionarlo
     Si esta actiu, a la que detectem que estem al limit dret (FCx2 on), el desactivem    
     */
    if(finalDretDetectat){
      if(llegirPinMux16(FCx2) == HIGH){
      //if(!comprovaFinalCarrera(FCx2)){
        finalDretDetectat = false;
      }
    }
    else{
      if(llegirPinMux16(FCx2) == LOW){
      //if(comprovaFinalCarrera(FCx2)){
        finalDretDetectat = true;  
      }
    }
    /* CAS DELICAT 2: ens estem movent cap a l'esquerra i arribem al final de l'esquerra
     activem el finalEsquerraDetectat i aturem el motor amb tone
     */
    if(llegirPinMux16(FCx1) == LOW){
    //if(comprovaFinalCarrera(FCx1)){
      finalEsquerreDetectat = true;
      fcEsquerreAssolit = 1;
      aturarCarro(stepsEixXmanual);
      comprovarMovimentCarroEsquerreManual = false;
    }
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * PREVIS AUTOMATIC FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 
 
void establirIntsAutomatics(){
  // Desbast decimal
  desbastDec = vars[idxDesbastDec_low] + vars[idxDesbastDec_high]*256;
  // Desbast divisor decimal
  desbastDivDec = vars[idxDesbastDivDec_low] + vars[idxDesbastDivDec_high]*256;
  // Acabat decimal
  acabatDec = vars[idxAcabatDec_low] + vars[idxAcabatDec_high]*256;
  if(programaQuartEix()){
    nDents = vars[idxNdents_low] + vars[idxNdents_high]*256;
  }
}

void establirMovimentsAutomatics(){
  if(!programaQuartEix()){
    existeixAngle2 = esmolarAngle2();
  }
  else{
    existeixAngle2 = false;  
  }
  if(vars[idxDesbast] != 0 || desbastDec != 0){
    arrayEsmolat[0] = 1;
    hiHaEtapes = true;
  }
  if(acabatDec != 0){
    if(existeixAngle2){
      arrayEsmolat[3] = 1;
    }
    else{
      arrayEsmolat[1] = 1;
    }
    hiHaEtapes = true;
  }
  if(vars[idxLapejat] != 0){
    arrayEsmolat[2] = 1;
    if(arrayEsmolat[3] == 1){
      arrayEsmolat[4] = 1;
    }
    hiHaEtapes = true;
  }
  for(byte i=0; i<3; i++){
    if(arrayEsmolat[i] == 1){
      existeixAngle1 = true;
      i = 3;  
    }     
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * MOVIMENTS MANUALS I PREVIS AUTOMATIC SUBFUNCIONS COMUNES 
 ************************************************************************************
 ************************************************************************************
 */


void manualsIprevisConfigInicialComuna(){
  encendreLlumBotons();
  printManualsIprevis();
}


/*
 ************************************************************************************
 ************************************************************************************
 * ESMOLAT AUTOMATIC FUNCIONS 
 ************************************************************************************
 ************************************************************************************
 */


/* 
 *** INICIALITZACIO AUTOMATICS ***
 */

void initAutomatic(){
  // Prints
  noBlincar();
  printAutomatic();
  printAutomaticValors();
  printOpcionsBotonsAutomatic(2);
  
  // Variables
  movimentsAutomatics = 1;
  carregaEixZtotal = 0;
  pasDentAcumulat = 0L;
  puntDeCanvi = true;
  
  // Accions
  activarTempsCicle();
  if(iniciPerBotoIniciar && !programaQuartEix()){
    if(cAux == 32){
      columna = cAux;
      filera = fAux;
    }
    else{
      columna = 32;
      filera = fAux;  
    }
    if(mode == 1){
      mode = 0;  
    } 
  }
  blinkSegonsMode();
  /*
  if(iniciPerBotoIniciar){
    if(!programaQuartEix()){
      activarGirMola();
    }
    else{
      activarGirMola2();
    }
  }
  */
  aturarParpadeigBotoDret();
  encendreLlumBotons();
  if(vars[idxMedicio] == 0 || iniciPerBotoIniciar){
    escriureOut16(aiguaMotor, HIGH);
    // Per donar temps que l'aigua arribi a la zona de la mola, per no es crema la cutxilla
    if(iniciPerBotoIniciar){
      if(programaQuartEix()){
        activarGirMola2();
        if(tempsConfirmarAutoOsalir > 0){
          unsigned long t = millis() - tempsConfirmarAutoOsalir;
          if(t < 5000){
            delayEnWhile(t*2,0);
          }
          else{
            delayEnWhile(10000,0);
          }
          tempsConfirmarAutoOsalir = 0UL;
        }
        else{
          // Han passat ja 4 segons i per tant la mola s'ha aturat del tot. Apliquem els 10 segons
          delayEnWhile(10000,0);  
        }
      }
      else{
        activarGirMola();
        delayEnWhile(3000,0); 
      }
    }
    else{
      if(gestioTemps > millis()){
        // Mola quart eix necessita 10 segons per arribar a maxima velocitat
        if(gestioTemps-millis() < 3000){
          delayEnWhile(3000,0);  
        }
        else{
          delayEnWhile(gestioTemps-millis(),0);  
        }
      }
      else{
        delayEnWhile(3000,0);
      }
      gestioTemps = 0UL;
    }
    iniciPerBotoIniciar = false;
  }
  ultimValorDesbaste = 0;
  ultimValorAcabado = 0;
}

void initAutomatic2(){
  carregaEixZtotal = 0;
  iniciantEsmolatAngle2 = false; 
  tempsCarroDeFCx1AFCx2 = 0UL;
  tempsConfirmarAutoOsalir = 0UL;
  encendreLlumBotons();
  printOpcionsBotonsAutomatic(2);
  if(aplicarMenysVelPrimeraPasada){
    printVelocitatNova();
  }
  blinkSegonsMode();
}

/* 
 *** INICIALITZACIO ETAPES D'ESMOLAT ***
 */

void initDesbaste(){
  idxArrayEsmolat = 0;
  movimentAutoActual = 1;
  initDesbasteVariables();   
}

void initDesbasteVariables(){
  count = 0;
  total = vars[idxDesbast] * 1000 + desbastDec;
  divisor = desbastDivDec;
  nBaixades = total / divisor;
  augment = 100/(nBaixades+1);
  carregaEixZetapa = 0;
  aux = nBaixades * divisor;
  ultimaBaixadaResteValor = total - aux;  // Hauria de donar sempre un valor menor a div o zero, sino es que nBaixades+1
  if(!programaQuartEix()){
    idxCurrentVel = idxDesbastVel;
    idxVarAux = idxDesbastVel;
    actualitzarVelocitat(); // steps reals a la funcio tone
    velAux = vars[idxDesbastVel];
  }
  else{
    stepsEixYautomatic = VEL_EIXY_CONSTANT_AUTO * (vars[idxDesbastVel]-1);
  }
}

void initAcabado(){
  idxArrayEsmolat = 1;
  movimentAutoActual = 2;
  initAcabadoVariables();   
}

void initAcabado2(){
  idxArrayEsmolat = 3;  
  movimentAutoActual = 2;
  initAcabadoVariables();      
}

void initAcabadoVariables(){
  count = 0;
  total = acabatDec;
  divisor = vars[idxAcabatDivDec];
  nBaixades = total / divisor;
  augment = 100/(nBaixades+1);
  carregaEixZetapa = 0;
  aux = nBaixades * divisor;
  ultimaBaixadaResteValor = total - aux; // Hauria de donar sempre un valor menor a div o zero, sino es que nBaixades+1
  if(!programaQuartEix()){
    idxCurrentVel = idxAcabatVel;
    idxVarAux = idxAcabatVel;
    actualitzarVelocitat(); // steps reals a la funcio tone
    velAux = vars[idxAcabatVel];
  }
  else{
    stepsEixYautomatic = VEL_EIXY_CONSTANT_AUTO * (vars[idxAcabatVel]-1);
  }
}

void initLapeado(){
  idxArrayEsmolat = 2;
  movimentAutoActual = 3;
  initLapeadoVariables();   
}

void initLapeado2(){
  idxArrayEsmolat = 4;
  movimentAutoActual = 3;
  initLapeadoVariables();  
}

void initLapeadoVariables(){
  count = 0;
  nBaixades = vars[idxLapejat];
  if(!programaQuartEix()){
    idxCurrentVel = idxLapejatVel;
    idxVarAux = idxLapejatVel;
    actualitzarVelocitat(); // steps reals a la funcio tone
    velAux = vars[idxLapejatVel];
  }
  else{
    stepsEixYautomatic = VEL_EIXY_CONSTANT_AUTO * (vars[idxLapejatVel]-1);
  }
}

/* 
 *** INICIALITZACIO DIRECCIONS EIXOS ***
 */

void inicialitzarDireccioCarro(){
  if(llegirPinMux16(FCx1) == LOW || finalEsquerreDetectat){
    finalEsquerreDetectat = true;
    direccioCarroAuto = 0; // a la dreta
    if(!programaQuartEix()){
      if( vars[idxBaixada] == 0 || vars[idxBaixada] == 2 ){
        // Tot i ser la primera etapa de l'esmolat, ja estem en un extrem. No és necessari
        countEtapaIniciat = true;
      }
      else{
        countEtapaIniciat = false; 
      }
      /*
      if(primerLapeado){
        primerFCenLapeado = true;
      }
      else{
        if( vars[idxBaixada] == 2 || vars[idxBaixada] == 0 ){
          // Tot i ser la primera etapa de l'esmolat, ja estem en un extrem. No és necessari
          primeraBaixadaMola = true;
        }
        else{
          primeraBaixadaMola = false; 
        }
      }
      */
    }
  }
  else if(llegirPinMux16(FCx2) == LOW || finalDretDetectat){
    finalDretDetectat = true;
    direccioCarroAuto = 1; // a l'esquerra
    if(!programaQuartEix()){
      if( vars[idxBaixada] == 1 || vars[idxBaixada] == 2 ){
        // Tot i ser la primera etapa de l'esmolat, ja estem en un extrem. No és necessari
        countEtapaIniciat = true;
      }
      else{
        countEtapaIniciat = false; 
      }
      /*
      if(primerLapeado){
        primerFCenLapeado = true;
      }
      else{
        if( vars[idxBaixada] == 2 || vars[idxBaixada] == 1 ){
          primeraBaixadaMola = true;
        }
        else{
          primeraBaixadaMola = false;  
        }
      }
      */
    }
  }
  else{
    finalEsquerreDetectat = false;
    finalDretDetectat = false;
    if(programaQuartEix()){
      direccioCarroAuto = 0; // a la dreta    
    }
    else{
      if(vars[idxBaixada] == 1){
        direccioCarroAuto = 0; // cap final detectat i baixada a la dreta, movem a la dreta
      }
      else{
        direccioCarroAuto = 1; // cap final detectat i baixada a l'esquerra, movem a l'esquerra
      }
      countEtapaIniciat = false;
      /*
      if(primerLapeado){
        primerFCenLapeado = false;
      }
      else{
        // No estem a cap extrem, per tant es necessari primer anar a trobar un extrem. Variable a false
        primeraBaixadaMola = false;  
      }
      */
    }
  }
}

/* 
 *** COMPROVACIONS FINALS DE CARRERA I ESMOLAT ***
 */

void comprovacionsMovCarroDretAuto(){
  if(comprovarMovimentCarroDretAuto){
    /* CAS DELICAT 1: estem al final de l'esquerra i ordenem mourens a la dreta
     finalEsquerreDetectat estara activat, i es quan ens movem a la dreta quan cal gestionarlo
     Si esta actiu, a la que detectem que estem al limit esquerra (FCx1 on), el desactivem    
     */
    if(finalEsquerreDetectat){
      if(llegirPinMux16(FCx1) == HIGH){
        finalEsquerreDetectat = false;
      }
    }
    else{
      if(llegirPinMux16(FCx1) == LOW){
        finalEsquerreDetectat = true;
      }
    }
    /* CAS DELICAT 2: ens estem movent cap a la dreta i arribem al final de la dreta
     activem el finalDretDetectat i aturem el motor amb tone
     */
    if(llegirPinMux16(FCx2) == LOW){
      finalDretDetectat = true;
      if(desplasantCarroPrevi){
        aturarCarro(40 + (vars[idxIncrementVel_low]+vars[idxIncrementVel_high]*256) * (vars[idxDesbastVel]-1));
        delayEnWhile(400, 2);
        comprovarMovimentCarroDretAuto = false;
        finalitzarDesplasarCarro(); 
      }
      else if(rectificantTrobarFC){
        delayEnWhile(500,0);
        aturarCarro(STEPS_RECTIFICAT_TROBARFC);
        rectificantTrobarFC = false;
        comprovarMovimentCarroDretAuto = false;
        finalitzarModeAutomatic();
      } 
      else if(iniciantEsmolatAngle2){
        comprovarMovimentCarroDretAuto = false;
        nextMoveFromIniciantEsmolatAngle2();
      }   
      else{
        comprovarMovimentCarroDretAuto = false;
        aturarCarro(stepsEixXautomatic);
        delayEnWhile(400, 1);
        puntDeCanvi = esPuntDeCanvi();
        comprovacionsEsmolat();
      }
    }
  }
}

void comprovacionsMovCarroEsquerreAuto(){
  if(comprovarMovimentCarroEsquerreAuto){
    /* CAS DELICAT 1: estem al final dret i ordenem mourens a l'esquerra
     finalDretDetectat estara activat, i es quan ens movem a l'esquerra quan cal gestionarlo
     Si esta actiu, a la que detectem que estem al limit dret (FCx2 on), el desactivem    
     */
    if(finalDretDetectat){
      if(llegirPinMux16(FCx2) == HIGH){
      //if(!comprovaFinalCarrera(FCx2)){
        finalDretDetectat = false;
      }
    }
    else{
      if(llegirPinMux16(FCx2) == LOW){
      //if(comprovaFinalCarrera(FCx2)){
        finalDretDetectat = true;  
      }
    }
    /* CAS DELICAT 2: ens estem movent cap a l'esquerra i arribem al final de l'esquerra
     activem el finalEsquerraDetectat i aturem el motor amb tone
     */
    if(llegirPinMux16(FCx1) == LOW){
    //if(comprovaFinalCarrera(FCx1)){
      finalEsquerreDetectat = true;
      if(desplasantCarroPrevi){
        aturarCarro(40 + (vars[idxIncrementVel_low]+vars[idxIncrementVel_high]*256) * (vars[idxDesbastVel]-1));  
        delayEnWhile(400, 2);
        comprovarMovimentCarroEsquerreAuto = false;
        finalitzarDesplasarCarro();
      }
      else if(rectificantTrobarFC){
        delayEnWhile(500,0);
        aturarCarro(STEPS_RECTIFICAT_TROBARFC);
        rectificantTrobarFC = false;
        comprovarMovimentCarroEsquerreAuto = false;
        finalitzarModeAutomatic();
      }
      else if(iniciantEsmolatAngle2){
        comprovarMovimentCarroEsquerreAuto = false;
        nextMoveFromIniciantEsmolatAngle2();
      }
      else{
        comprovarMovimentCarroEsquerreAuto = false;
        aturarCarro(stepsEixXautomatic);
        delayEnWhile(400, 1);
        puntDeCanvi = esPuntDeCanvi();
        comprovacionsEsmolat();
      }
    }
  }
}

void comprovacionsEsmolat(){
  if(!pausaEnEspera && !aturaEnEspera){
    if(movimentAutoActual < 3){
      nextMoveFromDesAcaCarro();  
    }
    else if(movimentAutoActual == 3){
      nextMoveFromLapeadoCarro();
    }
  }
  else{
    if(pausaEnEspera){
      pausaEnEspera = false;
      pausarModeAutomatic();  
    }
    else if(aturaEnEspera){
      aturaEnEspera = false;
      //posCursor(6,3);
      //lcd.print("     ");
      aturarModeAutomatic();  
    }
  }
}

void comprovarSiVelActualReduida(){
  if(aplicarMenysVelPrimeraPasada){
    vars[idxCurrentVel] = velAux;
    idxCurrentVel = idxVarAux;
    actualitzarVelocitat();
    printVelocitatNova();
    blinkSegonsMode();
    aplicarMenysVelPrimeraPasada = false;
  }
}

/* 
 *** APLICAR SEGUENT MOVIMENT ***
 */

void nextMoveFromDesAcaCarro(){
  if(puntDeCanvi){
    if(countEtapaIniciat){
      if(mode == 0){
        noBlincar();
      }
      switch(movimentAutoActual){
        case 1:
          ultimValorDesbaste = carregaEixZetapa;
        break;
        case 2:
          ultimValorAcabado = carregaEixZetapa;
        break;
      }
      printDesAcaCompleto();
      blinkSegonsMode();
    }
    if((count == nBaixades && ultimaBaixadaResteValor > 0) || (count < nBaixades)){
      if(!programaQuartEix()){
        if(!countEtapaIniciat){
          countEtapaIniciat = true;
          if(!programaQuartEix()){
            canviDirImoureCarroAuto();
          }
          else{
            canviDirImoureEixYauto();
          }
        }
        else{
          moureMolaAutomatic();
        }
      }
      else{
        moureMolaAutomatic();  
      }
    }
    else if((count == nBaixades && ultimaBaixadaResteValor == 0) || (count > nBaixades)){
      if(!programaQuartEix()){
        nextEtapaAutomatic();
      }
      else{
        nextEtapaAutomatic2();
      }
    }
  }
  else{
    // En aqui el nou esmolat mai entra. per programes >= 8, puntDeCanvi = true sempre
    canviDirImoureCarroAuto(); 
  }
}

void nextMoveFromDesAcaMola(){
  count++;
  // Proxim moviment de carro cap a l'altre sentit
  if(!programaQuartEix()){
    comprovarSiVelActualReduida();         
    canviDirImoureCarroAuto();
  }
  else{
    canviDirImoureEixYauto();
  }
}

void nextMoveFromLapeadoCarro(){
  if(puntDeCanvi){
    if(count < nBaixades-1){ // si nº moviments de carro = 5, count arribaria fins a 4, ja que comença pel 0
      if(!countEtapaIniciat){
        countEtapaIniciat = true;
      }
      else{
        count++; // Amb lapeado el comptador es basa en moviments de carro. Sumem un si escau
        noBlincar();
        printLapeadoCompleto();
        blinkSegonsMode();
      }
      if(!programaQuartEix()){
        comprovarSiVelActualReduida();
        canviDirImoureCarroAuto();
      }
      else{
        delayEnWhile(250,0);
        canviDirImoureEixYauto();
      }
    }
    else{ // count == nBaixades-1
      count++; // Amb lapeado el comptador es basa en moviments de carro. Sumem un si escau
      noBlincar();
      printLapeadoCompleto();
      blinkSegonsMode();
      if(!programaQuartEix()){
        nextEtapaAutomatic();
      }
      else{
        nextEtapaAutomatic2();
      }
    }
  }
  else{
    // En aqui el nou esmolat mai entra. per programes >= 8, puntDeCanvi = true sempre
    canviDirImoureCarroAuto(); 
  }
}

void nextMoveFromIniciantEsmolatAngle2(){
  aturarCarro(stepsEixXautomaticAux);
  delayEnWhile(400, 1);
  //if(tempsConfirmarAutoOsalir > 0){
    // Estem en primera fase: moure el carro per saber quan tarda
    // Calculem el temps que necessita per fer la meitat del recorregut: la meitat del temps per a tot el trajecte
  tempsCarroDeFCx1AFCx2 = ((millis()-tempsConfirmarAutoOsalir) / 2) + millis();
  tempsConfirmarAutoOsalir = millis();
  //}
  apagarLlumBotons();
  if(mode == 0){
    noBlincar();
  }
  printOpcionsBotonsAutomatic(3);
  blinkSegonsMode();
  canviDirImoureCarroAuto();
}

/* 
 *** APLICAR SEGUENT ETAPA ***
 */

void nextEtapaAutomatic(){
  if(mode == 1){
    aModeMoures();  
  }
  switch(idxArrayEsmolat){
    case -1:
      if(arrayEsmolat[0] == 1){ // Desbaste en angle 1
        //posCursor(11,3);
        //lcd.print("a1");
        initDesbaste();
      }
      else{
        if(arrayEsmolat[1] == 1){ // Acabat en angle 1
          //posCursor(11,3);
          //lcd.print("a4");
          initAcabado();
        }
        else{
          if(arrayEsmolat[2] == 1){ // Lapejat en angle 1
            //posCursor(11,3);
            //lcd.print("a2");
            initLapeado();
          }
          else{
            if(arrayEsmolat[3] == 1){ // Acabat en angle 2
              //posCursor(11,3);
              //lcd.print("a3");
              // Primera etapa d'esmolat: apliquem l'angle2 com si fós l'angle1
              initAcabado2();    
            }
            else{
              // No hauria d'arribar aqui, ja que previament hiHeEtapes es false    
            }
          }  
        }
      }
    break;
    case 0: // Desbaste realitzat
      ultimValorDesbaste = carregaEixZetapa;
      if(arrayEsmolat[1] == 1){ // Acabat en angle 1
        //posCursor(11,3);
        //lcd.print("b1");
        ultimValorDesbaste = carregaEixZetapa;
        initAcabado();
        canviDirImoureCarroAuto();
      }
      else{
        if(arrayEsmolat[2] == 1){ // Lapejat en angle 1
          //posCursor(11,3);
          //lcd.print("b1");
          initLapeado();
          canviDirImoureCarroAuto();
        }
        else{
          if(arrayEsmolat[3] == 1){ // Acabat en angle 2
            //posCursor(11,3);
            //lcd.print("b2");
            initAcabado2();
            initPreviAngle2();
          } 
          else{
            //posCursor(11,3);
            //lcd.print("b4");
            // Fi esmolat
            //movimentAutoActual = 5;
            fiFulla();
          }
        }
      }
    break;
    case 1: // Acabado en angle 1 realitzat
      ultimValorAcabado = carregaEixZetapa;
      if(arrayEsmolat[2] == 1){ // Lapejat en angle 1
        //posCursor(11,3);
        //lcd.print("c1");
        initLapeado();
        canviDirImoureCarroAuto();
      }
      else{
        //posCursor(11,3);
        //lcd.print("c2");
        // Fi esmolat
        //movimentAutoActual = 5;
        fiFulla();
      }
    break; 
    case 2: // Lapejat en angle 1 realitzat
      if(arrayEsmolat[3] == 1){ // Acabat en angle 2
        //posCursor(11,3);
        //lcd.print("d1");
        // Aplicar angle 2  
        initAcabado2();
        initPreviAngle2();  
      }
      else{
        //posCursor(11,3);
        //lcd.print("d2");
        // Fi esmolat
        //movimentAutoActual = 5;
        fiFulla();
      }    
    break;
    case 3: // Acabat en angle 2 realitzat
      ultimValorAcabado = carregaEixZetapa;
      if(arrayEsmolat[4] == 1){ // Lapejat en angle 2
        //posCursor(11,3);
        //lcd.print("e1");
        initLapeado2();
        canviDirImoureCarroAuto();
      }
      else{
        //posCursor(11,3);
        //lcd.print("e2");
        // Fi esmolat
        //movimentAutoActual = 5;
        fiFulla();
      }
    break;
    case 4:
      //posCursor(11,3);
      //lcd.print("f1");
      // Fi esmolat
      //movimentAutoActual = 5;
      fiFulla();
    break;
  }
}

void nextEtapaAutomatic2(){
  switch(idxArrayEsmolat){
    case -1:
      if(arrayEsmolat[0] == 1){ // Hi ha desbast
        initDesbaste();
      }
      else{
        if(arrayEsmolat[1] == 1){ // Hi ha acabat
          initAcabado();
        }
        else{
          if(arrayEsmolat[2] == 1){ // Hi ha lapejat
            initLapeado();
          }
          else{
            // No hauria d'arribar aqui (ja que hiHaEtapes haruia de ser false)  
          }
        }
      }
    break;
    case 0:
      if(arrayEsmolat[1] == 1){
        initAcabado();
        canviDirImoureEixYauto();
      }
      else{
        if(arrayEsmolat[2] == 1){
          initLapeado();
          canviDirImoureEixYauto();
        }
        else{
          fiDentOfiFulla();  
        }
      }
    break;
    case 1:
      if(arrayEsmolat[2] == 1){
        initLapeado();
        canviDirImoureEixYauto();
      }
      else{
        fiDentOfiFulla();
      }
    break;
    case 2:
      fiDentOfiFulla();
    break;
  }   
}

void fiFulla(){
  if(nCuchilla < 99){
    nCuchilla++;
  }
  inicialitzarRectificarDistFrenada();
}

void fiDentOfiFulla(){
  if((dentActual == nDents-1) || (direccioCarroAuto == 0 && llegirPinMux16(FCx2) == LOW) || (direccioCarroAuto == 1 && llegirPinMux16(FCx1) == LOW)){
    // Ja no podem fer mes dents n'hi hagi o no, hem arribat al limit de l'eix X
    // Esmolat automatic per a la fulla actual finalitzat
    if(nCuchilla < 99){
      nCuchilla++;
    }
    dentActual = nDents;
    printDentsRestants();
    dentActual = 0;
    if(eixYcurrentPosition == initPosYauto - vars[idxRecAfilat]*200){
      // recoloquem eixY al principi
      tornarEixYa0 = true;
      canviDirImoureEixYauto();
    }
    else if(eixYcurrentPosition == initPosYauto){
      aplicarMoveRunXauto(-pasDentAcumulat);
      finalitzarModeAutomatic();
    }
  }
  else{ // cas dentActual < nDents-1
    // Iniciar nou esmolat per a la seguent dent
    dentActual++;
    printDentsRestants();
    posicionarNovaDent();

    carregaEixZtotal = 0;
    ultimValorDesbaste = 0;
    ultimValorAcabado = 0;
    idxArrayEsmolat = -1;
    nextEtapaAutomatic2();
    
    printAutomatic();
    printAutomaticValors();
    printSegonsCicle();
    printMinutsCicle();
    printOpcionsBotonsAutomatic(2);
    
    //countEtapaIniciat = true;
    dirEixYauto *= -1;
    moureEixYautomatic();
  }
}

/* 
 *** INICIALITZAR SEGON ANGLE D'ESMOLAT ***
 */

void initPreviAngle2(){
  if(molaPosicioAngle2 != 0 && vars[idxProduccio] == 1){
    // Ja sabem la posició on la mola ha d'esmolar. Ens estalviem el previ per medicio automatica
    if(mode == 0){
      noBlincar();
    }
    aplicarSegonAngleSenseMedir();
    printLapeadoCompleto(); // Printa 0 ja que prèviament count s'ha posat a 0 en el initAcabadoVariables();
    blinkSegonsMode();
    canviDirImoureCarroAuto();
  }
  else{
    iniciantEsmolatAngle2 = true;
    tempsConfirmarAutoOsalir = millis();
    stepsEixXautomaticAux = 40 + (vars[idxIncrementVel_low]+vars[idxIncrementVel_high]*256) * (vars[idxDesbastVel]-1);
    canviDirImoureCarroAuto();
  }
}

void desplasarCarro(){
  if(vars[idxBaixada] == 1){ // baixada dreta
    if(llegirPinMux16(FCx2) == HIGH || (fcDretAssolit == 1 && !finalDretDetectat)){
      desplasantCarroPrevi = true;
      direccioCarroAuto = 0; // cap final detectat i baixada a la dreta o ambdos: movem a la dreta
      //comprovarMovimentCarroDretAuto = true;
    }
  }
  else if(vars[idxBaixada] == 0){ // baixada esquerra
    if(llegirPinMux16(FCx1) == HIGH || (fcEsquerreAssolit == 1 && !finalEsquerreDetectat)){
      desplasantCarroPrevi = true;
      direccioCarroAuto = 1; // cap final detectat i baixada a l'esquerra: movem a l'esquerra
      //comprovarMovimentCarroEsquerreAuto = true;
    }
  }
  else if(vars[idxBaixada] == 2){
    if( (llegirPinMux16(FCx2) == HIGH || (fcDretAssolit == 1 && !finalDretDetectat)) && (llegirPinMux16(FCx1) == HIGH || (fcEsquerreAssolit == 1 && !finalEsquerreDetectat) )){
      desplasantCarroPrevi = true;
      direccioCarroAuto = 0; 
    }
  }
  
  escriureOut16(aiguaMotor, HIGH);
  if(desplasantCarroPrevi){
    if(!iniciantEsmolatAngle2){
      printDesplasarCarro();
    }
    alsarMolaSeguretat();
    // Hem de donar un valor de steps per al moviment de carro post medicio automatica, previ a l'esmolat
    stepsEixXautomaticAux = 40 + (vars[idxIncrementVel_low]+vars[idxIncrementVel_high]*256) * (vars[idxDesbastVel]-1);
    moureCarroAutomatic();
  }
  else{
    // Per donar temps que l'aigua arribi a la zona de la mola, per no es crema la cutxilla
    delayEnWhile(3000,0);
    encoderPitjarAConfigNivell(11);
  }
}

void finalitzarDesplasarCarro(){
  recuperarPosMolaEsmolat();
  eixZcurrentPosition = eixZ.currentPosition();
  eixZposActual = eixZcurrentPosition;
  desplasantCarroPrevi = false;
  encoderPitjarAConfigNivell(11);
}

/* 
 *** CONDICIONS D'ACTUACIO ***
 */

boolean esPuntDeCanvi(){
  if(vars[idxBaixada] == 2 || (direccioCarroAuto == 1 && vars[idxBaixada] == 0) || (direccioCarroAuto == 0 && vars[idxBaixada] == 1)){
    return true;
  }
  else{
    return false;
  }
}

boolean esmolarAngle2(){
  if(vars[idxMedicio] == 1 && angle2MajorAngle1()){
    return true;
  }
  else{
    return false;
  }  
}

boolean angle2MajorAngle1(){
  return (vars[idxAngle2]*10 + vars[idxAngle2Dec] > vars[idxAngle1]*10 + vars[idxAngle1Dec]);
}

boolean programaQuartEix(){
  return vars[idxPrograma] >= 8;  
}

/* 
 *** CANVI DIRECCIO EIXOS ***
 */

void canviDirImoureCarroAuto(){
  canviDireccioCarro();
  delayEnWhile(500,1);
  moureCarroAutomatic();
}

void canviDireccioCarro(){
  if(direccioCarroAuto == 1){
    direccioCarroAuto = 0;
  }
  else{
    direccioCarroAuto = 1;
  }  
}

void canviDirImoureEixYauto(){
  dirEixYauto *= -1;
  moureEixYautomatic();    
}

/* 
 *** INTERRUPCIONS I FINALITZACIO D'ESMOLAT ***
 */

void pausarModeAutomatic(){
  movimentsAutomatics = 0;
  pausat = true;
  apagarElementsFisicsAutomatic();
  initParpadeigBotoDret();
  printOpcionsBotonsAutomatic(1);
  if(programaQuartEix()){
    // Utilitzat per comptar el temps que porta 
    tempsConfirmarAutoOsalir = millis();
  }
  else{
    if(mode == 1){
      aModeMoures();  
    }
    printPlatEnEsmolatPlat();
  }
  blinkSegonsMode();
}

void rependreAutomatic(){
  aturarParpadeigBotoDret();
  posicionarDivisor();
  platAzero = false;
  movimentsAutomatics = 1;
  pausat = false;
  if(vars[idxActivarMagnetisme] == 0){ // cas magnetisme activat ja esta en noBlink
    noBlincar();  
  }
  printAutomatic();
  printAutomaticValors();
  printOpcionsBotonsAutomatic(2);
  printDesUltimCompleto();
  printAcaUltimCompleto();
  if(movimentAutoActual == 3){
    printLapeadoCompleto();
  }
  escriureOut16(aiguaMotor, HIGH);
  if(!programaQuartEix()){
    activarGirMola();
    if(cAux == 32){
      columna = cAux;
      filera = fAux;
    }
    else{
      columna = 32;
      filera = fAux;  
    }
    if(mode == 1){
      mode = 0;  
    }
  }
  else{
    activarGirMola2();
  }
  blinkSegonsMode(); 
  // Per donar temps que l'aigua arribi a la zona de la mola, per no es crema la cutxilla
  if(programaQuartEix()){
    if(tempsConfirmarAutoOsalir > 0){
      unsigned long t = millis() - tempsConfirmarAutoOsalir;
      if(t < 5000){
        delayEnWhile(t*2,0);
      }
      else{
        delayEnWhile(10000,0);
      }
      tempsConfirmarAutoOsalir = 0UL;
    }
    else{
      // Han passat ja 4 segons i per tant la mola s'ha aturat del tot. Apliquem els 10 segons
      delayEnWhile(10000,0);  
    }
  }
  else{
    delayEnWhile(3000,0);
  }
  encendreLlumBotons();
  if(vars[idxActivarMagnetisme] == 1){
    blinkCoordenades();  
  }
  if(!programaQuartEix()){
    puntDeCanvi = esPuntDeCanvi();
  }
  if(movimentAutoActual != 3){
    nextMoveFromDesAcaCarro();  
  }
  else{
    nextMoveFromLapeadoCarro();
  }
}

void aturarModeAutomatic(){
  if(!programaQuartEix()){
    if(mode == 1){
      aModeMoures();  
    }
    comprovarMovimentCarroDretAuto = false;
    comprovarMovimentCarroEsquerreAuto = false;
    inicialitzarRectificarDistFrenada();
    //finalitzarModeAutomatic();  
  }
  else{
    if(eixYcurrentPosition == initPosYauto){
      aplicarMoveRunXauto(-pasDentAcumulat);
      //Si printem en els dos casos (pos inicial i pos final) el STOP respon bé
      //Si no printem res, el STOP mai entra a tornarEixYa0
      //Si hi posem un delay(5) a cada cas, el STOP respon bé
      //Total: problema de temps? deixem el delay i ja està
      delay(5);
      finalitzarModeAutomatic();
    }
    else if(eixYcurrentPosition == initPosYauto - vars[idxRecAfilat]*200){
      //Si printem en els dos casos (pos inicial i pos final) el STOP respon bé
      //Si no printem res, el STOP mai entra a tornarEixYa0
      //Si hi posem un delay(5) a cada cas, el STOP respon bé
      //Total: problema de temps? deixem el delay i ja està
      delay(5);
      tornarEixYa0 = true;
      canviDirImoureEixYauto();
    }
    else{
      esborrarPantalla();
      while(1){}  
    }
  }  
}

void aturarModeAutomatic2(){
  finalitzarModeAutomatic();  
}

void desferVariablesInicialsAutomatic(){
  movimentAutoActual = 0;
  idxArrayEsmolat = -1;
  for(byte i=0; i<5; i++){
    arrayEsmolat[i] = 0;  
  }
  hiHaEtapes = false;
  if(programaQuartEix()){
    dentActual = 0;
  }
}

void apagarElementsFisicsAutomatic(){
  if(!programaQuartEix()){
    aturarGirMola();
  }
  else{
    aturarGirMola2();
  }
  escriureOut16(aiguaMotor, LOW);
  apagarLlumBotons();
  llumOn = false;
}

void inicialitzarRectificarDistFrenada(){
  // És necessari perquè la funcio és cridada just després de la desacceleració,
  // i desacceleració es gestiona excepcionalment amb un while que no llegeix el mux
  if(finalDretDetectat){
    if(llegirPinMux16(FCx2) == HIGH){
    //if(!comprovaFinalCarrera(FCx2)){
      rectificantTrobarFC = true;
      direccioCarroAuto = 1; // cap a l'esquerra
    }
  }
  else if(finalEsquerreDetectat){
    if(llegirPinMux16(FCx1) == HIGH){
    //if(!comprovaFinalCarrera(FCx1)){
      rectificantTrobarFC = true;
      direccioCarroAuto = 0; // cap a la dreta
    }
  }
  if(rectificantTrobarFC){
    moureCarroAutomatic();
  }
  else{
    finalitzarModeAutomatic();
  }
}

void finalitzarModeAutomatic(){
  apagarElementsFisicsAutomatic();
  
  if(aplicarMenysVelPrimeraPasada){
    vars[idxCurrentVel] = velAux;
  }
  
  if(vars[idxProduccio] == 0){
    llumPerMagnetisme();
    dirEixYauto = 1;
    if(programaQuartEix()){
      // Utilitzat per comptar el temps que porta 
      tempsConfirmarAutoOsalir = 0UL; 
    }
    encoderPitjarAConfigNivell(4);
  }
  else{
    if((idxArrayEsmolat == 3 || idxArrayEsmolat == 4) && existeixAngle1){
      // El divisor s'ha posicionat a l'angle2: apliquem refet
      referMovimentsPerSeguentEsmolat();
    }
    else if(carregaEixZtotal != 0){
      moureMolaPosicioInicial();
    }
    else{
      eixZcurrentPosition = eixZ.currentPosition();  
    }
    if(aplicarMenysVelPrimeraPasada){
      printVelocitatNova();
      blinkSegonsMode();
      aplicarMenysVelPrimeraPasada = false;  
    }
    //carregaEixZtotalTotal -= carregaEixZtotal;
    //idxArrayEsmolat = idxPrimeraEtapa;
    movimentsAutomatics = 0;
    movimentAutoActual = 0;
    idxArrayEsmolat = -1;
    initParpadeigBotoDret();
    printOpcionsBotonsAutomatic(0);
    if(vars[idxActivarMagnetisme] == 1){
      printMagDesmag(1);
    }
    if(programaQuartEix()){
      // Utilitzat per comptar el temps que porta 
      tempsConfirmarAutoOsalir = millis();
    }
    else{
      printPlatEnEsmolatPlat();
    }
    blinkSegonsMode();
  }
  desactivarTempsCicle();
  idxVar = idxMinutsCicleRestants;
  byte aux = getVariable();
  if(minutsCicle + aux < 60){
    vars[idxVar] = minutsCicle + aux;
    setVariable();
  }
  else{
    if(vars[idxCodiFinalIntroduit] == 0){
      sumarHores();
    }
    else{
      intEnEdicio = vars[idxHoresTotals_low]+vars[idxHoresTotals_high]*256 + 1;
      guardarVariable(idxHoresRecanvis_low, vars[idxHoresRecanvis_low], true, vars[idxHoresRecanvis_high], false, 0);
      guardarIntEnEdicioAarray();
    }
    guardarVariable(idxMinutsCicleRestants, minutsCicle + aux - 60, false, 0, false, 0);
  }
}

void actualitzarVelocitat(){
  stepsEixXautomatic = 40 + (vars[idxIncrementVel_low]+vars[idxIncrementVel_high]*256) * (vars[idxCurrentVel]-1);
}


/*
 ************************************************************************************
 ************************************************************************************
 * PROVES FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 
 
void provesMUX16In(){
  if(millis() % 50 == 0){
    posCursor(0,0);
    printar("                ");
    posCursor(0,0);
    for(byte i=0; i < 16; i++){
      printarByte(llegirPinMux16(i));
    }
  }
}

void provesOUT16(){
  extendedOutput.write(llumBotoRight, HIGH);
  extendedOutput.write(llumBotoLeft, HIGH);  
}

void provesEncoder(){
  if(valorEncoderNew != valorEncoderOld){
    posCursor(18,1);
    printar("      ");
    posCursor(17,1);
    printarInt(valorEncoderNew); 
    valorEncoderOld = valorEncoderNew;
  }
  if(millis()%30 == 0){
    posCursor(30,1);
    printarByte(PINB & (1<<PORTB0)); // digitalRead(pitjar)
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * MEDICIO AUTOMATICA FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */


void ajustarMolaAutomatic(){
  if(!iniciantEsmolatAngle2){
    apagarLlumBotons();
    noBlincar();
    printAjustarMolaAutomatic();
    activarGirMola();
    delay(2000);
  }
  else{
    // Aturem bomba d'aigua i hem d'esperar força temps per a què no interfereixi en la medició de vibracions
    escriureOut16(aiguaMotor, LOW);
    delayEnWhile(5000, 0);
    printCancelarMedicio(0); 
  }
  escriureOut16(llumBotoLeft, HIGH);
  sorollNominal();
  /*
  lcd.clear();
  posCursor(0,0);
  lcd.print("Nominal:");
  lcd.print(valorNominal);
  posCursor(15,0);
  lcd.print("Limit+:");
  lcd.print(valorNominal + (vars[idxSensibilitat_low] + vars[idxSensibilitat_high]*256));
  posCursor(15,1);
  lcd.print("Limit-:");
  lcd.print(valorNominal - (vars[idxSensibilitat_low] + vars[idxSensibilitat_high]*256));
  delay(5000);
  */
  eixZ.setSpeed(velMola/6);
  mesurarSoroll = true;
  medicioActual = 0;
  gestioTemps = millis() + 50UL;
}

int sorollActual(int *valorActual){
  *valorActual += llegirPinMux16analog(envelope);
  //extendedInput.readA(xip2AnalogInputs);
  //*valorActual += xip2AnalogInputs[envelope];
}

void sorollNominal(){
  for(byte i=0; i<N_MEDICIONS; i++){
    sorollActual(&valorNominal);
    delay(50);
  }
  valorNominal /= N_MEDICIONS;
}

boolean sorollMedicio(){
  for(byte i=0; i<N_MEDICIONS; i++){
    sorollActual(&valorMedicio);   
  }
  valorMedicio /= N_MEDICIONS;
  if(valorMedicio > (valorNominal + (vars[idxSensibilitat_low] + vars[idxSensibilitat_high]*256)) || valorMedicio < (valorNominal - (vars[idxSensibilitat_low] + vars[idxSensibilitat_high]*256))){
    return true;
  }
  else{
    return false;
  }
}

void sorollAssolit(){
  eixZ.runSpeed();
  eixZcurrentPosition = eixZ.currentPosition();
  eixZposActual = eixZcurrentPosition;
  if(millis() > gestioTemps){
    sorollActual(&valorMedicio);
    if(medicioActual < N_MEDICIONS){
     medicioActual++;
    }
    else{
      valorMedicio /= N_MEDICIONS;
      if(valorMedicio > (valorNominal + (vars[idxSensibilitat_low] + vars[idxSensibilitat_high]*256)) || valorMedicio < (valorNominal - (vars[idxSensibilitat_low] + vars[idxSensibilitat_high]*256))){
        aturarMedicio();
        escriureOut16(llumBotoLeft, LOW);
        gestioTemps = 0UL;
        if(!iniciantEsmolatAngle2){
          delay(300);
        }
        else{
          printCancelarMedicio(1);
          blinkCoordenades();
          delayEnWhile(300,0);
        }
        desplasarCarro();
      }
      medicioActual = 0;
      valorMedicio = 0;
    }
    gestioTemps = millis() + 10UL; 
  }
}

void aturarMedicio(){
  mesurarSoroll = false;
  eixZ.stop();
}


/*
 ************************************************************************************
 ************************************************************************************
 * MAGNETISME
 ************************************************************************************
 ************************************************************************************
 */


void magnetitzar(){
  canviarMagnetisme(true);
  // senyal que fa de boto per a la placa del magnetitzador.
  // S'activa manualment: HIGH = magnetitzem o desmagnetitzem.
  // Tan bon punt hem aplicat el HIGH, hem de posarlo a LOW, simulant que hem desapretat el boto
  escriureOut16(platMagnetic, HIGH);
  delay(100); // enviem senyal durant 250 ms. El micro del magnetitzdor sabra que si es 250 ms, volem DESmagnetitzar
  escriureOut16(platMagnetic, LOW);
  delay(500);  
}

void desmagnetitzar(){
  gestioTemps = 0UL;
  comprovarDesactivarPlat = false;
  escriureOut16(platMagnetic, LOW);
  escriureOut16(llumBotoLeft, LOW);
  canviarMagnetisme(false);
  // S'activa manualment: HIGH = magnetitzem o desmagnetitzem.
  // Tan bon punt hem aplicat el HIGH, hem de posarlo a LOW, simulant que hem desapretat el boto
  escriureOut16(platMagnetic, HIGH);
  delay(1000); // enviem senyal durant 1 segon. El micro del magnetitzdor sabra que si es 1 segon, volem DESmagnetitzar
  escriureOut16(platMagnetic, LOW);
}

void canviarMagnetisme(boolean canvi){
  platMagneticOn = canvi;
  posAux = pos;
  pos = INIT_POS_VARSGENERALS_EEPROM;
  guardarVariable(idxMagnetitzat, platMagneticOn, false, 0, false, 0);
  pos = posAux;
}

void desactivarPlatMagnetic(){
  if(comprovarDesactivarPlat){
    if(llegirPinMux16(botoLeft) == HIGH){ // no desmagnetitzem
      gestioTemps = 0UL;
      comprovarDesactivarPlat = false;
    }
    else{
      if(millis() >= gestioTemps){
        if(configNivell == 11){
          aturarParpadeigBotoDret();
          desmagnetitzar();
          delay(4000);
          initParpadeigBotoDret();
        }
      }
    }
  }
}

void desactivarPlatMagneticEncoder(){
  if(comprovarDesactivarPlat){
    if((PINB & (1<<PORTB0)) == HIGH){ // digitalRead(pitjar)
      // No desmagnetitzem
      gestioTemps = 0UL;
      comprovarDesactivarPlat = false;
    }
    else{
      if(millis() >= gestioTemps){
        desmagnetitzar();
        delay(2000);
        printMagDesmag(0);
        noBlincar();
      }
    }
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * SEGON ROTATORI
 ************************************************************************************
 ************************************************************************************
 */
 

void canviarRotatori(){
  segonRotatori = !segonRotatori;
  digitalWrite(activarRotatori2, segonRotatori); 
}


/*
 ************************************************************************************
 ************************************************************************************
 * LLUMS BOTONS
 ************************************************************************************
 ************************************************************************************
 */


void encendreLlumBotons(){
  escriureOut16(llumBotoRight, HIGH);
  escriureOut16(llumBotoLeft, HIGH);
}

void apagarLlumBotons(){
  escriureOut16(llumBotoRight, LOW);
  escriureOut16(llumBotoLeft, LOW);  
}

void llumPerMagnetisme(){
  if(vars[idxActivarMagnetisme] == 1){
    if(platMagneticOn){
      escriureOut16(llumBotoLeft, HIGH);
    }
  }  
}

void initParpadeigBotoDret(){
  parpadeigLlum = true;
  gestioTemps2 = millis() + 500UL;
  escriureOut16(llumBotoRight, HIGH);
  llumOn = true;
}

void parpadeigBotoDret(){
  if(parpadeigLlum){
    if(millis() >= gestioTemps2){
      if(llumOn){
        escriureOut16(llumBotoRight, LOW);
        llumOn = false;
      }
      else{
        escriureOut16(llumBotoRight, HIGH);
        llumOn = true;
      }
      gestioTemps2 = millis() + 500UL;
    }
  }
}

void aturarParpadeigBotoDret(){
  gestioTemps2 = 0UL;
  parpadeigLlum = false;
  llumOn = false;
  escriureOut16(llumBotoRight, LOW);
}


/*
 ************************************************************************************
 ************************************************************************************
 * GIR MOLES
 ************************************************************************************
 ************************************************************************************
 */


void activarGirMola(){
  // Ens assegurem que nomes hi ha un pin de moviment de gir de mola activat, sino podem liarla molt
  if(vars[idxGirMola] == 0){
    escriureOut16(molaGirDreta, LOW);
    escriureOut16(molaGirEsquerra, HIGH);
  }
  else{
    escriureOut16(molaGirEsquerra, LOW);
    escriureOut16(molaGirDreta, HIGH);
  }
}

void aturarGirMola(){
  escriureOut16(molaGirEsquerra, LOW);
  escriureOut16(molaGirDreta, LOW);
}

void activarGirMola2(){
  // Versio pilot que no tindra velocitat analogica. Sempre anira a 24mil rev
  //calcularAnalogRpm();
  //analogWrite(mola2Vel, analogRpm);
  escriureOut16(mola2On, HIGH);     
}

void aturarGirMola2(){
  escriureOut16(mola2On, LOW);
}


/*
 ************************************************************************************
 ************************************************************************************
 * GESTIO TEMPS ESMOLAT
 ************************************************************************************
 ************************************************************************************
 */
 
 
void activarTempsCicle(){
  segonsCicle = 0;
  minutsCicle = 0;
  tempsCicle = millis() + 1000UL;
  tempsCicleActivat = true;
}

void desactivarTempsCicle(){
  tempsCicle = 0UL;
  tempsCicleActivat = false;
}

void sumarSegonsCicle(){
  if(tempsCicleActivat){
    //millis() = millis();
    if(tempsCicle < millis()){
      if(segonsCicle < 59){ // 59
        segonsCicle++;
      }
      else{
        segonsCicle = 0;
        minutsCicle++;
        if(vars[idxCodiFinalIntroduit] == 0){
          if(minutsCicle % 60 == 0){ // 60
            sumarHores();
          }
        }
      }
      printSegonsCicle();
      if(segonsCicle == 0){
        printMinutsCicle();
      }
      if(!blinkOff){
        blinkCoordenades();
      }
      tempsCicle = millis() + 1000UL;
    }
  }    
}

void sumarHores(){
  intEnEdicio = vars[idxHoresCicle_low]+vars[idxHoresCicle_high]*256 + 1;
  guardarVariable(idxHoresCicle_low, vars[idxHoresCicle_low], true, vars[idxHoresCicle_high], false, 0);
  guardarIntEnEdicioAarray();
  
  intEnEdicio = vars[idxHoresRecanvis_low]+vars[idxHoresRecanvis_high]*256 + 1;
  guardarVariable(idxHoresRecanvis_low, vars[idxHoresRecanvis_low], true, vars[idxHoresRecanvis_high], false, 0);
  guardarIntEnEdicioAarray();
  
  intEnEdicio = vars[idxHoresTotals_low]+vars[idxHoresTotals_high]*256 + 1;
  guardarVariable(idxHoresTotals_low, vars[idxHoresTotals_low], true, vars[idxHoresTotals_high], false, 0);
  guardarIntEnEdicioAarray();
}


/*
 ************************************************************************************
 ************************************************************************************
 * MUX IN I MUX OUT
 ************************************************************************************
 ************************************************************************************
 */

 
byte llegirPinMux16(byte pin){
  extendedInput.readD(pin); 
}

uint8_t llegirPinMux16analog(byte pin){
  extendedInput.readA(pin);  
}

void escriureOut16(byte pin, byte valor){
  extendedOutput.write(pin, valor);
}


/*
 ************************************************************************************
 ************************************************************************************
 * GESTIO EEPROM
 ************************************************************************************
 ************************************************************************************
 */


void guardarVariable(int idx1, byte valor1, boolean esInt, byte valor2, boolean esLong, byte valor3){
  idxVar = idx1;
  vars[idxVar] = valor1;
  setVariable();
  if(esInt){
    idxVar += PROPS_VAR;
    vars[idxVar] = valor2;
    setVariable();
    if(esLong){
      idxVar += PROPS_VAR;
      vars[idxVar] = valor3;
      setVariable();
      idxVar -= PROPS_VAR;  
    }
    idxVar -= PROPS_VAR;
  }
}

byte getVariable(){
  return llegirEeprom(pos + idxVar/PROPS_VAR);  
}

void setVariable(){
  escriureEeprom(pos + idxVar/PROPS_VAR, vars[idxVar]);
}

byte llegirEeprom(int pos){
  return EEPROM.read(pos); 
}

void escriureEeprom(int pos, byte x){
  if(llegirEeprom(pos) != x){
    EEPROM.write(pos, x);
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * GESTIO CURSOR
 ************************************************************************************
 ************************************************************************************
 */
 

void aModeEdicio(){
  noBlincar();
  mode = 1;
}

void aModeMoures(){
  blinkCoordenades();
  mode = 0;  
}

void setCoordenades(byte c, byte f){
  columna = c;
  filera = f;
}

void blinkCoordenades(){
  posCursor(columna,filera);
  lcd.blink();
  blinkOff = false;
}

void setBlink(byte c, byte f){
  setCoordenades(c, f);
  blinkCoordenades();  
}

void blinkSegonsMode(){
  if(!programaQuartEix()){
    if(mode == 0){
      blinkCoordenades();
    }
    else{
      noBlincar();
    }
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * GESTIO DELAY
 ************************************************************************************
 ************************************************************************************
 */


void delayEnWhile(int temps, byte tipus){
  unsigned long tempsAux = millis() + temps;
  while(millis() < tempsAux){
    switch(tipus){
      case 0:
        sumarSegonsCicle();
        navegacioAutomatics();
      break;
      case 2:
        sumarSegonsCicle();
      break;
    }
  }
}

 
/*
 ************************************************************************************
 ************************************************************************************
 * GESTIO LCD FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */

void printar(char* text){
  lcd.print(text); 
}

void printarWrite(byte valor){
  lcd.write(valor);  
}

void printarByte(byte valor){
  lcd.print(valor);  
}

void printarInt8(int8_t valor){
  lcd.print(valor);  
}

void printarInt(int valor){
  lcd.print(valor);  
}

void printarLong(long valor){
  lcd.print(valor);  
}

void posCursor(byte c, byte f){
  lcd.setCursor(c,f);
}

void noBlincar(){
  lcd.noBlink();
  blinkOff = true;  
}

void esborrarPantalla(){
  lcd.clear();  
}


/*
 ************************************************************************************
 ************************************************************************************
 * MOTORS
 ************************************************************************************
 ************************************************************************************
 */


/* 
 *** EXECUCIÓ DE MOVIMENTS GLOBALS ***
 */

void accelerar(int vel, byte dir){
  ramp_time = vars[idxAcc_low] + vars[idxAcc_high]*256;
  int currentVel;
  for(int i=1; i <= ramp_time; i++){
    currentVel = vel * (3 - 2*(i/ramp_time)) * (i/ramp_time) * (i/ramp_time);
    if(currentVel < 40){
      currentVel = 40;
    }
    tone(eixXstep, currentVel);
    if(comprovarInterrupcio(dir)){
      stepsAccInterrompuda = currentVel;
      i = ramp_time+1;
    }
    if(movimentsAutomatics == 1){
      sumarSegonsCicle();
      //navegacioAutomatics();
    }
  }
}

void desaccelerar(int vel){
  ramp_time = vars[idxDacc_low] + vars[idxDacc_high]*256;
  int currentVel;
  for(int i=ramp_time; i >= 1; i--){
    currentVel = vel * (3 - 2*(i/ramp_time)) * (i/ramp_time) * (i/ramp_time);
    if(currentVel < 40){
      currentVel = 40;  
    }
    tone(eixXstep, currentVel);
    i--;
    delay(1);
    if(movimentsAutomatics == 1){
      sumarSegonsCicle();
      navegacioAutomatics();
    }
  }
}

boolean comprovarInterrupcio(byte dir){
  boolean retorn = false;
  if(dir == 0){
    if(movimentsAutomatics == 0 && !desplasantCarroPrevi){
      if(llegirPinMux16(botoRight) == HIGH){
        retorn = true;
      }
    }
    if(llegirPinMux16(FCx2) == LOW){
      retorn = true;
    }
  }
  else if(dir == 1){
    if(movimentsAutomatics == 0 && !desplasantCarroPrevi){
      if(llegirPinMux16(botoLeft) == HIGH){
        retorn = true;
      }
    }
    if(llegirPinMux16(FCx1) == LOW){
      retorn = true;
    }
  }
  return retorn;
}

void aturarCarro(int velActual){
  if(stepsAccInterrompuda > 0){
    desaccelerar(stepsAccInterrompuda);
    stepsAccInterrompuda = 0;
  }
  else{
    desaccelerar(velActual);  
  }
  noTone(eixXstep);
  carroMoventse = false;
  //eixX.setAcceleration(2000);
  //eixX.stop();
  //eixX.runToPosition();
}

void aplicarMoveRunXmanual(long distancia, byte boto, byte fc){
  eixX.setCurrentPosition(0);
  eixX.moveTo(distancia);
  boolean despitjat = false;
  long posFinal = eixX.currentPosition() + distancia;
  
  while(eixX.currentPosition() != posFinal && llegirPinMux16(boto) == LOW && llegirPinMux16(fc) == HIGH){
    eixX.run();
  }
  eixX.stop();
  eixX.runToPosition();
}

void aplicarMoveRunXauto(long distancia){
  eixX.setCurrentPosition(0);
  eixX.move(distancia);
  long posFinal = eixX.currentPosition() + distancia;
  while(eixX.currentPosition() != posFinal){
    eixX.run();
  }
}

void aplicarMoveRunY(long distancia){
  eixY.move(distancia);
  long posFinal = eixY.currentPosition() + distancia;
  while(eixY.currentPosition() != posFinal){
    eixY.run();
  }
}

void aplicarMoveRunZ(long distancia, boolean aplicar){
  eixZ.move(distancia);
  long posFinal = eixZ.currentPosition() + distancia;
  while(eixZ.currentPosition() != posFinal){
    eixZ.run();
    if(aplicar){
      sumarSegonsCicle();
      navegacioAutomatics();
    }
  }
}

void aplicarMoveRunA(long distancia, boolean aplicar){
  eixA.move(distancia);
  long posFinal = eixA.currentPosition() + distancia;
  while(eixA.currentPosition() != posFinal){
    eixA.run();
    if(aplicar){
      sumarSegonsCicle();
      navegacioAutomatics();
    }
  }   
}

/* 
 *** REFERENCIAT ***
 */

void referenciar(){
  /* CONDICIO IMPRESCINDIBLE eix A:
       Impedir la possibilitat d'errors, cal que:
       - Tan el tope mecanic en horitzontal com la quantitat d'offset,
         no sobrepassin el FC, quedant aixi el plat fora de rang       
   */
  char* eix;
  boolean referenciatCorrecte = false;
  byte caseRef = 0;
  while(!referenciatCorrecte){
    switch(caseRef){
      case 0: // Referenciar eix Z
        noBlincar();
        printReferenciantEixZ();
        if(llegirPinMux16(FCz) == LOW){
          // Busquem FC
          eixZsurtObuscaFC(-velMola, LOW, 0);
          delay(300);
        }
        // Sortim de FC
        if(eixZsurtObuscaFC(velMola/4, HIGH, 1)){
          eix = "Z";
          caseRef = 4;
        }
        else{
          delay(300);
          // Separem direccio Sortir de FC
          eixZaplicaOffset();
          eixZinicialitzarPosInicial();
          if(vars[idxActivarQuartEix] == 1){
            caseRef = 1;  
          }
          else{
            caseRef = 2; 
          }
        }
      break;
      case 1: // Referenciar eix Y
        printReferenciantEixY();
        if(llegirPinMux16(FCy) == LOW){
          // Busquem FC
          eixYsurtObuscaFC(2000, LOW, 0);
          delay(300);
        }
        // Sortim de FC
        if(eixYsurtObuscaFC(-80, HIGH, 1)){
          eix = "Y";
          caseRef = 4;
        }
        else{
          delay(300);
          // Separem direccio Sortir de FC
          eixYaplicaOffset();
          eixYinicialitzarPosInicial();
          caseRef = 2;
        }
      break;
      case 2: // Referenciar eix A
        printReferenciantEixA();
        if(llegirPinMux16(FCa) == LOW){
          // Busquem FC
          eixAsurtObuscaFC(3000, LOW, 0);
          delay(300);
        }
        // Sortim de FC
        if(eixAsurtObuscaFC(-300, HIGH, 1)){
          eix = "A";
          caseRef = 4;
        }
        else{
          delay(300);
          // Separem direccio Sortir de FC
          eixAaplicaOffset();
          eixAinicialitzarPosInicial();
          caseRef = 3;
        }
      break;
      case 3: // Error referenciant
        referenciatCorrecte = true;
      break;
      case 4: // Error referenciant
        printErrorReferenciat(eix);
        caseRef = 255;
      break;
      case 255:
        // Do nothing. Referencing error
      break;    
    }
  }
}

boolean eixZsurtObuscaFC(long distancia, byte estatFC, byte tipus){
  boolean retorn = false;
  eixZ.setSpeed(distancia);
  long distMaxima;
  if(tipus == 1){
    // Moviment deixant FC. Si passada x distancia segueix detectant FC, entenem que hi ha error
    distMaxima = abs(eixZ.currentPosition()) + LATCH_BACKOFF_Z;
  }
  while(llegirPinMux16(FCz) == estatFC || !retorn){
    eixZ.runSpeed();
    if(tipus == 1){
      // Moviment deixant FC. Si passada x distancia segueix detectant FC, entenem que hi ha error
      if(abs(eixZ.currentPosition()) > distMaxima){
        retorn = true;  
      }  
    }
  }
  return retorn;
}

void eixZaplicaOffset(){
  eixZ.setSpeed(velMola);
  aplicarMoveRunZ(2000, false);  
}

void eixZinicialitzarPosInicial(){
  eixZ.setCurrentPosition(0);
  eixZcurrentPosition = eixZ.currentPosition();
  eixZposActual = eixZcurrentPosition;  
}

boolean eixYsurtObuscaFC(long distancia, byte estatFC, byte tipus){
  boolean retorn = false;
  eixY.setSpeed(distancia);
  long distMaxima;
  if(tipus == 1){
    // Moviment deixant FC. Si passada x distancia segueix detectant FC, entenem que hi ha error
    distMaxima = abs(eixY.currentPosition()) + LATCH_BACKOFF_Y;
  }
  while(llegirPinMux16(FCy) == estatFC || !retorn){
    eixY.runSpeed();
    if(tipus == 1){
      // Moviment deixant FC. Si passada x distancia segueix detectant FC, entenem que hi ha error
      if(abs(eixY.currentPosition()) > distMaxima){
        retorn = true;  
      }  
    }
  }
  return retorn;
}

void eixYaplicaOffset(){
  eixY.setSpeed(2000);
  aplicarMoveRunY(-1000);
}

void eixYinicialitzarPosInicial(){
  eixY.setCurrentPosition(vars[idxRecorregutY_low] + vars[idxRecorregutY_high]*256L + vars[idxRecorregutY_high2]*256L*256L);
  eixYcurrentPosition = eixY.currentPosition();
  eixYposActual = eixYcurrentPosition;  
}

boolean eixAsurtObuscaFC(long distancia, byte estatFC, byte tipus){
  boolean retorn = false;
  eixA.setSpeed(distancia);
  long distMaxima;
  if(tipus == 1){
    // Moviment deixant FC. Si passada x distancia segueix detectant FC, entenem que hi ha error
    distMaxima = abs(eixA.currentPosition()) + LATCH_BACKOFF_A;
  }
  while(llegirPinMux16(FCa) == estatFC || !retorn){
    eixA.runSpeed();
    if(tipus == 1){
      // Moviment deixant FC. Si passada x distancia segueix detectant FC, entenem que hi ha error
      if(abs(eixA.currentPosition()) > distMaxima){
        retorn = true;  
      }  
    }
  }
  return retorn;
}

void eixAaplicaOffset(){
  // 100 voltes motor = 360º eix divisor A. 
  // 1 volta = 3.6º, i 1 volta = 2000 steps
  // Per tant, 1º = 2000steps/3.6º = 556 steps arrodonits
  // Per tant, 0.1º = 56 steps arrodonits
  int eixAoffset = (vars[idxOffsetA]*10 + vars[idxOffsetAdec]) * 56;
  if(eixAoffset != 0){
    // Un cop detectat en bona forma el final de carrera d'A, ens allunyem segons el offset establert
    eixA.setSpeed(4000);
    aplicarMoveRunA(eixAoffset, false);
  }
}

void eixAinicialitzarPosInicial(){
  // Hem arribat al nostre punt A de referència 0
  eixA.setCurrentPosition(RECORREGUT_A);
  eixAcurrentPosition = eixA.currentPosition();
  eixAposActual = eixAcurrentPosition;
  grauDivisor = 0;
  grauDivisorDecimal = 0;
}

/* 
 *** EXECUCIÓ DE MOVIMENTS MANUALS ***
 */

void molaHdivisorAHmanual(long *eixPosActual, int stepsEix){
  if(*eixPosActual > 0){
    if(*eixPosActual < stepsEix){
      //*eixPosActual -= *eixPosActual;
      *eixPosActual = 0;
    }
    else{
      //*eixPosActual -= stepsEix;
      *eixPosActual = *eixPosActual - stepsEix;
    }
    aplicarSpeedDirManual(-1);
  }
}

void molaAHdivisorHmanual(long *eixPosActual, int stepsEix, long recorregut){
  if(*eixPosActual < recorregut){
    if(recorregut - *eixPosActual < stepsEix){
      *eixPosActual = *eixPosActual + (recorregut - *eixPosActual);
    }
    else{
      //*eixPosActual += stepsEix;
      *eixPosActual = *eixPosActual + stepsEix;
    }
    aplicarSpeedDirManual(1);
  }
}

void aplicarSpeedDirManual(int8_t signe){
  if(runSpeedActivat == 0){
    switch(funcioRotatoriManual){
      case 0:
        eixZ.setSpeed(velMola*signe);
      break;
      case 1:
        eixA.setSpeed(VEL_PLAT*signe);  
      break;
      case 2:
        eixY.setSpeed(VEL_EIXY*signe);
      break;
    }
    if(signe < 0){
      sumantGrausDivisor = true;
    }
    else{
      sumantGrausDivisor = false;
    }
    runSpeedActivat = 1;
    bloqueigBotons = true;
    posCursor(10,3);
  }
}

void moureCarroManual(){
  int stepsManual = stepsEixXmanual;
  digitalWrite(eixXdirection, direccioCarroManual);
  delay(5);
  accelerar(stepsEixXmanual, direccioCarroManual);
  carroMoventse = true;
  if(stepsAccInterrompuda > 0){
    stepsManual = stepsAccInterrompuda;
  }
  tone(eixXstep, stepsManual);
}

void checkStepsConstantsEixYmanual(){
  if(runSpeedActivat != 0){
    eixY.runSpeed();
    eixYcurrentPosition = eixY.currentPosition();
    if(configNivell == 9 || configNivell == 10){
      if(millis()%200 == 0){
        printEixYcurrentPosition(); 
      }
    }
    if(eixYcurrentPosition == eixYposActual){
      bloqueigBotons = false;
      eixY.stop();
      runSpeedActivat = 0;
      printEixYcurrentPosition();
    }
    else{
      if(runSpeedActivat == 1){
        if(eixYcurrentPosition > eixYposActual){
          runSpeedActivat = 2;
          eixY.setSpeed(-VEL_EIXY);
        }
      }
      else if(runSpeedActivat == 2){
        if(eixYcurrentPosition < eixYposActual){
          runSpeedActivat = 1;
          eixY.setSpeed(VEL_EIXY);
        } 
      }
    }
  }
}

void checkStepsConstantsMolaManual(){
  if(runSpeedActivat != 0){
    eixZ.runSpeed();
    eixZcurrentPosition = eixZ.currentPosition();
    if(configNivell == 9 || configNivell == 10){
      if(millis()%500 == 0){
        printEixZcurrentPosition();
      }
    }
    if(eixZcurrentPosition == eixZposActual){
      bloqueigBotons = false;
      eixZ.stop();
      runSpeedActivat = 0;
      printEixZcurrentPosition();
    }
    else{
      if(runSpeedActivat == 1){
        if(eixZcurrentPosition > eixZposActual){
          runSpeedActivat = 2;
          eixZ.setSpeed(-velMola);
        }
      }
      else if(runSpeedActivat == 2){
        if(eixZcurrentPosition < eixZposActual){
          runSpeedActivat = 1;
          eixZ.setSpeed(velMola);
        } 
      }
    }
  }  
}

void checkStepsConstantsDivisorManual(){
  if(runSpeedActivat != 0){
    eixA.runSpeed();
    eixAcurrentPosition = eixA.currentPosition();
    
    if(eixAcurrentPosition % STEPS_EIXA == 0){
      delay(1);
      editarGrausDivisor();
    }
    if(millis()%500 == 0){
      printEixAcurrentPosition();
    }
    /*
    if(configNivell == 9 || configNivell == 10){
      if(millis()%200 == 0){
        printEixAcurrentPosition();
      }
    }
    */
    if(eixAcurrentPosition == eixAposActual){
      bloqueigBotons = false;
      //eixA.stop();
      runSpeedActivat = 0;
      printEixAcurrentPosition();
    }
    else{
      if(runSpeedActivat == 1){
        if(eixAcurrentPosition > eixAposActual){
          runSpeedActivat = 2;
          eixA.setSpeed(-VEL_PLAT);
          sumantGrausDivisor = true;
        }
      }
      else if(runSpeedActivat == 2){
        if(eixAcurrentPosition < eixAposActual){
          runSpeedActivat = 1;
          eixA.setSpeed(VEL_PLAT);
          sumantGrausDivisor = false;
        } 
      }
    }
  }
}

/* 
 *** EXECUCIÓ DE MOVIMENTS PREVI AUTOMATIC ***
 */

void posicionarDivisor(){
  long angleStepsRelatius = establirStepsAbsolutsIrelatiusAngleActual();
  int multiplicador = 1;
  if(angleStepsRelatius != 0){
    if(!iniciantEsmolatAngle2){
      noBlincar();
      printPosicionantUnEix("A");
    }
    eixA.setSpeed(3000);
    aplicarMoveRunA(angleStepsRelatius, iniciantEsmolatAngle2);
    eixAcurrentPosition = eixA.currentPosition();
    eixAposActual = eixAcurrentPosition;
  }
}

/* 
 *** EXECUCIÓ DE MOVIMENTS ESMOLAT AUTOMATIC ***
 */

void posicionarNovaDent(){
  moureMolaPosicioInicial();
  //alsarMolaSeguretat();
  moureEixXnextDent();
  //recuperarPosMolaEsmolat();
}

void moureMolaAutomatic(){
  int steps = 0; 
  if(count < nBaixades){
    steps = divisor;
  }
  else if(count == nBaixades && ultimaBaixadaResteValor > 0){
    steps = ultimaBaixadaResteValor;
  }
  carregaEixZtotal += steps;
  carregaEixZetapa += steps;
  aplicarMoveRunZ(steps, true);
  nextMoveFromDesAcaMola();
}

void checkMeitatDeCami(){
  if(millis() > tempsCarroDeFCx1AFCx2){
    aturarCarro(stepsEixXautomaticAux);
    // Elevem mola una certa distancia, abans de posicionar el segon angle
    eixZ.setSpeed(-velMola);
    alsarMolaSegonsPosicio(true);
    eixZcurrentPosition = eixZ.currentPosition();
    eixZposActual = eixZcurrentPosition;
    // Posicionem el divisor al nou angle
    posicionarDivisor();
    encoderPitjarAConfigNivell(15);
  }
}

void alsarMolaSeguretat(){
  alsarMolaSegonsPosicio(iniciantEsmolatAngle2);
  eixZ.setSpeed(velMola);
}

void alsarMolaSegonsPosicio(boolean aplicarTemps){
  distanciaAlsada = SEPARACIO_MOLA;
  aplicarMoveRunZ(-distanciaAlsada, aplicarTemps);
}

void recuperarPosMolaEsmolat(){
  aplicarMoveRunZ(/*SEPARACIO_MOLA*/distanciaAlsada - (vars[idxCompensacio_low] + vars[idxCompensacio_high]*256), iniciantEsmolatAngle2);
}

void recuperarPosicioMolaAngle(long molaPosicioAngle){
  long molaPosicioRelativaAngle1;
  if(molaPosicioAngle < eixZcurrentPosition){
    molaPosicioRelativaAngle1 = -(eixZcurrentPosition - molaPosicioAngle);
  }
  else{
    molaPosicioRelativaAngle1 = molaPosicioAngle - eixZcurrentPosition;
  }
  aplicarMoveRunZ(molaPosicioRelativaAngle1, true);
  eixZcurrentPosition = eixZ.currentPosition();
  eixZposActual = eixZcurrentPosition;
}

void moureEixXnextDent(){
  int pasDentTotal = (vars[idxPasDent]*10 + vars[idxPasDentDec])*10;
  aplicarMoveRunXauto(pasDentTotal);
  pasDentAcumulat += pasDentTotal;
  /*
  for(byte i=0; i<pasDentTotal; i++){
    // Apliquem un step
    HIGH_D14
    delayMicroseconds(1000);
    LOW_D14
    delayMicroseconds(1000);
    //sumarSegonsCicle();
    //navegacioAutomatics();
  }
  */
}

void moureEixYautomatic(){
  eixY.setSpeed(stepsEixYautomatic*dirEixYauto);
  comprovarMovEixY = true;
}

void moureCarroAutomatic(){
  int stepsAuto = stepsEixXautomatic;
  if(direccioCarroAuto == 1){
    if(rectificantTrobarFC){
      comprovarMovimentCarroDretAuto = true;
      stepsAuto = STEPS_RECTIFICAT_TROBARFC;
    }
    else{
      if(desplasantCarroPrevi){
        stepsAuto = stepsEixXautomaticAux;
      }
      else if(iniciantEsmolatAngle2){
        stepsAuto = stepsEixXautomaticAux; 
      }
      comprovarMovimentCarroEsquerreAuto = true;
    }
  }
  else{
    if(rectificantTrobarFC){
      comprovarMovimentCarroEsquerreAuto = true;
      stepsAuto = STEPS_RECTIFICAT_TROBARFC;
    }
    else{
      if(desplasantCarroPrevi){
        stepsAuto = stepsEixXautomaticAux;
      }
      else if(iniciantEsmolatAngle2){
        stepsAuto = stepsEixXautomaticAux; 
      }
      comprovarMovimentCarroDretAuto = true;
    }
  }
  digitalWrite(eixXdirection, direccioCarroAuto);
  delay(5);
  accelerar(stepsAuto, direccioCarroAuto);
  carroMoventse = true;
  if(stepsAccInterrompuda > 0){
    stepsAuto = stepsAccInterrompuda;
  }
  tone(eixXstep, stepsAuto);
}

void comprovacionsPosEixYauto(){
  if(comprovarMovEixY){
    eixY.runSpeed();
    eixYcurrentPosition = eixY.currentPosition();
    eixYposActual = eixYcurrentPosition;
    if(eixYcurrentPosition == initPosYauto || eixYcurrentPosition == initPosYauto - vars[idxRecAfilat]*200){
      comprovarMovEixY = false;
      if(tornarEixYa0){
        tornarEixYa0 = false;
        aplicarMoveRunXauto(-pasDentAcumulat);
        //posCursor(7,3);
        //lcd.print("C");
        finalitzarModeAutomatic();        
      }
      else{
        comprovacionsEsmolat();  
      }
    }
  }
}

void moureMolaPosicioInicial(){
  aplicarMoveRunZ(-carregaEixZtotal, true);
  eixZcurrentPosition = eixZ.currentPosition();
}

void aplicarSegonAngleSenseMedir(){
  iniciantEsmolatAngle2 = true;
  alsarMolaSeguretat();
  eixZcurrentPosition = eixZ.currentPosition();
  posicionarDivisor();
  iniciantEsmolatAngle2 = false;
  recuperarPosicioMolaAngle(molaPosicioAngle2);
}

void referMovimentsPerSeguentEsmolat(){
  // Cal tornar a les posicions de l'angle 1:
  // Pas 1: Alçem mola per seguretat
  iniciantEsmolatAngle2 = true;
  alsarMolaSeguretat();
  eixZcurrentPosition = eixZ.currentPosition();
  // Pas 2: Mouve divisor a angle 1 altre cop
  idxArrayEsmolat = 0;
  posicionarDivisor();
  iniciantEsmolatAngle2 = false;
  idxArrayEsmolat = -1;
  // Pas 3: recuperar la posoicio de la mola per a esmolar en
  recuperarPosicioMolaAngle(molaPosicioAngle1);
}

void actualitzarVelocitatAutomatic(){
  if(carroMoventse && movimentsAutomatics == 1){
    if(filera == movimentAutoActual-1){
      if(!programaQuartEix()){ 
        idxCurrentVel = idxVar;     
        actualitzarVelocitat();
        tone(eixXstep, stepsEixXautomatic);
      }
      else{
        stepsEixYautomatic = VEL_EIXY_CONSTANT_AUTO * (vars[idxVar]-1);
        eixY.setSpeed(stepsEixYautomatic*dirEixYauto);  
      }
    }
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * PRINTS FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */


/* 
 *** PRINTS ARRENCADA DE SISTEMA ***
 */

void printBenvinguda(){
  esborrarPantalla();
  posCursor(12,0);
  printar("ELITE CU PRO");
  posCursor(5,2);
  printar("Cargando, por favor espere");
}

void printPunts(){
  posCursor(31+nPunts, 2);
  printar(".");
}

void printResultatArrencada(){
  byte row = 0;
  esborrarPantalla();
  if(errors){
    printar("Se han encontrado errores de arranque:");
    row++;
    posCursor(0,row);
    if(respostaFA[2] == 233){
      printar("Sin respuesta de Fuente de alimentacion");
    }
  }
  else{
    if(avisos){
      row = 0;
      posCursor(5,0);
      printar("ARRANQUE CORRECTO CON ALERTAS:");
      if(respostaFA[2] == 0){
        row++;
        posCursor(0,row);
        printar("-Temperatura erronea en ultimo ejercicio"); 
      }
      if(respostaRTC == 0){
        row++;
        posCursor(0,row);
        printar("-Fecha/Hora desfasada. Comprovar RTC"); 
      }
    }
    else{
      posCursor(11,0);
      printar("ARRANQUE CORRECTO"); 
    }
    posCursor(10,3);
    printar("Pulse para continuar");
  }  
}

/* 
 *** PRINTS REFERENCIAT ***
 */

void printReferenciantEixZ(){
  esborrarPantalla();
  posCursor(9,0);
  printar("Referenciando Eje Z...");  
}

void printReferenciantEixY(){
  printEixZreferenciat();
  posCursor(9,1);
  printar("Referenciando Eje Y..."); 
}

void printReferenciantEixA(){
  if(vars[idxActivarQuartEix] == 0){
    printEixZreferenciat();
  }
  else{
    printEixYreferenciat();
  }
  posCursor(9,2);
  printar("Referenciando Eje A...");
}

void printEixZreferenciat(){
  posCursor(9,0);
  printar("Eje Z referenciado    ");
}

void printEixYreferenciat(){
  posCursor(9,1);
  printar("Eje Y referenciado    ");
}

void printReferenciatFinalitzat(){
  esborrarPantalla();
  posCursor(9,1);
  printar("REFERENCIADO COMPLETO ");
  delay(2000);
}

void printErrorReferenciat(char* eix){
  esborrarPantalla();
  posCursor(7,0);
  printar("ERROR REFERENCIANDO EJE ");
  lcd.print(eix);
  posCursor(3,2);
  printar("Revise sensores finales de carrera");
  posCursor(4,3);
  printar("o contacte con servicio técnico");
}

/* 
 *** PRINTS MENU SECRET ***
 */

void printMenuSecret(){
  esborrarPantalla();
  printar("1. Mover cabezal");
  posCursor(0,1);
  printar("2. Activar medicion:");
  posCursor(0,2);
  printar("3. Incremento velocidad:");
  posCursor(0,3);
  printar("4. Tiempos: Acc:      ms   dAcc:      ms"); 
}

void printMenuSecretValors(){
  posCursor(26,1);
  printVariableBinaria(idxActivarMedicioPrint+vars[idxActivarMedicio]);
  setCoordenades(27,2);
  intEnEdicio = vars[idxIncrementVel_low] + vars[idxIncrementVel_high]*256;
  printVariableInt(idxIncrementVel_low);
  setCoordenades(20,3);
  intEnEdicio = vars[idxAcc_low] + vars[idxAcc_high]*256;
  printVariableInt(idxAcc_low);
  columna = 36;
  intEnEdicio = vars[idxDacc_low] + vars[idxDacc_high]*256;
  printVariableInt(idxDacc_low);
}

void printMenuSecret2(){
  esborrarPantalla();
  printar("5. Activar magnetismo:");
  posCursor(0,1);
  printar("6. Horas de trabajo:");
  posCursor(0,2);
  printar("7. Recorrido eje Z:");
  posCursor(0,3);
  printar("8. Cuarto eje:");
}

void printMenuSecret2Valors(){
  posCursor(25,0);
  printVariableBinaria(idxActivarMagnetismePrint+vars[idxActivarMagnetisme]);
  setCoordenades(26,1);
  intEnEdicio = vars[idxHoresTotals_low] + vars[idxHoresTotals_high]*256;
  printVariableInt(idxHoresTotals_low);
  setCoordenades(26,2);
  longEnEdicio = vars[idxRecorregutZ_low] + vars[idxRecorregutZ_high]*256L + vars[idxRecorregutZ_high2]*256L*256L;
  printVariableLong(idxRecorregutZ_low);
  posCursor(25,3);
  printVariableBinaria(idxActivarQuartEixPrint + vars[idxActivarQuartEix]);
}

void printMenuSecret3(){
  esborrarPantalla();
  printar("9.  Recorrido eje Y:");
  posCursor(0,1);
  printar("10. Velocidad eje Z:");
  posCursor(0,2);
  printar("11. Pruebas");
}

void printMenuSecret3Valors(){
  setCoordenades(26,0);
  longEnEdicio = (vars[idxRecorregutY_low] + vars[idxRecorregutY_high]*256L + vars[idxRecorregutY_high2]*256L*256L)*5;
  printVariableLong(idxRecorregutY_low);
  setCoordenades(26,1);
  intEnEdicio = vars[idxVelocitatZ_low] + vars[idxVelocitatZ_high]*256;
  printVariableInt(idxVelocitatZ_low); 
}

/* 
 *** PRINTS MENU INICIAL ***
 */
 
void printMenuInicial(){
  esborrarPantalla();
  if(pantalla == N_PANTALLES_MENUINICIAL){
    printar("a. Movimiento manual maquina");
    posCursor(0,1);
    printar("b. Valores de proceso");
    posCursor(0,2);
    printar("c. Configuracion de Programa");
  }
  else{
    byte aux = (pantalla-1)*4 + 1;
    for(byte i = 0; i < 4; i++){
      posCursor(0,i);
      printarByte(aux);
      if(aux < 10){
        printar(".  Programa ");
      }
      else{
        printar(". Programa ");  
      }
      printarByte(aux);
      aux++;
    }
  }
}

/* 
 *** PRINTS PROGRAMA D'ESMOLAT ***
 */

void printProgramaPantalla1(){
  esborrarPantalla();
  printar("DESBASTE:      ,    / 0,0   mm   V:    %");
  printar("ACABADO:      0,    / 0,0   mm   V:    %");
  printar("LAPEADO:");
  posCursor(33,2);
  printar("V:    %");
  printAccioEndavant(0);
}

void printProgramaPantalla1Valors(){
  setCoordenades(14,0);
  printVariableByte(idxDesbast);
  columna = 18;
  intEnEdicio = vars[idxDesbastDec_low] + vars[idxDesbastDec_high]*256;
  printVariableInt(idxDesbastDec_low);
  columna = 26;
  intEnEdicio = vars[idxDesbastDivDec_low] + vars[idxDesbastDivDec_high]*256;
  printVariableInt(idxDesbastDivDec_low);
  columna = 38;
  printVariableByte(idxDesbastVel);
  setCoordenades(18,1);
  intEnEdicio = vars[idxAcabatDec_low] + vars[idxAcabatDec_high]*256;
  printVariableInt(idxAcabatDec_low);
  columna = 26;
  printVariableByte(idxAcabatDivDec);
  columna = 38;
  printVariableByte(idxAcabatVel);
  setCoordenades(14,2);
  printVariableByte(idxLapejat);
  columna = 38;
  printVariableByte(idxLapejatVel);
}

void printProgramaPantalla2(){
  esborrarPantalla();
  printar("ANGULO FILO:   , ");
  printarWrite(223);
  printar(" /   , ");
  printarWrite(223);
  posCursor(0,1);
  printar("GIRO MUELA:");
  posCursor(0,2);
  printar("BAJADA:");
  printAccioEndarrere(0);
  printAccioEndavant(0);
}

void printProgramaPantalla2Valors(){
  setCoordenades(14,0);
  printVariableByte(idxAngle1);
  columna = 16;
  printVariableByte(idxAngle1Dec);
  columna = 22;
  printVariableByte(idxAngle2);
  columna = 24;
  printVariableByte(idxAngle2Dec);
  posCursor(14,1);
  printVariableBinaria(idxGirMolaPrint+vars[idxGirMola]);
  posCursor(14,2);
  printVariableBinaria(idxBaixadaPrint+vars[idxBaixada]);
}

void printProgramaPantalla2b(){
  esborrarPantalla();
  printar("N.dientes:          Paso diente:   ,  mm");
  printar("Anuglo filo:   , ");
  printarWrite(223);
  printar("  V.muela:   24000 rpm");
  printar("Recorrido afilado:     mm"); 
  printAccioEndarrere(0);
  printAccioEndavant(0);
}

void printProgramaPantalla2bValors(){
  setCoordenades(14,0);
  intEnEdicio = vars[idxNdents_low] + vars[idxNdents_high]*256;
  printVariableInt(idxNdents_low);
  if(vars[idxPasDent] == vars[idxPasDent+1] && vars[idxPasDentDec] == 0){
    vars[idxPasDentDec] = 1;
  }
  columna = 34;
  printVariableByte(idxPasDent);
  columna = 36;
  printVariableByte(idxPasDentDec);
  setCoordenades(14,1);
  printVariableByte(idxAngle3);
  columna = 16;
  printVariableByte(idxAngle3Dec);
  /*
  columna = 35;
  intEnEdicio = vars[idxMolaRpm_low] + vars[idxMolaRpm_high]*256;
  printVariableInt(idxMolaRpm_low);
  */
  setCoordenades(21,2);
  printVariableByte(idxRecAfilat);
}

void printProgramaPantalla3(){
  esborrarPantalla();
  printar("PRODUCCION:                      GUARDAR");
  if(vars[idxActivarMedicio] == 1){
    if(!programaQuartEix()){
      printar("MEDICION AUTO:");
    }
  }
  posCursor(30,1);
  printar("TRANSMITIR");
  printAccioEndarrere(0);
  printAccioEndavant(1);
}

void printProgramaPantalla3Valors(){
  posCursor(15,0);
  printVariableBinaria(idxProduccioPrint+vars[idxProduccio]);
  if(vars[idxActivarMedicio] == 1){
    if(!programaQuartEix()){
      posCursor(15,1);
      printVariableBinaria(idxMedicioPrint+vars[idxMedicio]);
    }
  }
}

void printGuardarPrograma(){
  esborrarPantalla();
  posCursor(13,1);
  printar("DATOS GUARDADOS");  
}

/* 
 *** PRINTS VALORS DE PROCES ***
 */

void printValorsProces(){
  esborrarPantalla();
  printar("Velocidad manual carro:");
  posCursor(38,0);
  printar("% ");
  printar("Avance manual cabezal:");
  posCursor(32,1);
  printar("0,    mm");
  if(vars[idxActivarQuartEix] == 1){
    printar("Avance manual eje Y:");
    posCursor(32,2);
    printar("0,    mm");
  }
  printAccioEndavant(0);
}

void printValorsProcesValors(){
  setCoordenades(36,0);
  printVariableByte(idxVelManualX);
  setCoordenades(36,1);
  printVariableByte(idxStepsManualZ);
  if(vars[idxActivarQuartEix] == 1){
    vars[idxStepsManualY] *= 5;
    setCoordenades(36,2);
    printVariableByte(idxStepsManualY);
    vars[idxStepsManualY] /= 5; 
  }
}

void printValorsProces2(){
  esborrarPantalla();
  printar("Offset referencia plato:");
  posCursor(35,0);
  printar(",  ");
  printarWrite(223);
  printar(" Sensibilidad medicion auto:");
  posCursor(0,2);
  printar("Compensacion medicion auto:     0,    mm");
  printAccioEndarrere(0);
  printAccioEndavant(0);
}

void printValorsProces2Valors(){
  setCoordenades(34,0);
  printVariableSigned(idxOffsetA);
  columna = 36;
  printVariableByte(idxOffsetAdec);
  setCoordenades(36,1);
  intEnEdicio = vars[idxSensibilitat_low] + vars[idxSensibilitat_high]*256;
  printVariableInt(idxSensibilitat_low);
  setCoordenades(36,2);
  intEnEdicio = vars[idxCompensacio_low] + vars[idxCompensacio_high]*256;
  printVariableInt(idxCompensacio_low);
}

void printValorsProces3(){
  esborrarPantalla();
  printar("Refrigeracion modo manual:");
  posCursor(0,1);
  printar("Nr serie software:          000000000 OK");
  printAccioEndarrere(0);
  printAccioEndavant(1);
}

void printValorsProces3Valors(){
  posCursor(28,0);
  printVariableBinaria(idxRefrigeracioPrint + vars[idxRefrigeracio]);
}

/* 
 *** PRINTS ELIMINAT DE PROGRAMES, ESBORRAR UN I TOTS ***
 */

void printEliminatProgrames(){
  esborrarPantalla();
  printar("1. Borrar programa");
  posCursor(0,1);
  printar("2. Borrar todos");
  printAccioEndarrere(2);
}

void printEliminarUnPrograma(){
  esborrarPantalla();
  posCursor(7,1);
  printar("Escoja programa a borrar");
  posCursor(9,3);
  printar("Aceptar     Cancelar");
}

void printEliminarUnProgramaValors(){
  posCursor(18,2);
  printValor2(idxPrograma, vars[idxPrograma]+1);
}

void printProgramaEliminat(){
  esborrarPantalla();
  posCursor(10,1);
  printar("Programa Borrado");
}

void printEliminarTotsProgrames(){
  esborrarPantalla();
  posCursor(5,1);
  printar("Reiniciar todos los programas?");
  posCursor(10,2);
  printar("Aceptar    Cancelar");
}

void printProgramesEliminats(){
  esborrarPantalla();
  posCursor(10,1);
  if(columna == 10){
    printar("Programas Borrados");
  }
  if(columna == 21){
    printar("Borrado cancelado");
  }
}

/* 
 *** PRINTS MOVIMENTS MANUALS, PREVI AUTOMATIC ***
 */

void printManualsIprevis(){
  esborrarPantalla();
  posCursor(0,3);
  printar("IZQUIERDA");
  posCursor(33,3);
  printar("DERECHA");
  if(configNivell == 10){
    printar("    Situe la muela sobre la cuchilla");
    if(!programaQuartEix()){
      posCursor(16,3);
      printar("continuar");
    }
    else{
      posCursor(11,3);
      printar("EJE: Z Y   Continuar");
    }
  }
}

void printPosMolaEixYStatic(){
  posCursor(18,2);
  printar(",    mm");
}

void printEixStaticPrevis(char* eix){
  posCursor(13,2);
  printar(eix);
}

void printBorrarMolaEixY(){
  posCursor(13,2);
  printar("            ");
}

void printEixZcurrentPosition(){
  printCurrentPosition(eixZcurrentPosition);
}

void printEixYcurrentPosition(){
  printCurrentPosition((vars[idxRecorregutY_low] + vars[idxRecorregutY_high]*256L + vars[idxRecorregutY_high2]*256L*256L)*5 - eixYcurrentPosition*5);
}

void printCurrentPosition(long currentPos){
  byte nXifres = 6;
  byte limitZeroEsquerre = 2;
  posCursor(15,2);
  printEnterComAdecimal(currentPos, nXifres, limitZeroEsquerre);
}

void printEixAcurrentPosition(){
  posCursor(17,2);
  printValor(grauDivisor, 2, 0);
  posCursor(20,2);
  printarByte(grauDivisorDecimal);
}

void printSalir(){
  if(!programaQuartEix()){
    posCursor(16,3);
    printar("  Salir  ");
  }
  else{
    posCursor(22,3);
    printar("Salir    ");
  }
}

void printSalir2(){
  posCursor(7,0);
  printar("Pulse 1 segundo para salir");  
}

void printPlato(){
  posCursor(16,2);
  printar("   , ");
  printarWrite(223);
  printar("   ");
  posCursor(17,3);
  printar("PLATO");
}

void printMuela(){
  posCursor(17,2);
  printar("     ");
  posCursor(17,3);
  printar("MUELA");
}

void printEixY(){
  posCursor(17,2);
  printar("     ");
  posCursor(17,3);
  printar("EJE Y");
}

void printPosicionantUnEix(char* eix){
  esborrarPantalla();
  posCursor(9,1);
  printar("Posicionando eje ");
  printar(eix);
  printar("..."); 
}

/* 
 *** PRINTS ESMOLAT AUTOMATIC ***
 */

void printAutomatic(){
  esborrarPantalla();
  printar("DESBASTE:  0,000 de   ,     V:   %      ");
  printar("ACABADO:   0,000 de  0,     V:   % NrC  ");
  printar("LAPEADO:       0 de         V:   %      ");
  if(programaQuartEix()){
    posCursor(35,0);
    printar("D");  
  }
}

void printAutomaticValors(){
  setCoordenades(21,0);
  printVariableByte(idxDesbast);
  columna = 25;
  intEnEdicio = vars[idxDesbastDec_low] + vars[idxDesbastDec_high]*256;
  printVariableInt(idxDesbastDec_low);
  columna = 32;
  printVariableByte(idxDesbastVel);
  setCoordenades(25,1);
  intEnEdicio = vars[idxAcabatDec_low] + vars[idxAcabatDec_high]*256;
  printVariableInt(idxAcabatDec_low);
  columna = 32;
  printVariableByte(idxAcabatVel);
  //columna = 39;
  printNcuchilla();
  setCoordenades(21,2);
  printVariableByte(idxLapejat);
  columna = 32;
  printVariableByte(idxLapejatVel);
  posCursor(18,3);
  printar("0' 00''");
  if(programaQuartEix()){
    printDentsRestants();  
  }
}

void printDesAcaCompleto(){
  printDigitsDesAca(carregaEixZetapa, movimentAutoActual-1);
}

void printDesUltimCompleto(){
  printDigitsDesAca(ultimValorDesbaste, 0);
}

void printAcaUltimCompleto(){
  printDigitsDesAca(ultimValorAcabado, 1);
}

void printDigitsDesAca(int valor, byte c){
  int completoAux;
  byte x;
  posCursor(10,c);
  if(valor >= 10000){
    x = extreureDigit(5, valor);
    completoAux = valor - 10000*x;
    printarByte(x);
  }
  else if(valor >= 1000){
    x = extreureDigit(4, valor);
    completoAux = valor - 1000*x;
    printar(" ");
    printarByte(x);
  }
  else{
    completoAux = valor;
    printar(" 0");
  }
  posCursor(13,c);
  printValor(completoAux, 3, 1);
}

void printLapeadoCompleto(){
  posCursor(14,2);
  printValor(count, 2, 0);
}

void printNcuchilla(){
  if(nCuchilla < 100){
    posCursor(38,1);
    printValor(nCuchilla, 2, 0);
  }
}

void printDentsRestants(){
  if(dentActual < 3000){
    posCursor(36,0);
    printValor(nDents-dentActual, 4, 0);  
  }
}

void printOpcionsBotonsAutomatic(byte op){
  posCursor(0,3);
  switch(op){
    case 2:
      printar("STOP ");
    break;
    case 3:
      printar("     ");
    break;
    default:
      printar("SALIR");
    break; 
  }
  posCursor(32,3);
  switch(op){
    case 0:
      printar(" INICIAR");
    break;
    case 1:
      printar("REANUDAR");
    break;
    case 2:
      printar("  PAUSAR");
    break;
    case 3:
      printar("        ");
    break; 
  }
}

void printSegonsCicle(){
  posCursor(21,3);
  printValor(segonsCicle, 2, 1);
}

void printMinutsCicle(){
  posCursor(15,3);
  printValor(minutsCicle, 4, 0);
}

void printVelocitatNova(){
  byte fileraAux = filera;
  filera = movimentAutoActual-1;
  idxVar = idxCurrentVel;
  printVariableByte(idxCurrentVel);
  filera = fileraAux;
}

void printPlatEnEsmolatPlat(){
  posCursor(34,2);
  printar(" PLATO");
}

void printPlatEnEsmolatA0(){
  posCursor(34,2);
  printar(" a 0");
  printarWrite(223);
  printar(" ");
}

void printPlatEnEsmolatApos(){
  posCursor(34,2);
  printar("a  ,");
  printEixAposEsmolat();
  printarWrite(223);
}

void printEixAposEsmolat(){
  posCursor(35,2);
  printValor(grauDivisorAux, 2, 0);
  posCursor(38,2);
  printarByte(grauDivisorDecimalAux);  
}

/* 
 *** PRINTS CONFIRMAR PASSOS ***
 */

void printConfirmarUnPas(){
  esborrarPantalla();
  posCursor(15,0);
  printar("ATENCION!");
  if(!previReferenciar){
    posCursor(5,1);
    printar("SE PROCEDERA A GIRAR LA MUELA");
  }
  else{
    posCursor(4,1);
    printar("SE REFERENCIARAN MUELA Y PLATO");  
  }
  posCursor(15,2);
  printar("continuar?");
  posCursor(15,3);
  printar("SI  /  NO");
}

void printPantallaAvisRecanvis(){
  esborrarPantalla();
  posCursor(9,0);
  printar("INTERVALO DE SERVICIO");
  posCursor(15,1);
  printar("Por favor,");
  posCursor(3,2);
  printar("cambie los cartuchos engrasadores    ");
  printar("Ok, esta hecho         Lo hare mas tarde");
}

/* 
 *** PRINTS MODE PROVES ***
 */

void printProves(){
  esborrarPantalla();
  posCursor(0,1);
  printar("Posicion Encoder:");
}

/* 
 *** PRINTS MEDICIO AUTOMATICA ***
 */

void printAjustarMolaAutomatic(){
  esborrarPantalla();
  posCursor(7,0);
  printar("Ajuste de muela en proceso");
  posCursor(10,1);
  printar("Por favor, espere...");
  printCancelarMedicio(0);
}

void printCancelarMedicio(byte op){
  posCursor(0,3);
  if(op == 0){
    printar("CANCELAR");
  }
  else{
    printar("        ");
  }
}

void printDesplasarCarro(){
  posCursor(7,0);
  printar("    Posicionando Carro    ");
  posCursor(0,3);
  printar("        ");
}

/* 
 *** PRINTS MAGNETISME ***
 */

void printMagDesmag(byte op){
  posCursor(12,3);
  if(op == 0){
    printar(" MAGNETITZAR  ");
  }
  else{
    printar("DESMAGNETITZAR");
  }
}

void printErrorPlat(byte op){
  esborrarPantalla();
  posCursor(12,1);
  printar("PLATO NO IMANTADO");
  posCursor(0,2);
  if(op == 0){
    printar("Imante el plato antes de ajustar maquina");
  }
  else{
    printar("Imante el plato antes de iniciar ciclo");
  }
}

/*
 * PRINTS GLOBALS
 */

void printVariableByte(int idx){
  posCursor(columna-vars[idx+3]+1, filera);
  if(vars[idx+3] == 1){
    if(configNivell == 7){
      printarByte(vars[idx]+1);
    }
    else{
      printarByte(vars[idx]);  
    }  
  }
  else{
    if(configNivell == 7){
      printValor2(idx, vars[idx]+1);
    }
    else{
      printValor2(idx, vars[idx]); 
    }   
  }
}

void printVariableSigned(int idx){
  posCursor(columna-vars[idx+3], filera);
  if(int8_t(vars[idx]) >= 0){
    printar(" ");  
  }
  if(vars[idx+3] == 1){
    printarInt8(int8_t(vars[idx])); 
  }
  else{
    printValorSigned(idx);  
  }
}

void printVariableBinaria(int idx){
  lcd.print(varsString[idx]);
}

void printVariableInt(int idx){
  posCursor(columna-vars[idx+3]+1, filera);
  printValor2(idx, intEnEdicio);
}

void printVariableLong(int idx){
  posCursor(columna-vars[idx+3]+1, filera);
  printValor2Long(idx, longEnEdicio);  
}

void printAccioEndarrere(byte op){
  posCursor(0,3);
  printarWrite(127);
  switch(op){
    case 0:
      printar("Atras");
    break;
    case 1:
      printar("Cancelar");
    break;
    case 2:
      printar("Volver a menu principal");
    break;
  }
}

void printAccioEndavant(byte op){
  switch(op){
    case 0:
      posCursor(30,3);
      printar("Siguiente");
    break;
    case 1:
      posCursor(34,3);
      printar("Salir");
    break; 
  }
  printarWrite(126);
}

void printValor2(int idx, int v){
  int midaValor;
  for(byte i = vars[idx+3]; i > 1; i--){
    midaValor = int(pow(10,i-1));
    if(midaValor%10 != 0){
      midaValor++;
    }
    if(v < midaValor){
      if(vars[idx+4] == 0){
        printar(" ");
      }
      else{
        printar("0");
      }
    }
  }
  printarInt(v);
}

void printValor2Long(int idx, long v){
  long midaValor;
  for(byte i = vars[idx+3]; i > 1; i--){
    midaValor = long(pow(10,i-1));
    if(midaValor%10 != 0){
      midaValor++;
    }
    if(v < midaValor){
      if(vars[idx+4] == 0){
        printar(" ");
      }
      else{
        printar("0");
      }
    }
  }
  printarLong(v);
}

void printValor(unsigned long valor, byte maxim, byte tipus){
  int midaValor;
  for(byte i = maxim; i > 1; i--){
    midaValor = int(pow(10,i-1));
    if(midaValor%10 != 0){
      midaValor++;  
    }
    if(valor < midaValor){
      if(tipus == 0){
        printar(" ");
      }
      else{
        printar("0");
      }
    }
  }
  lcd.print(valor);
}

void printValorSigned(int idx){
  int midaValor;
  for(byte i = vars[idx+3]; i > 1; i--){
    midaValor = int(pow(10,i-1));
    if(midaValor%10 != 0){
      midaValor++;
    }
    if(abs(int8_t(vars[idx])) < midaValor){
      printar(" "); 
    }
  }
  printarInt8(int8_t(vars[idx]));
}

void printEnterComAdecimal(long currentPos, byte xifres, byte limitZeroEsq){
  boolean esZeroEsquerre = true;
  byte digit;
  long div = long(pow(10,xifres-1));
  if(div%10 != 0){ // Si no es modul10 = 0, es que la funcio pow ha generat un error tipus 9, 99, 999 .. en comptes de 10, 100, 1000
    div++;  
  }
  for(byte i=0;i<6;i++){
    digit = (currentPos/div)%10;
    if(digit > 9){ // A vegades dona un error de 255
      digit = 0;  
    }
    div /= 10;
    if(esZeroEsquerre){
      if(digit == 0){
        if(i < limitZeroEsq){
          printar(" ");
        }
        else{
          printar("0");
        }
      }
      else{
        esZeroEsquerre = false;
        printarByte(digit);
      }
    }
    else{
      printarByte(digit);
    }
    
    if(i == limitZeroEsq){
      posCursor(19,2);
    }
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * CALCULS ANGLES
 ************************************************************************************
 ************************************************************************************
 */


long establirStepsAbsolutsIrelatiusAngleActual(){
  byte angleEnter;
  byte angleDecimal;
  int angleTotal;
  unsigned long angleStepsAbsoluts;
  if(!programaQuartEix()){
    if(idxArrayEsmolat < 3){
      angleEnter = vars[idxAngle1];
      angleDecimal = vars[idxAngle1Dec];
    }
    else{
      angleEnter = vars[idxAngle2];
      angleDecimal = vars[idxAngle2Dec];
    }
  }
  else{
    angleEnter = vars[idxAngle3];
    angleDecimal = vars[idxAngle3Dec];
  }
  angleTotal = angleEnter*10 + angleDecimal;
  editarGrausDivisor2(angleEnter, angleDecimal, angleTotal);
  unsigned int stepsArecorrer = (angleTotal-angleEnter)*56 + angleEnter*52;
  angleStepsAbsoluts = RECORREGUT_A - stepsArecorrer;
  // Steps relatius
  if(angleStepsAbsoluts < eixAcurrentPosition){
    return -(eixAcurrentPosition - angleStepsAbsoluts);
  }
  else{
    return angleStepsAbsoluts - eixAcurrentPosition;
  }
}

void editarGrausDivisor(){
  if(sumantGrausDivisor){
    if(grauDivisor < 90){
      if(grauDivisorDecimal < 9){
        grauDivisorDecimal++;  
      }
      else{
        grauDivisorDecimal = 0;
        grauDivisor++;
      }
    }
  }
  else{
    if(grauDivisorDecimal > 0){
      grauDivisorDecimal--; 
    }
    else{
      if(grauDivisor > 0){
        grauDivisorDecimal = 9;
        grauDivisor--;
      }
    }
  }
}

void editarGrausDivisor2(byte angleEnter, byte angleDecimal, int angleTotal){
  int grauDivisorTotal = grauDivisor*10 + grauDivisorDecimal;
  int dif;
  if(angleTotal > grauDivisorTotal){
    sumantGrausDivisor = true;
    dif = angleTotal - grauDivisorTotal;
    aplicarDifAngleAgrausDivisor(dif);
  }
  else if(angleTotal < grauDivisorTotal){
    sumantGrausDivisor = false;
    dif = grauDivisorTotal - angleTotal;
    aplicarDifAngleAgrausDivisor(dif);
  }
}

void aplicarDifAngleAgrausDivisor(int dif){
  for(int i=0; i<dif; i++){
    editarGrausDivisor();
  }
}


/*
 ************************************************************************************
 ************************************************************************************
 * CALCULS CONSISTENCIA VARIABLES
 ************************************************************************************
 ************************************************************************************
 */


void consistenciaValorByte(){
  vars[idxVar] = getVariable(); 
  if(vars[idxVar] < vars[idxVar+1] || vars[idxVar] > vars[idxVar+2]){
    // Menor al minim i major al maxim. Valor inconsistent
    vars[idxVar] = vars[idxVar+1]; // = al valor minim
    setVariable();
  }
}

void consistenciaValorUint8(){
  vars[idxVar] = getVariable();
  if(vars[idxVar] < vars[idxVar+1] && vars[idxVar] > vars[idxVar+2]){
    // Valros no pertanyents a -90 i +90 en unsigned (<165 i >90). Valor inconsistent
    vars[idxVar] = 0;
    setVariable();
  }
}

void consistenciaValorInt(){
  vars[idxVar] = getVariable();
  intEnEdicio_limitLow = vars[idxVar+1] + vars[idxVar+2]*256; // Int real limit baix
  
  idxVar += PROPS_VAR; 
  vars[idxVar] = getVariable();
  intEnEdicio_limitHigh = vars[idxVar+1] + vars[idxVar+2]*256; // Int real limit alt
  
  idxVar -= PROPS_VAR;  
  intEnEdicio = vars[idxVar] + vars[idxVar+PROPS_VAR]*256; // Int real valor

  if(intEnEdicio < intEnEdicio_limitLow || intEnEdicio > intEnEdicio_limitHigh){
    guardarVariable(idxVar, vars[idxVar+1], true, 0, false, 0);
    vars[idxVar] = vars[idxVar+1];
    vars[idxVar+PROPS_VAR] = 0;
  }
}

void consistenciaValorLong(){
  vars[idxVar] = getVariable();
  intEnEdicio_limitLow = vars[idxVar+1] + vars[idxVar+2]*256; // Int real limit baix
  
  idxVar += PROPS_VAR;  
  vars[idxVar] = getVariable();
  
  idxVar += PROPS_VAR;
  vars[idxVar] = getVariable();
  
  idxVar -= PROPS_VAR*2;
  // Aixo fa el 983040 + 16896 + 1 = Quasi 1.000.000
  longEnEdicio_limitHigh = intEnEdicio_limitLow + (vars[idxVar+PROPS_VAR+1] + vars[idxVar+PROPS_VAR+2]*256L)*256L + (vars[idxVar+PROPS_VAR*2+1] + vars[idxVar+PROPS_VAR*2+2]*256L)*256L*256L;
  longEnEdicio = vars[idxVar] + vars[idxVar+PROPS_VAR]*256L + vars[idxVar+PROPS_VAR*2]*256L*256L;
  
  // Cas uLongs, el limit_high no el mirem com a valor total.
  // Agafem el tercer bit (el mes alt) i el comparem amb el valor maxim que pot tenir
  if(longEnEdicio < intEnEdicio_limitLow || longEnEdicio > longEnEdicio_limitHigh){
    guardarVariable(idxVar, vars[idxVar+1], true, 0, true, 0);
  }
}

void checkConsistenciaDecimal(byte offsetCol, byte signe, byte tipusComprovacio){
  // tipusComprovacio: 0 = baix, 1 = alt
  byte offsetIdxVar;
  boolean excepcio = false;
  if(offsetCol > 0){
    // Estem a la part sencera.
    // Per printar decimal, Hem d'anar al index de la variable decimal: +PROPS_VAR
    offsetIdxVar = PROPS_VAR; 
  }
  else{
    // Estem a la part decimal.
    // Per printar decimal, ens mantenim en l'index acutal
    offsetIdxVar = 0;  
  }
  switch(tipusComprovacio){
    case 2:
      excepcio = consistenciaDecimal(0, offsetIdxVar, signe) || consistenciaDecimal(1, offsetIdxVar, signe);
    break;
    default:
      excepcio = consistenciaDecimal(tipusComprovacio, offsetIdxVar, signe);
    break;
  }
  if(excepcio){
    // Decimal major a 0 quan no pot estar-hi. Posem decimal de la variable a 0
    idxVar += offsetIdxVar;
    vars[idxVar] = vars[idxVar + 1];
    if(signe == 1){
      // Decimal rectificat es int. Rectifiquem tambe el segon byte
      vars[idxVar+PROPS_VAR] = 0;
      if(offsetCol == 0){
        // A mes el decimal int es la variable actual en edicio. Actualitzem intEnEdicio
        intEnEdicio = vars[idxVar] + vars[idxVar+PROPS_VAR]*256;
      }
    }
    colAux = columna;
    columna += offsetCol;
    // Amb +offsetIdxVar indiquem l'index de la seguent variable
    // Aixo es, la part decimal que es la que volem printar
    printVariableByte(idxVar);
    columna = colAux;
    idxVar -= offsetIdxVar;
  }
}

boolean consistenciaDecimal(byte tipusLimit, byte offsetPrintDec, byte tipusValor){
  // Per casos on es vol comprovar: Valor enter = max/min i Valor decimal > 0
  // tipusLimit: propietat MINIM (0), propietat MAXIM (1) DEL VALOR ENTER (no del decimal)
  // enter: true = sencera actual, false = decimal actual
  // tipusValor: 0 = valor int8_t amb signe. 1 = valor decimal int16
  if(offsetPrintDec != 0){
    // Estem a variable sencera. Offset per al idx decimal
    offsetV = 0;
    offsetVdec = PROPS_VAR;
  }
  else{
    // Estem a variable decimal. Offset per al idx sencer
    offsetV = -PROPS_VAR;
    offsetVdec = 0;
  }
  switch(tipusValor){
    case 0: // valor int8_t amb signe
      valor = int8_t(vars[idxVar + offsetV]);
      valorLim = int8_t(vars[idxVar + offsetV + 1 + tipusLimit]);
      valorDec = vars[idxVar + offsetVdec];
      //return valorEnter == valorEnterLimit && vars[idxVar + offsetDecimal] > vars[idxVar + offsetDecimal + 1];
    break;
    case 1: // valor decimal int16
      valor = vars[idxVar + offsetV];
      valorLim = vars[idxVar + offsetV + 1 + tipusLimit];
      valorDec = vars[idxVar + offsetVdec] + vars[idxVar + offsetVdec + PROPS_VAR]*256;
    break;
    case 2:
      valor = vars[idxVar + offsetV];
      valorLim = vars[idxVar + offsetV + 1 + tipusLimit];
      valorDec = vars[idxVar + offsetVdec];
    break; 
  }
  valorDecLim = vars[idxVar + offsetVdec + 1]; // Limit LOW
  return valor == valorLim && valorDec > valorDecLim;
}

void consistenciaDivisorTotal(byte pos, boolean valorDecEs16b, boolean divDecEs16b){
  byte offsetCol;
  // Obtenim les posicions a l'array vars[] de les tres variables a gestionar (valor, valorDec, divisorDec)
  switch(pos){
    case 0: // Cursor a enter total
      offsetV = 0;
      offsetVdec = PROPS_VAR;
      if(valorDecEs16b){
        offsetDivDec = PROPS_VAR * 3;
      }
      else{
        offsetDivDec = PROPS_VAR * 2;  
      }
      offsetCol = 12; // Offset de nova posicio columna on printar el divDec, si cal
    break;
    case 1: // Cursor a decimal total
      offsetV = -PROPS_VAR;
      offsetVdec = 0;
      if(valorDecEs16b){
        offsetDivDec = PROPS_VAR * 2;
      }
      else{
        offsetDivDec = PROPS_VAR;  
      }
      offsetCol = 8; // Offset de nova posicio columna on printar el divDec, si cal
    break;
    case 2: // Cursor a decimal divisor
      if(valorDecEs16b){
        offsetV = -PROPS_VAR * 3;
        offsetVdec = -PROPS_VAR * 2;
      }
      else{
        offsetV = -PROPS_VAR * 2;
        offsetVdec = -PROPS_VAR * 2;
      }
      offsetDivDec = 0;
      offsetCol = 0; // Offset de nova posicio columna on printar el divDec, si cal
    break;  
  }
  // Obtenim els valors dels decimals, per si son de 1byte o de 2 bytes
  if(valorDecEs16b){
    if(pos == 1){
      valorDec = intEnEdicio;
    }
    else{
      valorDec = vars[idxVar+offsetVdec] + vars[idxVar+offsetVdec+PROPS_VAR]*256;
    }
  }
  else{
    valorDec = vars[idxVar+offsetVdec];
  }
  if(divDecEs16b){
    if(pos == 2){
      // Cal guardar el valor actual del int que representa el divisorDecimal, en 
      //guardarIntEnEdicioAarray();
      divDec = intEnEdicio;
    }
    else{
      divDec = vars[idxVar+offsetDivDec] + vars[idxVar+offsetDivDec+PROPS_VAR]*256;
    }
  }
  else{
    divDec = vars[idxVar+offsetDivDec];    
  }
  // Obtinguts els valors correctes, els comparem i rectifiquem divisorDecimal si cal
  if(vars[idxVar+offsetV] == 0){
    if(valorDec < divDec){
      if(valorDec == 0){
        divDec = 1;
      }
      else{
        divDec = valorDec;
      }
      // Modifiquem la columna temporalment per printar a la columna correcte
      colAux = columna;
      columna += offsetCol;
      if(divDecEs16b){
        if(pos == 2){
          intEnEdicio = divDec;  
        }
        intAbytes(divDec);
        vars[idxVar+offsetDivDec] = myBytes[0]; // divisorDecimal part baixa
        vars[idxVar+offsetDivDec+PROPS_VAR] = myBytes[1]; // divisorDecimal part baixa
        intAux = intEnEdicio;
        intEnEdicio = divDec;
        printVariableInt(idxVar+offsetDivDec);
        intEnEdicio = intAux;
      }
      else{
        vars[idxVar+offsetDivDec] = divDec;
        printVariableByte(idxVar+offsetDivDec);
      }
      // Recuperem columna de navegacio actual
      columna = colAux;
    }
  }
}

boolean esIntDePrograma(){
  return (idxVar == idxDesbastDec_low || idxVar == idxDesbastDivDec_low || idxVar == idxAcabatDec_low || idxVar == idxNdents_low || idxVar == idxMolaRpm_low);
}

void reiniciarValorsMedicio(){
  for(byte i = 0; i < nProgs; i++){
    pos = i * N_VARS_PROGRAMA; // pos + idxVar/PROPS_VAR
    idxVar = idxMedicio;
    vars[idxVar] = 0;
    setVariable();
  }  
}


/*
 ************************************************************************************
 ************************************************************************************
 * CALCULS CONVERSIO TIPUS I MANIPULACIO
 ************************************************************************************
 ************************************************************************************
 */
 

void intAbytes(int valor){
  myBytes[0] = valor & 0xff;	
  myBytes[1] = (valor >> 8) & 0xff;
}

void longAbytes(long valor){
  myBytes[0] = valor & 0xff;		
  myBytes[1] = (valor >> 8) & 0xff;
  myBytes[2] = (valor >> 16) & 0xff;
  myBytes[3] = (valor >> 24) & 0xff;
}

void uLongAbytes(unsigned long valor){
  myBytes[0] = valor & 0xff;		
  myBytes[1] = (valor >> 8) & 0xff;
  myBytes[2] = (valor >> 16) & 0xff;
  myBytes[3] = (valor >> 24) & 0xff;
}

byte extreureDigit(byte digitsTotals, int valor){
  unsigned long m = pow(10,digitsTotals);
  return (valor - valor/m*m)/(m/10);
}


/*
 ************************************************************************************
 ************************************************************************************
 * CALCULS ALTRES
 ************************************************************************************
 ************************************************************************************
 */


/*
void calcularAnalogRpm(){
  float mil = 1000;
  int augment = 106;
  float div = (vars[idxMolaRpm_low] + vars[idxMolaRpm_high]*256) / mil;
  analogRpm = (div * augment) / 10;
}
*/


/*
 ************************************************************************************
 ************************************************************************************
 * INTERRUPCIONS FUNCIONS
 ************************************************************************************
 ************************************************************************************
 */
 

void actualitzarEncoderEsquerra(){
  valorEncoderNew -= 1; 
}

void actualitzarEncoderDreta(){
  valorEncoderNew += 1;
}
